#include <QtSql>
#include "akkredlist.h"
#include "utils.h"

AkkredListe::AkkredListe(QWidget *parent) : QDialog(parent)
{
  QLabel      *l;
  int         z;
  QGridLayout *layout = new QGridLayout;

  setWindowTitle("Akkreditierungsliste");

  wGliederung = new WGliederung(this);
  wDatum      = new QDateEdit(this);
  wUnbez      = new QComboBox(this);
  wSort       = new QComboBox(this);
  wDatei      = new CSVDatei(false,false,this);
  bExport     = new QPushButton("&Export",this);
  bCancel     = new QPushButton("&Abbruch",this);

  wGliederung->setText(UtilSelbst());

  wDatum->setDisplayFormat("dd.MM.yyyy");
  wDatum->setFixedSize(wDatum->sizeHint());
  wDatum->setDate(QDate::currentDate());

  wUnbez->insertItem(0,"Alle Mitglieder");
  wUnbez->insertItem(1,"Nur gezahlte Mitglieder");
  wUnbez->insertItem(2,"Nur nicht gezahlte Mitglieder");
  wUnbez->setFixedSize(wUnbez->sizeHint());

  wSort->insertItem(0,"Mitgliedsnummer");
  wSort->insertItem(1,"Nachname");
  wSort->insertItem(2,"Postleitzahl");
  wSort->insertItem(3,"Gliederung/Mitgliedsnummer");
  wSort->insertItem(4,"Gliederung/Nachname");
  wSort->insertItem(5,"Gliederung/Postleitzahl");
  wSort->setFixedSize(wSort->sizeHint());

  bExport->setAutoDefault(false);
  bCancel->setAutoDefault(false);

  z=1;
  l=new QLabel("Gliederung");
  layout->addWidget(l,z,1);
  layout->addWidget(wGliederung,z,2,1,6);
  z++;

  l=new QLabel("Bezahlt bis");
  layout->addWidget(l,z,1);
  layout->addWidget(wDatum,z,2);
  z++;

  l=new QLabel("Zahlstatus");
  layout->addWidget(l,z,1);
  layout->addWidget(wUnbez,z,2,1,2);
  z++;

  l=new QLabel("Sortierung");
  layout->addWidget(l,z,1);
  layout->addWidget(wSort,z,2,1,2);
  z++;

  z=wDatei->FuellLayout(layout,z,1);
  layout->addWidget(bExport,z,1);
  layout->addWidget(bCancel,z,2);

  setLayout(layout);

  connect(bExport,SIGNAL(clicked()),this,SLOT(pExport()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(reject()));

}

AkkredListe::~AkkredListe()
{
  delete wGliederung;
  delete wDatum;
  delete wUnbez;
  delete wSort;
  delete wDatei;
  delete bExport;
  delete bCancel;
  return;
}

void AkkredListe::pFuellListe()
{
  QStringList wGlList;
  QSqlQuery   query,query2,query3;
  int         iUnbez,iSort,iMid;
  bool        flag_bez;
  QDate       dBezDatum;
  QString     sDatum,sBezDatum;
  QString     sBezTest,sMid,sSort,sZahlart;
  int         i;

  iUnbez=wUnbez->currentIndex();
  iSort=wSort->currentIndex();
  sDatum=wDatum->date().toString("yyyy-MM-dd");
  dBezDatum=wDatum->date().addMonths(-3);
  sBezDatum=dBezDatum.toString("yyyy-MM-dd");
  switch(iUnbez)
  {
     case 1 : sBezTest=" AND bezahlt>='" + sBezDatum + "'"; break;
     case 2 : sBezTest=" AND bezahlt <'" + sBezDatum + "'"; break;
     default: sBezTest="";
  }

  wGlList.append(wGliederung->text());
  for (i=0; i<wGlList.size(); i++)
  {
    query.exec("SELECT namekurz FROM verband WHERE teilvon='" +
              wGlList[i] + "'");
    while(query.next())
    {
      if (!wGlList.contains(query.value(0).toString()))
         wGlList.append(query.value(0).toString());
    }
    query.finish();
  }

  query.exec("CREATE TEMPORARY TABLE zwiakkred (sort1 text, sort2 text, mid int, name1 text, name2 text, strasse text, plz text, ort text, land text, gliederung text, wohngliederung text, bezahlt date, beitrag int64, zahlart char, PRIMARY KEY (sort1 ASC, sort2 ASC, mid ASC))");

  for (i=0; i<wGlList.size(); i++)
  {
    query.exec("SELECT mid, austritt, bezahlt, beitrag, zahlart, wohngliederung FROM mitglied WHERE gliederung='" +
               wGlList[i] + "' AND austritt>'" + sDatum +
               "' AND mid<'1000000'" + sBezTest);
    while(query.next())
    {
      if (query.value(2).toDate() >= dBezDatum)
         sZahlart="X";
      else
         sZahlart=query.value(4).toString();
      iMid=query.value(0).toInt();
      sMid=sMid.sprintf("%08i",iMid);
      query2.exec("SELECT name1,name2,strasse,plz,ort,land FROM adresse WHERE art='M' AND key='" +
                  sMid + "' ORDER BY art,key");
      if (query2.next())
      {
        switch(iSort)
        {
          case 0 : sSort="'X','X'"; break;
          case 1 : sSort="'" + query2.value(0).toString() + "," +
                         query2.value(1).toString() + "','X'";
                   break;
          case 2 : sSort="'" + query2.value(3).toString() + "','" +
                          query2.value(0).toString() + "," +
                          query2.value(1).toString() + "'";
                   break;
          case 3 : sSort="'" + wGlList[i] + "','X'"; break;
          case 4 : sSort="'" + wGlList[i] + "','" +
                         query2.value(0).toString() + "," +
                         query2.value(1).toString() + "'";
                   break;
          case 5 : sSort="'" + wGlList[i] + "','" +
                         query2.value(3).toString() + "," +
                         query2.value(0).toString() + "," +
                         query2.value(1).toString() + "'";
                   break;
          default: sSort="'X','X'"; break;
        }
        query3.exec("INSERT INTO zwiakkred (sort1,sort2,mid,name1,name2,strasse,plz,ort,land,gliederung,wohngliederung,bezahlt,beitrag,zahlart) VALUES (" +
                     sSort + ",'" + query.value(0).toString() + "','" +
                     query2.value(0).toString() + "','" +
                     query2.value(1).toString() + "','" +
                     query2.value(2).toString() + "','" +
                     query2.value(3).toString() + "','" +
                     query2.value(4).toString() + "','" +
                     query2.value(5).toString() + "','" +
                     wGlList[i]                 + "','" +
                     query.value(5).toString()  + "','" +
                     query.value(2).toString()  + "','" +
                     query.value(3).toString()  + "','" +
                     sZahlart                   + "')");
        query2.finish();
      }
    }
    query.finish();
  }
  return;
}

void AkkredListe::pExport()
{
  if (!wDatei->isValid()) return;

  QSqlQuery query,query2;
  QString   s,s2;

  pFuellListe();
  wDatei->write("MID",true,false);
  wDatei->write("Vorname",true,false);
  wDatei->write("Nachname",true,false);
  wDatei->write("Strasse",true,false);
  wDatei->write("PLZ",true,false);
  wDatei->write("Stadt",true,false);
  wDatei->write("Land",true,false);
  wDatei->write("Gliederung",true,false);
  wDatei->write("Bezahlt bis",true,false);
  wDatei->write("Beitrag",true,false);
  wDatei->write("Zahlart",true,true);
  wDatei->newLine();

  query.exec("SELECT mid,name1,name2,strasse,plz,ort,land,gliederung,bezahlt,beitrag,zahlart,wohngliederung FROM zwiakkred ORDER BY sort1,sort2,mid");
  while(query.next())
  {
    wDatei->write(query.value(0).toString(),false,false);
    wDatei->write(query.value(2).toString(),true,false);
    wDatei->write(query.value(1).toString(),true,false);
    wDatei->write(query.value(3).toString(),true,false);
    wDatei->write(query.value(4).toString(),true,false);
    wDatei->write(query.value(5).toString(),true,false);
    wDatei->write(query.value(6).toString(),true,false);
    s=query.value(7).toString();
    s2=query.value(11).toString();
    if(s2.length()>0 && s2!=s && s2 != wGliederung->text())
    {
      while(s2!=s && s2!=wGliederung->text())
      {
        query2.exec("SELECT teilvon FROM verband WHERE namekurz='" + s2 + "'");
        if (!query2.next())
        {
          s=s+" (" + query.value(11).toString() + ")";
          break;
        }
        s2=query2.value(0).toString();
        query2.finish();
        if (s2.length()<1)
        {
          s=s+" (" + query.value(11).toString() + ")";
          break;
        }
      }
    }
    wDatei->write(s,true,false);
    if (query.value(10).toString() == "X")
    {
      wDatei->write("",false,false);
      wDatei->write("",false,false);
      wDatei->write("",false,true);
    }
    else
    {
      wDatei->write(query.value(8).toDate().toString("dd.MM.yyyy"),false,false);
      wDatei->write(QLocale::system().toString((double)(query.value(9).toDouble()/100.0l),'f',2),true,false);
      if (query.value(10).toString()=="L")
         wDatei->write("Lastschrift",true,true);
      else if (query.value(10).toString()=="G")
         wDatei->write("! Lastschrift !",true,true);
      else
         wDatei->write("Vorkasse",true,true);
    }
    wDatei->newLine();
  }
  wDatei->closeFile();
  query.finish();
  query.exec("DROP TABLE zwiakkred");
  QMessageBox::information(this,"Akkreditierungsliste","Fertig");
  accept();
}
