#include "csvdatei.h"

CSVDatei::CSVDatei(bool flag_erstdat, bool flag_exists, QWidget *parent)
{
  wParent=parent;
  fErstdat=flag_erstdat;
  fExists=flag_exists;

  wDateiname=new QLineEdit(wParent);
  bSelect=new QPushButton("&Whlen",wParent);
  wFeldtrenner=new QLineEdit(wParent);
  wTexttrenner=new QLineEdit(wParent);
  if (flag_erstdat)
     wErstellt=new QDateEdit(wParent);
  else
     wErstellt=NULL;
}

CSVDatei::~CSVDatei()
{
  if (wDatei.isOpen()) wDatei.close();
  delete wDateiname;
  delete wFeldtrenner;
  delete wTexttrenner;
  delete bSelect;
  if (wErstellt!=NULL) { delete wErstellt; wErstellt=NULL; }
}

void CSVDatei::setFlags(bool flag_erstdat, bool flag_exists)
{
  fErstdat=flag_erstdat;
  fExists=flag_exists;

  if (flag_erstdat && wErstellt==NULL)
     wErstellt=new QDateEdit(wParent);
}

int CSVDatei::FuellLayout(QGridLayout *wLayout, int ys, int x)
{
  int y;
  QLabel *l;

  wDateiname->setFixedSize(400,25);
  wFeldtrenner->setFixedSize(25,25);
  wTexttrenner->setFixedSize(25,25);

  wDateiname->setText("");
  wFeldtrenner->setText(",");
  wTexttrenner->setText("\"");
  bSelect->setAutoDefault(false);
  if (fErstdat)
  {
    wErstellt->setFixedSize(100,25);
    wErstellt->setDate(QDate::currentDate());
    wErstellt->setDisplayFormat("dd.MM.yyyy");
  }

  y=ys;
  l=new QLabel("Dateiname");
  wLayout->addWidget(l,y,1);
  wLayout->addWidget(wDateiname,y,2,1,6);
  wLayout->addWidget(bSelect,y,9);
  y++;
  l=new QLabel("Feldtrenner");
  wLayout->addWidget(l,y,1);
  wLayout->addWidget(wFeldtrenner,y,2);
  l=new QLabel("Texttrenner");
  wLayout->addWidget(l,y,3);
  wLayout->addWidget(wTexttrenner,y,4);
  if (fErstdat)
  {
    l=new QLabel("Erstelldatum");
    wLayout->addWidget(l,y,5);
    wLayout->addWidget(wErstellt,y,6);
  }
  y++;

  wLayout->setColumnStretch(7,1);

  connect(bSelect,SIGNAL(clicked()),this,SLOT(pDateiWahl()));
  wSpalte=-1;

  return(y);
}

bool CSVDatei::isValid()
{
  int i;
  bool f;

  wSpalte=-1;
  if (wDatei.isOpen()) wDatei.close();
  wDatei.setFileName(wDateiname->text());

  if (fExists)
     f=wDatei.open(QIODevice::ReadOnly);
  else
  {
     if (QFile::exists(wDateiname->text()))
     {
       i=QMessageBox::question(NULL,"CSV-Datei","Datei " + wDateiname->text() +
                      " berschreiben?",QMessageBox::Ok | QMessageBox::Cancel);
       if (i!=QMessageBox::Ok) return(false);
     }
     f=wDatei.open(QIODevice::WriteOnly);
  }

  if (!f)
  {
    QMessageBox::critical(NULL,wDateiname->text(),wDatei.errorString());
    return(false);
  }
  cFeldtrenner=wFeldtrenner->text().toAscii()[0];
  cTexttrenner=wTexttrenner->text().toAscii()[0];
  if (cFeldtrenner==0) cFeldtrenner==';';
  if (cTexttrenner==0) cFeldtrenner=='"';
  return(true);
}

void CSVDatei::pDateiWahl()
{
  QFileDialog wDiag;
  QString     sDatei;

  wDiag.setDefaultSuffix("csv");
  wDiag.setNameFilters(QString("CSV file (*.csv),Any file (*)").split(","));
  wDiag.selectNameFilter("*.csv");
  if (fExists)
  {
    wDiag.setConfirmOverwrite(false);
    wDiag.setAcceptMode(QFileDialog::AcceptOpen);
    wDiag.setFileMode(QFileDialog::ExistingFile);
  }
  else
  {
    wDiag.setConfirmOverwrite(true);
    wDiag.setAcceptMode(QFileDialog::AcceptSave);
    wDiag.setFileMode(QFileDialog::AnyFile);
  }
  if (wDiag.exec() == QDialog::Accepted)
  {
     sDatei=wDiag.selectedFiles()[0];
     if (sDatei!="") wDateiname->setText(sDatei);
  }
}

bool CSVDatei::startLine()
{

  wSpalte=-1; wLen=0;
  while(true)
  {
    if (wDatei.atEnd()) return(false);
    wZeile=wDatei.readLine();
    if (!wZeile.isEmpty()) break;
  }
  wSpalte=0;
  wLen=wZeile.length();
  return(true);
}

QString CSVDatei::readField()
{
  QByteArray wErg;
  int        i;
  bool       flag_text;

  if (wSpalte<0 || wSpalte>=wLen) return("");
  wErg.clear();
  flag_text=false;
  while(wSpalte<wLen)
  {
    if (wZeile[wSpalte]==cFeldtrenner && !flag_text)
    {
      wSpalte++;
      break;
    }
    if (wZeile[wSpalte]==cTexttrenner)
    {
      wSpalte++;
      flag_text=!flag_text;
    }
    else
    {
      for (i=wSpalte+1;
           i<wLen && wZeile[i]!=cFeldtrenner && wZeile[i]!=cTexttrenner;
           i++);
      if (i>wSpalte) wErg.append(wZeile.mid(wSpalte,i-wSpalte));
      wSpalte=i;
    }
  }
  return(QString::fromUtf8(wErg.data()));
}

QDate CSVDatei::toDate(QString s, QDate fehlwert)
{
  QString f;
  bool flag_j2;
  QDate d;
  int   y,yh;

  flag_j2=false;
  if (s.length()!=8 && s.length()!=10) return(fehlwert);
  if (s[0].isDigit() && s[1].isDigit() &&
      s[3].isDigit() && s[4].isDigit() &&
      s[6].isDigit() && s[7].isDigit())
  {
    if (s.length()>=10)
       f="dd" + s[2] + "MM" + s[5] + "yyyy";
    else
    {  f="dd" + s[2] + "MM" + s[5] + "yy"; flag_j2=true; }
  }
  else
  {
    if (s.length()==10 &&
        s[0].isDigit() && s[1].isDigit() &&
        s[2].isDigit() && s[3].isDigit() &&
        s[5].isDigit() && s[6].isDigit() &&
        s[8].isDigit() && s[9].isDigit())
       f="yyyy" + s[4] + "MM" + s[7] + "dd";
    else
       return(fehlwert);
  }
  d=QDate::fromString(s,f);
  if (flag_j2)
  {
     y=d.year() % 100;
     yh=QDate::currentDate().year();
     if ((y+2000) > (yh + 10))
        y=y+1900;
     else
        y=y+2000;
     return(QDate(y,d.month(),d.day()));
  }
  return(d);
}

bool CSVDatei::eol()
{
  if (wSpalte<0 || wSpalte>=wLen) return(true);
  return(false);
}

QDate CSVDatei::creationDate()
{
  return(wErstellt->date());
}

void CSVDatei::newLine()
{
  wDatei.write("\n");
}

void CSVDatei::write(QString s, bool flag_text,bool flag_last)
{
  if (flag_text && cTexttrenner!=0 && cTexttrenner!=' ')
     wDatei.write(&cTexttrenner,1);
  wDatei.write(s.replace(cTexttrenner," ").toAscii().data());
  if (flag_text && cTexttrenner!=0 && cTexttrenner!=' ')
     wDatei.write(&cTexttrenner,1);
  if (!flag_last && cFeldtrenner!=0)
     wDatei.write(&cFeldtrenner,1);
}

void CSVDatei::closeFile()
{
  if (wDatei.isOpen()) wDatei.close();
}
