#include <QtGui>
#include <QSqlQuery>

#include "einzelkto.h"
#include "journal.h"
#include "utils.h"
#include "druckfkt.h"
#include "wkonto.h"

EinzelKto::EinzelKto(QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  QGridLayout *layout = new QGridLayout;

  setWindowTitle("Einzelkonto");

  lKontoText = new QLabel("");
  wKonto  = new WKonto(lKontoText,true,this);
  wKonto->setFixedSize(150,25);
  wVonDat = new QDateEdit(UtilGJVon());
  wVonDat->setDisplayFormat("dd.MM.yyyy");
  wBisDat = new QDateEdit(QDate::currentDate());
  wBisDat->setDisplayFormat("dd.MM.yyyy");
  wASaldo = new QLineEdit();
  wASaldo->setEnabled(false);
  wASaldo->setFixedSize(95,25);
  wASaldo->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
  wESaldo = new QLineEdit();
  wESaldo->setEnabled(false);
  wESaldo->setFixedSize(95,25);
  wESaldo->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
  wKost = new QLineEdit();
  wKost->setFixedSize(150,25);
  bOk=new QPushButton("&OK");
  bKtoVor = new QPushButton("&Vorwrts");
  bKtoVor->setAutoDefault(false);
  bKtoRu  = new QPushButton("&Zurck");
  bKtoRu->setAutoDefault(false);
  bDrucken=new QPushButton("&Drucken");
  bDrucken->setAutoDefault(false);

  wListe = new QTableView;
  wModel = new QStandardItemModel();
  wModel->setColumnCount(7);
  wListe->setModel(wModel);
  wListe->setEnabled(true);
  wListe->setFixedSize(1000,500);

  l=new QLabel("Konto");
  layout->addWidget(l,1,1);
  layout->addWidget(wKonto,1,2);
  layout->addWidget(lKontoText,1,3,1,7);
  l=new QLabel("Von");
  layout->addWidget(l,2,1);
  layout->addWidget(wVonDat,2,2);
  l=new QLabel("Anfangssaldo");
  layout->addWidget(l,2,3);
  layout->addWidget(wASaldo,2,4);
  l=new QLabel("Bis");
  layout->addWidget(l,3,1);
  layout->addWidget(wBisDat,3,2);
  l=new QLabel("Endsaldo");
  layout->addWidget(l,3,3);
  layout->addWidget(wESaldo,3,4);
  l=new QLabel("Kost.");
  layout->addWidget(l,3,5);
  layout->addWidget(wKost,3,6);
  layout->addWidget(bKtoRu,3,8);
  layout->addWidget(bKtoVor,3,9);
  layout->addWidget(bDrucken,3,10);
  layout->addWidget(bOk,3,11);
  layout->setColumnStretch(7,1);
  layout->addWidget(wListe,4,1,10,11);

  setLayout(layout);

  pFuellListe();

  connect(wKonto,SIGNAL(returnPressed()),this,SLOT(pEnter()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pEnter()));
  connect(bDrucken,SIGNAL(clicked()),this,SLOT(pDrucken()));
  connect(bKtoVor,SIGNAL(clicked()),this,SLOT(pKtoVor()));
  connect(bKtoRu,SIGNAL(clicked()),this,SLOT(pKtoRu()));
  connect(wListe,SIGNAL(clicked(QModelIndex)),this,SLOT(pAuswahl(QModelIndex)));
}

EinzelKto::~EinzelKto()
{
  delete wKonto;
  delete lKontoText;
  delete wVonDat;
  delete wBisDat;
  delete wASaldo;
  delete wESaldo;
  delete wKost;
  delete bOk;
  delete bKtoVor;
  delete bKtoRu;
  delete wListe;
  delete wModel;
}

void EinzelKto::pFuellListe()
{
  QStandardItem *wItem;
  QSqlQuery     q1,q2,q3;
  QDate         wGJVon,wDat;
  long long     wSAnf,wSEnd,wCent;
  int           i;
  bool          flag_kost;

  pFelder();
  flag_kost=(sKost.length()>0);
  wModel->clear();
  wModel->setHorizontalHeaderLabels(QString("Datum,Nr,Kostenst.,Text,GKonto,Betrag").split(","));
  wListe->setColumnWidth(0,85);
  wListe->setColumnWidth(1,40);
  wListe->setColumnWidth(2,125);
  wListe->setColumnWidth(3,480);
  wListe->setColumnWidth(4,115);
  wListe->setColumnWidth(5,95);
  if (sKonto.length()<1) return;

  wGJVon=UtilGJVonDatum(wVonDat->date());
  q1.exec("SELECT datum,lfd,cent,kg FROM kgkto WHERE konto='" +
          sKonto + "' AND datum>='" +
          wGJVon.toString("yyyy-MM-dd") + "' AND datum <='" +
          wBisDat->date().toString("yyyy-MM-dd") + "'");
  wSAnf=wSEnd=0;
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
  for (i=0; q1.next();)
  {
    q2.exec("SELECT kost,butext,beschluss,art FROM journdat WHERE datum='" +
            q1.value(0).toString() + "' AND lfd='" +
            q1.value(1).toString() + "'");
    if (!q2.next()) continue;
    if (flag_kost && q2.value(0).toString().toUpper() != sKost)
    {
      q2.finish();
      continue;
    }
    wDat = q1.value(0).toDate();
    wCent= q1.value(2).toLongLong();
    if (q2.value(3).toString()!="A" || wDat==wGJVon)
       wSEnd += wCent;
    if (wDat<wVonDat->date() ||
        (wDat==wVonDat->date() && q2.value(3).toString()=="A"))
       wSAnf += wCent;
    if (wDat>=wVonDat->date())
    {
      wItem=new QStandardItem(q1.value(0).toDate().toString("dd.MM.yyyy"));
      wItem->setEditable(false);
      wModel->setItem(i,0,wItem);
      wItem=new QStandardItem(QString::number(q1.value(1).toInt()));
      wItem->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
      wItem->setEditable(false);
      wModel->setItem(i,1,wItem);
      wItem=new QStandardItem(q2.value(0).toString());
      wItem->setEditable(false);
      wModel->setItem(i,2,wItem);
      if (q2.value(2).toString().length()<1)
        wItem=new QStandardItem(q2.value(1).toString());
      else
        wItem=new QStandardItem(q2.value(2).toString() + ", " + q2.value(1).toString());
      wItem->setEditable(false);
      wModel->setItem(i,3,wItem);
      q3.exec("SELECT konto FROM kgkto WHERE datum='" +
              q1.value(0).toString() + "' AND lfd='" +
              q1.value(1).toString() + "' AND kg!='" +
              q1.value(3).toString() + "'");
      if (q3.next())
      {
        wItem=new QStandardItem(q3.value(0).toString());
        wItem->setEditable(false);
        wModel->setItem(i,4,wItem);
        q3.finish();
      }
      wItem=new QStandardItem(QString::number((double)wCent/100.0f,'f',2));
      wItem->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
      wItem->setEditable(false);
      wModel->setItem(i,5,wItem);
      i++;
    }
    q2.finish();
  }
  QApplication::restoreOverrideCursor();
  wASaldo->setText(QString::number((double)wSAnf/100.0f,'f',2));
  wESaldo->setText(QString::number((double)wSEnd/100.0f,'f',2));
}

void EinzelKto::pEnter()
{
  pFuellListe();
}

void EinzelKto::pFelder()
{
  sKonto=wKonto->text().replace("'","`").simplified().toUpper();
  wKonto->setText(sKonto);
  sKost=wKost->text().replace("'","`").simplified().toUpper();
  wKost->setText(sKost);
}

void EinzelKto::pDrucken()
{
  pFelder();
  if (sKonto.length()<1) return;

  QPrinter *wDrucker = new QPrinter(QPrinter::HighResolution);
  wDrucker->setOrientation(QPrinter::Landscape);
  QPrintDialog wDiag(wDrucker,this);
  if (wDiag.exec() == QDialog::Accepted)
  {
    wDrucker=wDiag.printer();
    QPainter *wPainter = new QPainter();
    wPainter->begin(wDrucker);
    DruckKonto(wDrucker,wPainter,sKonto,wVonDat->date(),wBisDat->date(),sKost);
    wPainter->end();
    delete wPainter;
  }
  delete(wDrucker);
  return;
}

void EinzelKto::pAuswahl(QModelIndex i)
{
  int   wZeile = i.row();
  QDate wDatum;
  int   wLfd,r;
  Journal wJournalDiag;

  wDatum=QDate::fromString(wModel->item(wZeile,0)->text(),"dd.MM.yyyy");
  wLfd  =wModel->item(wZeile,1)->text().toInt();
  r=wJournalDiag.exec(wDatum,wLfd);
  if (r=QDialog::Accepted) pFuellListe();
}

void EinzelKto::pKtoVor()
{
  QSqlQuery query;

  pFelder();
  query.exec("SELECT konto FROM kgkto WHERE konto>'" + sKonto +
          "' AND datum >='" + wVonDat->date().toString("yyyy-MM-dd") +
          "' AND datum <='" + wBisDat->date().toString("yyyy-MM-dd") +
          "' ORDER BY konto,datum,lfd LIMIT 1");
  if (query.next())
  {
    wKonto->setText(query.value(0).toString());
    query.finish();
  }
  else
    wKonto->setText("");
  pEnter();
}

void EinzelKto::pKtoRu()
{
  QSqlQuery query;

  pFelder();

  if (sKonto.length()<1)
     query.exec("SELECT konto FROM kgkto WHERE datum<='" +
             wBisDat->date().toString("yyyy-MM-dd") +
             "' AND datum >='" + wVonDat->date().toString("yyyy-MM-dd") +
             "' ORDER BY konto DESC,datum DESC,lfd DESC LIMIT 1");
  else
     query.exec("SELECT konto FROM kgkto WHERE konto<'" + sKonto +
             "' AND datum <='" + wBisDat->date().toString("yyyy-MM-dd") +
             "' AND datum >='" + wVonDat->date().toString("yyyy-MM-dd") +
             "' ORDER BY konto DESC,datum DESC,lfd DESC LIMIT 1");
  if (query.next())
  {
    wKonto->setText(query.value(0).toString());
    query.finish();
  }
  else
    wKonto->setText("");
  pEnter();
}
