#include <QtGui>
#include <QtSql>
#include "treeitem.h"
#include "gliederungsbaum.h"

GliederungsBaum::GliederungsBaum(QObject *parent) : QAbstractItemModel(parent)
{
  rootItem=NULL;
}

GliederungsBaum::~GliederungsBaum()
{
  if (rootItem!=NULL) { delete rootItem; rootItem=NULL; }
}

int GliederungsBaum::columnCount(const QModelIndex &parent) const
{
  if (parent.isValid())
     return static_cast<TreeItem*>(parent.internalPointer())->columnCount();
  else if (rootItem!=NULL)
     return rootItem->columnCount();
  else
     return(0);
}

QVariant GliederungsBaum::data(const QModelIndex &index, int role) const
{
  if (!index.isValid() || role!=Qt::DisplayRole) return QVariant();
  TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
  return item->data(index.column());
}

QVariant GliederungsBaum::datac(const QModelIndex &index, int column) const
{
  if (!index.isValid()) return QVariant();
  TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
  return item->data(column);
}

Qt::ItemFlags GliederungsBaum::flags(const QModelIndex &index) const
{
  if (!index.isValid()) return(0);
  return(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
}

QVariant GliederungsBaum::headerData(int section, Qt::Orientation orientation,
                                     int role) const
{
  if (rootItem!=NULL && orientation == Qt::Horizontal && role == Qt::DisplayRole)
     return(rootItem->data(section));
  else
     return(QVariant());
}

QModelIndex GliederungsBaum::index(int row, int column,
                                   const QModelIndex &parent) const
{
  TreeItem *parentItem;

  if (rootItem==NULL || !hasIndex(row,column,parent)) return(QModelIndex());

  if (!parent.isValid())
     parentItem = rootItem;
  else
     parentItem = static_cast<TreeItem*>(parent.internalPointer());
  TreeItem *childItem = parentItem->child(row);
  if (childItem) return(createIndex(row,column,childItem));
  return(QModelIndex());
}

QModelIndex GliederungsBaum::parent(const QModelIndex &index) const
{
  if (!index.isValid()) return(QModelIndex());

  TreeItem *childItem  =static_cast<TreeItem*>(index.internalPointer());
  TreeItem *parentItem =childItem->parent();

  if (rootItem==NULL || parentItem == rootItem)
     return(QModelIndex());
  else
     return(createIndex(parentItem->row(),0,parentItem));
}

int GliederungsBaum::rowCount(const QModelIndex &parent) const
{
  TreeItem *parentItem;

  if (rootItem==NULL || parent.column()>0) return(0);
  if (!parent.isValid())
     parentItem=rootItem;
  else
     parentItem=static_cast<TreeItem*>(parent.internalPointer());
  return(parentItem->childCount());
}

QModelIndex GliederungsBaum::setupModelData(QString sDarstell,int ebene)
{
  QList<TreeItem*> wListe;
  QList<QVariant>  rootData;
  QSqlQuery        query;
  int              wEbene;
  QString          vglTeilvon;
  TreeItem         *vglParent,*wItem,*wDarstell;
  int              i,n;

  beginResetModel();
  wDarstell=NULL;
  if (rootItem!=NULL) { delete(rootItem);  rootItem=NULL; }
  rootData << "Kurzname" << "Konto" << "Schatzmeister";
  rootItem = new TreeItem(rootData);

  for (wEbene=0; wEbene<5 && wEbene<ebene; wEbene++)
  {
    vglTeilvon="";
    vglParent=rootItem;
    query.exec("SELECT namekurz,bukto,schatzname,teilvon FROM verband WHERE ebene='" +
               QString::number(wEbene) + "' ORDER BY ebene,teilvon,namekurz");
    while(query.next())
    {
      if (query.value(3).toString() != vglTeilvon)
      {
        if (query.value(3).toString().length()<1)
        {
          vglTeilvon="";
          vglParent=rootItem;
        }
        else
        {
          n=wListe.count();
          vglTeilvon=query.value(3).toString();
          vglParent=rootItem;
          for (i=0; i<n; i++)
          {
            if (wListe[i]->data(0) == vglTeilvon)
            {
              vglParent=wListe[i];
              break;
            }
          }
          if (vglParent==rootItem)
          {
              vglTeilvon="";
          }
        }
      }
      QList<QVariant> wData;
      wData << query.value(0).toString() << query.value(1).toString() << query.value(2).toString();
      wItem=new TreeItem(wData,vglParent);
      vglParent->appendChild(wItem);
      wListe << wItem;
      if (sDarstell==query.value(0).toString()) wDarstell=wItem;
    }
  }

  endResetModel();

  if (wDarstell==NULL) return(QModelIndex());
  return(createIndex(0,0,wDarstell));
}
