#include <QtGui>
#include <QSqlQuery>

#include "journal.h"
#include "utils.h"
#include "wkonto.h"

Journal::Journal(QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  QGridLayout *layout = new QGridLayout;
  int z;

  setWindowTitle("Journal");
  fEinzeln=false;

  lKontoText = new QLabel("",this);
  lGKontoText = new QLabel("",this);

  wDatum  = new QDateEdit(QDate::currentDate(),this);
  wDatum->setDisplayFormat("dd.MM.yyyy");
  wDatum->setFixedSize(wDatum->sizeHint());
  wLfd    = new QLineEdit(this);
  wLfd->setValidator(new QIntValidator(0,9999,this));
  wLfd->setFixedSize(50,25);
  wText   = new QLineEdit(this);
  wText->setFixedSize(900,25);
  wKost  = new QLineEdit(this);
  wKost->setFixedSize(200,25);
  wBeschluss= new QLineEdit(this);
  wBeschluss->setFixedSize(200,25);
  wBeleg  = new QLineEdit(this);
  wBeleg->setFixedSize(100,25);
  wBetrag = new QDoubleSpinBox(this);
  wBetrag->setDecimals(2);
  wBetrag->setMinimum(-99999999.99);
  wBetrag->setMaximum( 99999999.99);
  wBetrag->setSuffix(" EUR");
  wBetrag->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
  wBetrag->setFixedSize(150,25);
  bSpende = new QPushButton("S&penden",this);
  bSpende->setFixedSize(bSpende->sizeHint());
  lSpenderName = new QLabel("",this);
  wKonto  = new WKonto(lKontoText,true,this);
  wKonto->setFixedSize(150,25);
  wGKonto = new WKonto(lGKontoText,true,this);
  wGKonto->setFixedSize(150,25);
  bOk = new QPushButton("&Speichern");
  bCancel = new QPushButton("&Abbruch");
  bDel = new QPushButton("&Lschen");

  wListe = new QTableView();
  wModel = new QStandardItemModel();
  wModel->setColumnCount(7);
  wListe->setModel(wModel);
  wListe->setEnabled(true);

  wListe->setFixedSize(1000,300);

  z=1;
  l=new QLabel("Datum",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wDatum,z,2);
  l=new QLabel("Nummer",this);
  layout->addWidget(l,z,3);
  layout->addWidget(wLfd,z,4);
  z++;
  l=new QLabel("Text",this);
  layout->addWidget(l,2,1);
  layout->addWidget(wText,2,2,1,9);
  z++;
  l=new QLabel("Kostenstelle",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wKost,z,2);
  l=new QLabel("Beschluss",this);
  layout->addWidget(l,z,3);
  layout->addWidget(wBeschluss,z,4);
  l=new QLabel("Beleg",this);
  layout->addWidget(l,z,5);
  layout->addWidget(wBeleg,z,6);
  z++;
  l=new QLabel("Betrag",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wBetrag,z,2);
  layout->addWidget(bSpende,z,3);
  layout->addWidget(lSpenderName,z,4,1,7);
  z++;
  l=new QLabel("Konto",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wKonto,z,2);
  layout->addWidget(lKontoText,z,3,1,4);
  z++;
  l=new QLabel("Gegenkonto",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wGKonto,z,2);
  layout->addWidget(lGKontoText,z,3,1,4);
  layout->setColumnStretch(7,1);
  layout->addWidget(bOk,z,8);
  layout->addWidget(bCancel,z,9);
  layout->addWidget(bDel,z,10);
  z++;
  layout->addWidget(wListe,z,1,10,10);

  setLayout(layout);

  pFuellListe();
  pSetWahl(true);
  wLfd->setFocus();

  connect(wLfd,SIGNAL(returnPressed()),this,SLOT(pEnter()));
  connect(wLfd,SIGNAL(editingFinished()),this,SLOT(pRead()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(pCancel()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pWrite()));
  connect(bDel,SIGNAL(clicked()),this,SLOT(pDelete()));
  connect(bSpende,SIGNAL(clicked()),this,SLOT(pSpende()));
  connect(wListe,SIGNAL(clicked(QModelIndex)),this,SLOT(pAuswahl(QModelIndex)));
  return;
}

Journal::~Journal()
{
  delete lKontoText;
  delete lGKontoText;
  delete wDatum;
  delete wLfd;
  delete wText;
  delete wKost;
  delete wBeschluss;
  delete wBeleg;
  delete wBetrag;
  delete bSpende;
  delete lSpenderName;
  delete wKonto;
  delete wGKonto;
  delete bOk;
  delete bCancel;
  delete bDel;
  delete wModel;
  delete wListe;
}

void Journal::pSetWahl(bool flag_wahl)
{
  fWahl=flag_wahl;

  bOk->setEnabled(!flag_wahl);
  bCancel->setEnabled(!flag_wahl);
  bDel->setEnabled(!flag_wahl);

  wDatum->setEnabled(flag_wahl);
  wLfd->setEnabled(flag_wahl);

  wKost->setEnabled(!flag_wahl);
  wBeleg->setEnabled(!flag_wahl);
  wBeschluss->setEnabled(!flag_wahl);
  wText->setEnabled(!flag_wahl);
  wKonto->setEnabled(!flag_wahl);
  wGKonto->setEnabled(!flag_wahl);
  wBetrag->setEnabled(!flag_wahl);
  bSpende->setEnabled(!flag_wahl);

  if (fWahl)
  {
    wLfd->setText("");
    wKost->setText("");
    wBeleg->setText("");
    wBeschluss->setText("");
    wText->setText("");
    wKonto->setText("");
    wGKonto->setText("");
    wBetrag->setValue(0);
    lKontoText->setText("");
    lGKontoText->setText("");
    lSpenderName->setText("");
    wSpende.clear();
  }

  if (fWahl)
     wLfd->setFocus();
  else
     if (pCheck())
        bCancel->setFocus();
  
}

void Journal::closeEvent(QCloseEvent *e)
{
  if (fWahl)
    e->accept();
  else
  {
    pCancel();
    e->ignore();
  }
}

void Journal::keyPressEvent(QKeyEvent *e)
{
  if (e->key()==Qt::Key_Escape)
  {
    if (fWahl)
       close();
    else
       pCancel();
    e->ignore();
  }
  e->accept();
}

void Journal::pCancel()
{
  if (fEinzeln) reject();
  if (fWahl)
    close();
  else
    pSetWahl(true);

}

void Journal::pRead()
{
  QSqlQuery query;

  if (!fWahl || wLfd->text().simplified().length()<1) return;

  pFelder();
  if (iLfd>0)
  {
    query.exec("SELECT kost,beleg,butext,beschluss FROM journdat WHERE datum='" +
               sDatum + "' AND lfd='" + sLfd + "'");
    if (!query.next()) return;
    wKost->setText(query.value(0).toString());
    wBeleg->setText(query.value(1).toString());
    wText->setText(query.value(2).toString());
    wBeschluss->setText(query.value(3).toString());
    query.finish();
    query.exec("SELECT konto,cent FROM kgkto WHERE datum='" +
               sDatum + "' AND lfd='" + sLfd + "' AND kg='1'");
    if (query.next())
    {
      wKonto->setText(query.value(0).toString());
      wBetrag->setValue(query.value(1).toDouble() / (double)100.0l);
      query.finish();
    }
    query.exec("SELECT konto FROM kgkto WHERE datum='" +
               sDatum + "' AND lfd='" + sLfd + "' AND kg='2'");
    if (query.next())
    {
      wGKonto->setText(query.value(0).toString());
      query.finish();
    }
    wSpende.read(wDatum->date(),iLfd);
    lSpenderName->setText(wSpende.disName());
  }
  pSetWahl(false);
}

void Journal::pWrite()
{
  int wBuLfd;
  if (fWahl) return;
  pFelder();
  if (!pCheck()) return;
  wBuLfd=UtilBuch(wDatum->date(),iLfd,sKost,sBeleg,sText,
                  (qint64)round((wBetrag->value() * (double)(100.0l))),
                  sKonto,sGKonto,sBeschluss,"");
  if (wBuLfd<1)
  {
    QMessageBox::critical(this,"Journal","Fehler beim Generieren der Nummer");
    return;
  }
  wSpende.write(wDatum->date(),wBuLfd);
  if (fEinzeln) accept();
  pFuellListe();
  pSetWahl(true);
}

void Journal::pDelete()
{
  QSqlQuery query;

  if (fWahl) return;
  pFelder();
  if (iLfd>0)
  {
    if (!UtilQDelete()) return;
    query.exec("BEGIN TRANSACTION");
    query.exec("DELETE FROM spende WHERE jdatum='" + sDatum +
               "' AND jlfd='" + sLfd + "'");
    query.exec("DELETE FROM kgkto WHERE datum='" + sDatum +
               "' AND lfd='" + sLfd + "'");
    query.exec("DELETE FROM journdat WHERE datum='" + sDatum +
               "' AND lfd='" + sLfd + "'");
    query.exec("COMMIT TRANSACTION");
    pFuellListe();
  }
  if (fEinzeln) accept();
  pSetWahl(true);
}

void Journal::pFelder()
{
  iLfd=wLfd->text().simplified().toInt();
  sDatum=wDatum->date().toString("yyyy-MM-dd");
  sLfd=QString::number(iLfd);
  sKonto=wKonto->text().replace("'","`").simplified();
  sGKonto=wGKonto->text().replace("'","`").simplified();
  sKost=wKost->text().replace("'","`").simplified();
  sBeleg=wBeleg->text().replace("'","`").simplified();
  sBeschluss=wBeschluss->text().replace("'","`").simplified();
  sText=wText->text().replace("'","`").simplified();
}

void Journal::pEnter()
{
  if (!fWahl) return;
  if (wLfd->text().simplified().length()<1) wLfd->setText("0");
  pRead();
}

bool Journal::pCheck()
{
  if (wText->text().simplified().length()<1)
  {
    wText->setFocus();
    return(false);
  }
  if (wKost->text().simplified().length()<1)
  {
    wKost->setFocus();
    return(false);
  }
  if (wKonto->text().simplified().length()<1)
  {
    wKonto->setFocus();
    return(false);
  }
  if (wGKonto->text().simplified().length()<1)
  {
    wGKonto->setFocus();
    return(false);
  }
  if (wBetrag->value()==0)
  {
    wBetrag->setFocus();
    return(false);
  }
  return(true);
}

void Journal::pFuellListe()
{
  int           i;
  QStandardItem *wItem;
  QSqlQuery     q1,q2;

  wModel->clear();
  wModel->setHorizontalHeaderLabels(QString("Datum,Nr,Kostenst.,Text,Konto,GKonto,Betrag").split(","));

  q1.exec("SELECT datum,lfd,kost,butext FROM journdat ORDER BY datum DESC,lfd DESC LIMIT 10");
  for (i=0; i<10 && q1.next(); i++)
  {
    wItem=new QStandardItem(q1.value(0).toDate().toString("dd.MM.yyyy"));
    wItem->setEditable(false);
    wModel->setItem(i,0,wItem);
    wItem=new QStandardItem(QString::number(q1.value(1).toInt()));
    wItem->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
    wItem->setEditable(false);
    wModel->setItem(i,1,wItem);
    wItem=new QStandardItem(q1.value(2).toString());
    wItem->setEditable(false);
    wModel->setItem(i,2,wItem);
    wItem=new QStandardItem(q1.value(3).toString());
    wItem->setEditable(false);
    wModel->setItem(i,3,wItem);
    q2.exec("SELECT cent,konto FROM kgkto WHERE datum='" +
            q1.value(0).toString() + "' AND lfd='" +
            q1.value(1).toString() + "' AND kg='1'");
    if (q2.next())
    {
      wItem=new QStandardItem(q2.value(1).toString());
      wItem->setEditable(false);
      wModel->setItem(i,4,wItem);
      wItem=new QStandardItem(QString::number((double)q2.value(0).toInt()/100.0f,'f',2));
      wItem->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
      wItem->setEditable(false);
      wModel->setItem(i,6,wItem);
      q2.finish();
    }
    q2.exec("SELECT konto FROM kgkto WHERE datum='" +
            q1.value(0).toString() + "' AND lfd='" +
            q1.value(1).toString() + "' AND kg='2'");
    if (q2.next())
    {
      wItem=new QStandardItem(q2.value(0).toString());
      wItem->setEditable(false);
      wModel->setItem(i,5,wItem);
      q2.finish();
    }
  }
  q1.finish();
  wModel->setRowCount(i);
  wListe->setColumnWidth(0,85);
  wListe->setColumnWidth(1,35);
  wListe->setColumnWidth(2,110);
  wListe->setColumnWidth(3,410);
  wListe->setColumnWidth(4,115);
  wListe->setColumnWidth(5,115);
  wListe->setColumnWidth(6,85);
}

void Journal::pAuswahl(QModelIndex i)
{
  int wZeile = i.row();

  if (fWahl)
  {
    wDatum->setDate(QDate::fromString(wModel->item(wZeile,0)->text(),"dd.MM.yyyy"));
    wLfd->setText(wModel->item(wZeile,1)->text().simplified());
    pRead();
  }
}

void Journal::pSpende()
{
  wSpende.exec(round(wBetrag->value()*100.0l));
  lSpenderName->setText(wSpende.disName());
}

int Journal::exec()
{
  return(QDialog::exec());
}

int Journal::exec(QDate wBestDatum, int wBestLfd)
{
  int r;

  fEinzeln=true;
  wDatum->setDate(wBestDatum);
  wLfd->setText(QString::number(wBestLfd));
  pRead();
  r=QDialog::exec();
  return(r);
}
