#include <QtGui>
#include <QSqlQuery>

#include "kontenplan.h"
#include "utils.h"

Kontenplan::Kontenplan(bool flag_anlegen, QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  QGridLayout *layout = new QGridLayout;

  fAnlegen=flag_anlegen;
  sNeuBez="";
  setWindowTitle("Kontenplan");
  wKonto = new WKonto(NULL,fAnlegen,this);
  wKonto->setMaxLength(15);
  wKonto->setFixedSize(150,25);
  wText = new QLineEdit;
  wText->setMaxLength(80);
  wText->setFixedSize(500,25);
  wAbgrArt = new QComboBox();
  wAbgrArt->insertItem(0,"");
  wAbgrArt->insertItem(1,"Saldo");
  wAbgrArt->insertItem(2,"OPs");
  bOk = new QPushButton("&Speichern");
  bCancel = new QPushButton("&Abbruch");
  bOk->setAutoDefault(false);
  bCancel->setAutoDefault(false);
  if (!fAnlegen)
  {
    bDel = new QPushButton("&Lschen");
    bDel->setAutoDefault(false);
  }

  wListe = new QTableView;
  wModel = new QStandardItemModel();
  wModel->setColumnCount(2);
  wListe->setModel(wModel);
  wListe->setEnabled(true);

  wAbgrArt->setFixedSize(wAbgrArt->sizeHint());

  l=new QLabel("Konto");
  layout->addWidget(l,1,1);
  l=new QLabel("Text");
  layout->addWidget(l,1,2);
  layout->addWidget(wKonto,2,1);
  layout->addWidget(wText,2,2);
  l=new QLabel("Abgrenzung");
  layout->addWidget(l,3,1);
  layout->addWidget(wAbgrArt,3,2);
  layout->addWidget(bOk,1,3);
  layout->addWidget(bCancel,2,3);
  if (!fAnlegen) layout->addWidget(bDel,3,3);
  layout->addWidget(wListe,4,1,5,3);
  setLayout(layout);

  pFuellListe();
  pSetWahl(true);

  connect(wKonto,SIGNAL(returnPressed()),this,SLOT(pRead()));
  connect(wKonto,SIGNAL(editingFinished()),this,SLOT(pRead()));
  connect(wText,SIGNAL(returnPressed()),this,SLOT(pWrite()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(pCancel()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pWrite()));
  if (!fAnlegen)
     connect(bDel,SIGNAL(clicked()),this,SLOT(pDelete()));
  connect(wListe,SIGNAL(clicked(QModelIndex)),this,SLOT(pAuswahl(QModelIndex)));
}

Kontenplan::~Kontenplan()
{
  return;
}

void Kontenplan::pSetWahl(bool flag_wahl)
{
  fWahl=flag_wahl;
  bOk->setEnabled(!flag_wahl);
  bCancel->setEnabled(!flag_wahl);
  if (!fAnlegen) bDel->setEnabled(!flag_wahl);
  wKonto->setEnabled(flag_wahl);
  wText->setEnabled(!flag_wahl);
  wAbgrArt->setEnabled(!flag_wahl);
  if (flag_wahl)
  {
    wKonto->setText("");
    wText->setText("");
    wAbgrArt->setCurrentIndex(0);
  }
  if (flag_wahl)
     wKonto->setFocus();
  else if (wText->text().length()<1)
     wText->setFocus();
  else
     bCancel->setFocus();
}

void Kontenplan::pFelder()
{
  sKonto=wKonto->text().replace("'","`").simplified();
  sText=wText->text().replace("'","`").simplified();
  switch(wAbgrArt->currentIndex())
  {
    case 1: sAbgr="S"; break;
    case 2: sAbgr="O"; break;
    default: sAbgr=" "; break;
  }
}

void Kontenplan::pRead()
{
  QSqlQuery query;

  pFelder();
  if (!fWahl || sKonto.length()<1) return;

  query.exec("SELECT text,abgrart FROM kontenplan WHERE konto='" + sKonto + "'");
  if (query.next())
  {
    wText->setText(query.value(0).toString());
    if (query.value(1).toString() == "S")
       wAbgrArt->setCurrentIndex(1);
    else if (query.value(1).toString() == "O")
       wAbgrArt->setCurrentIndex(2);
    else
       wAbgrArt->setCurrentIndex(0);
  }
  query.finish();
  pSetWahl(false);
}

void Kontenplan::pCancel()
{
  if (fWahl || fAnlegen)
    reject();
  else
  {
    pSetWahl(true);
  }
}

void Kontenplan::pWrite()
{
  QSqlQuery query;
  QString q;

  pFelder();
  if (fWahl || sText.length()<1) return;
  query.exec("UPDATE kontenplan SET text='" + sText + "',abgrart='" + sAbgr +
             "' WHERE konto = '" + sKonto + "'");
  if (query.numRowsAffected()<1)
  {
    query.exec("INSERT INTO kontenplan (konto,text,abgrart) VALUES ('" +
               sKonto + "','" + sText + "','" + sAbgr + "')");
  }
  if (fAnlegen)
  {
    sNeuBez=sKonto;
    accept();
    return;
  }
  pFuellListe();
  pSetWahl(true);
}

void Kontenplan::pDelete()
{
  QSqlQuery query;

  if (fWahl || fAnlegen) return;
  if (UtilQDelete())
  {
    pFelder();
    query.exec("DELETE FROM kontenplan WHERE konto='" + sKonto + "'");
    pFuellListe();
    pSetWahl(true);
  }
}

void Kontenplan::pFuellListe()
{
  int i;
  QStandardItem *wItem;
  QSqlQuery query;

  wModel->clear();
  wModel->setHorizontalHeaderLabels(QString("Konto,Text").split(","));
  query.exec("SELECT konto,text FROM kontenplan ORDER BY konto");
  for (i=0; query.next(); i++)
  {
    wItem=new QStandardItem(query.value(0).toString());
    wItem->setEditable(false);
    wModel->setItem(i,0,wItem);
    wItem=new QStandardItem(query.value(1).toString());
    wItem->setEditable(false);
    wModel->setItem(i,1,wItem);
  }
  wModel->setRowCount(i);
  wListe->setColumnWidth(0,150);
  wListe->setColumnWidth(1,500);
}

void Kontenplan::pAuswahl(QModelIndex i)
{
  int wZeile = i.row();

  if (fWahl)
  {
    wKonto->setText(wModel->item(wZeile,0)->text());
    pRead();
  }
}

void Kontenplan::closeEvent(QCloseEvent *e)
{
  if (fWahl)
    e->accept();
  else
  {
    pCancel();
    e->ignore();
  }
}

void Kontenplan::keyPressEvent(QKeyEvent *e)
{
  if (e->key()==Qt::Key_Escape)
  {
    if (fWahl)
       close();
    else
       pCancel();
    e->ignore();
  }
  e->accept();
}

QString Kontenplan::angelegt()
{
  return(sNeuBez);
}
