#include <QSqlQuery>

#include "ktodruck.h"
#include "utils.h"
#include "druckfkt.h"

Kontendruck::Kontendruck(QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  QGridLayout *layout = new QGridLayout;

  setWindowTitle("Kontendruck");

  lKontoVonText.setFixedSize(400,25);
  lKontoBisText.setFixedSize(400,25);
  lKontoAkt.setFixedSize(150,25);
  wKontoVon = new WKonto(&lKontoVonText,true,this);
  wKontoBis = new WKonto(&lKontoBisText,true,this);
  bDrucken.setText("&Drucken");
  bCancel.setText("&Abbruch");
  wMonatsweise.setText("&Monatsweise");
  wVonDat.setDate(UtilGJVon());
  wVonDat.setDisplayFormat("dd.MM.yyyy");
  wVonDat.setFixedSize(wVonDat.sizeHint());
  wBisDat.setDate(QDate::currentDate());
  wBisDat.setDisplayFormat("dd.MM.yyyy");
  wBisDat.setFixedSize(wBisDat.sizeHint());

  l=new QLabel("Konto von");
  layout->addWidget(l,1,1);
  layout->addWidget(wKontoVon,1,2,1,2);
  layout->addWidget(&lKontoVonText,1,4,1,3);
  l=new QLabel("Konto bis");
  layout->addWidget(l,2,1);
  layout->addWidget(wKontoBis,2,2,1,2);
  layout->addWidget(&lKontoBisText,2,4,1,3);
  l=new QLabel("Datum von");
  layout->addWidget(l,3,1);
  layout->addWidget(&wVonDat,3,2);
  l=new QLabel("bis");
  layout->addWidget(l,3,3);
  layout->addWidget(&wBisDat,3,4);
  layout->addWidget(&wMonatsweise,3,5);
  layout->addWidget(&bDrucken,4,1);
  layout->addWidget(&bCancel,4,2);
  layout->addWidget(&lKontoAkt,4,3,1,2);

  layout->setColumnStretch(6,1);

  setLayout(layout);

  connect(&bDrucken,SIGNAL(clicked()),this,SLOT(pDrucken()));
  connect(&bCancel,SIGNAL(clicked()),this,SLOT(reject()));
}

Kontendruck::~Kontendruck()
{
  delete wKontoVon;
  delete wKontoBis;
}

void Kontendruck::pDrucken()
{
  int       wAnzKto,wAnzSeiten;
  QSqlQuery query;
  QString   sAktKonto,sVgl;
  QDate     wAktVon,wAktBis;
  int       r;
  bool      flag_page;

  sKontoVon=wKontoVon->text().simplified().toUpper().replace("'","`");
  sKontoBis=wKontoBis->text().simplified().toUpper().replace("'","`");
  if (sKontoVon.length()<1)
  {
    wKontoVon->setFocus();
    return;
  }
  if (sKontoBis.length()<1)
     sKontoBis=sKontoVon;
  else if (sKontoBis<sKontoVon)
  {
    wKontoBis->setFocus();
    return;
  }
  if (wVonDat.date()>wBisDat.date())
  {
    wBisDat.setFocus();
    return;
  }
  wKontoVon->setText(sKontoVon);
  wKontoBis->setText(sKontoBis);

  QPrinter *wDrucker = new QPrinter(QPrinter::HighResolution);
  wDrucker->setOrientation(QPrinter::Landscape);
  QPrintDialog wDiag(wDrucker,this);
  if (wDiag.exec() == QDialog::Accepted)
  {
    wAnzKto=0;
    flag_page=false;
    sAktKonto=sKontoVon;
    sVgl=">=";
    wAnzSeiten=0;
    wDrucker=wDiag.printer();
    QPainter *wPainter = new QPainter();
    wPainter->begin(wDrucker);
    while(true)
    {
      query.exec("SELECT konto FROM kgkto WHERE konto " + sVgl + " '" +
                 sAktKonto + "' AND konto<='" + sKontoBis + "' AND datum>='" +
                 wVonDat.date().toString("yyyy-MM-dd") + "' AND datum<='" +
                 wBisDat.date().toString("yyyy-MM-dd") + "'");
      if (!query.next()) break;
      sAktKonto=query.value(0).toString();
      lKontoAkt.setText(sAktKonto);
      sVgl=">";
      wAnzKto++;
      QCoreApplication::processEvents();
      if (wMonatsweise.checkState() == Qt::Unchecked)
      {
        if (flag_page) { wDrucker->newPage(); flag_page=false; }
        r=DruckKonto(wDrucker,wPainter,sAktKonto,
                     wVonDat.date(),wBisDat.date(),"");
        if (r>0) flag_page=true;
        wAnzSeiten+=r;
      }
      else
      {
        wAktVon=wVonDat.date();
        while(wAktVon<=wBisDat.date())
        {
          wAktBis = QDate(wAktVon.year(),wAktVon.month(),1).addMonths(1).addDays(-1);
          if (wAktBis>wBisDat.date()) wAktBis=wBisDat.date();
          if (flag_page) { wDrucker->newPage(); flag_page=false; }
          r=DruckKonto(wDrucker,wPainter,sAktKonto,wAktVon,wAktBis,"");
          if (r>0) flag_page=true;
          wAnzSeiten+=r;
          wAktVon=wAktBis.addDays(1);
        }
      }
    }
    wPainter->end();
    delete wPainter;
    QMessageBox::information(this,"Kontendruck",
                 QString::number(wAnzKto) + " Konten auf " +
                 QString::number(wAnzSeiten) + " Seiten gedruckt");
  }
  delete(wDrucker);
  lKontoAkt.setText("");
}
