#include <QtGui>
#include <QtSql>
#include "lieferant.h"
#include "lssuche.h"
#include "utils.h"

Lieferant::Lieferant(bool flag_anlegen, QWidget *parent) : QDialog(parent)
{
  QLabel *l;

  fAnlegen=flag_anlegen;
  sNeuBez ="";
  QVBoxLayout *topLayout = new QVBoxLayout();
  QHBoxLayout *bezLayout = new QHBoxLayout();
  QHBoxLayout *buttonLayout = new QHBoxLayout();
  QGridLayout *allgLayout = new QGridLayout();
  QGridLayout *bankLayout = new QGridLayout();
  QTabWidget  *wTabWidget = new QTabWidget();
  QWidget     *wTabAllg   = new QWidget();
  QWidget     *wTabBank   = new QWidget();

  wTabWidget->addTab(wTabAllg,"A&dresse");
  wTabWidget->addTab(wTabBank,"&Bankverbindung");

  topLayout->addLayout(bezLayout);
  topLayout->addWidget(wTabWidget);
  topLayout->addLayout(buttonLayout);

  setWindowTitle("Lieferanten");
  bOk = new QPushButton("&Speichern");
  bCancel = new QPushButton("&Abbruch");
  bOk->setAutoDefault(false);
  bCancel->setAutoDefault(false);
  if (!fAnlegen)
  {
    bDel = new QPushButton("&Lschen");
    bDel->setAutoDefault(false);
  }

  wNameKurz=new QLineEdit(this); wNameKurz->setFixedSize(60,25);
  wAdresse =new WAdresse(0,this);
  wTel=new QLineEdit(this); wTel->setFixedSize(250,25);
  wTel->setMaxLength(25);
  wFax=new QLineEdit(this); wFax->setFixedSize(250,25);
  wFax->setMaxLength(25);
  wBukto = new QLineEdit(this); wBukto->setFixedSize(150,25);
  wBukto->setMaxLength(15);
  wBank    =new WBank(this);

  l=new QLabel("Bezeichnung");
  bezLayout->addWidget(l);
  bezLayout->addWidget(wNameKurz);
  bezLayout->addStretch(1);

  wAdresse->FuellLayout(allgLayout,1,1);
  l=new QLabel("Telefon");
  allgLayout->addWidget(l,8,1);
  allgLayout->addWidget(wTel,8,2,1,3);
  l=new QLabel("Fax");
  allgLayout->addWidget(l,9,1);
  allgLayout->addWidget(wFax,9,2,1,3);

  l=new QLabel("Buchungskonto");
  bankLayout->addWidget(l,1,1);
  bankLayout->addWidget(wBukto,1,2);
  wBank->FuellLayout(bankLayout,2,1);
  bankLayout->setColumnStretch(4,1);
  bankLayout->setRowStretch(99,1);

  buttonLayout->addWidget(bOk);
  buttonLayout->addWidget(bCancel);
  if (!fAnlegen)
     buttonLayout->addWidget(bDel);
  buttonLayout->addStretch(1);

  wTabAllg->setLayout(allgLayout);
  wTabBank->setLayout(bankLayout);
  setLayout(topLayout);

  pSetWahl(true);

  connect(wNameKurz,SIGNAL(returnPressed()),this,SLOT(pRead()));
  connect(wNameKurz,SIGNAL(editingFinished()),this,SLOT(pRead()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pWrite()));
  if (!fAnlegen)
     connect(bDel,SIGNAL(clicked()),this,SLOT(pDelete()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(pCancel()));

  return;
}

Lieferant::~Lieferant()
{
  delete wBank;
  delete wAdresse;
  delete wBukto;
  return;
}

void Lieferant::pSetWahl(bool flag_wahl)
{
  fWahl=flag_wahl;

  wNameKurz->setEnabled(flag_wahl);
  wTel->setEnabled(!flag_wahl);
  wFax->setEnabled(!flag_wahl);
  wBukto->setEnabled(!flag_wahl);
  wBank->setEnabled(!flag_wahl);
  wAdresse->setEnabled(!flag_wahl);

  bOk->setEnabled(!flag_wahl);
  bCancel->setEnabled(!flag_wahl);
  if (!fAnlegen) bDel->setEnabled(!flag_wahl);

  if (flag_wahl)
  {
    wAdresse->clear();
    wBank->clear();
    wNameKurz->setText("");
    wTel->setText("");
    wFax->setText("");
    wBukto->setText("");
  }

  if (flag_wahl)
    wNameKurz->setFocus();
  else if (wAdresse->isEmpty())
    wAdresse->Name1()->setFocus();
  else
    bCancel->setFocus();

  return;
}

void Lieferant::pRead()
{
  QSqlQuery query;
  int       r;

  pFelder();
  wNameKurz->setText(sNameKurz);
  if (!fWahl || sNameKurz.length()<1) return;
  if (sNameKurz.right(1)=="?")
  {
    wNameKurz->setText("");
    fWahl=false;
    LsSuche wDiag(false);
    r=wDiag.exec(sNameKurz);
    fWahl=true;
    if (r==QDialog::Accepted)
    {
      wNameKurz->setText(wDiag.auswahl());
      pFelder();
      if (sNameKurz.length()<1) return;
    }
    else
    {
      wNameKurz->setText("");
      wNameKurz->setFocus();
      return;
    }
  }
  if (sNameKurz.length()>6)
  {
     sNameKurz=sNameKurz.left(6);
     wNameKurz->setText(sNameKurz);
  }

  query.exec("SELECT tel,fax,bukto FROM lieferant WHERE bez='" + sNameKurz + "'");
  if (query.next())
  {
    wTel->setText(query.value(0).toString());
    wFax->setText(query.value(1).toString());
    wBukto->setText(query.value(2).toString());
  }
  wAdresse->read("L",sNameKurz);
  wBank->read("L",sNameKurz);
  pSetWahl(false);
  return;
}

void Lieferant::pCancel()
{
  if (fWahl || fAnlegen)
     close();
  else
     pSetWahl(true);
}

void  Lieferant::pFelder()
{
  sNameKurz=wNameKurz->text().replace("'","`").simplified().toUpper();
  sTel=wTel->text().replace("'","`").simplified();
  sFax=wFax->text().replace("'","`").simplified();
  sBukto=wBukto->text().replace("'","`").simplified().toUpper();
}

void Lieferant::pWrite()
{
  QSqlQuery query;

  if (fWahl) return;
  pFelder();
  if (wAdresse->Name1()->text().simplified().length()<1) return;

  query.exec("START TRANSACTION");
  query.exec("UPDATE lieferant SET tel='" + sTel +"',fax='" + sFax +
              "',bukto='" + sBukto +
              "' WHERE bez='" + sNameKurz  + "'");
  if (query.numRowsAffected()<1)
  {
    query.exec("INSERT INTO lieferant (bez,tel,fax,bukto,zahlart) VALUES ('" +
               sNameKurz + "','" + sTel + "','" + sFax + "','" + sBukto +
               "',' ')");
  }
  wAdresse->write("L",sNameKurz);
  wBank->write("L",sNameKurz);
  query.exec("COMMIT TRANSACTION");
  pSetWahl(true);
  if (fAnlegen)
  {
     sNeuBez=sNameKurz;
     accept();
  }
  return;
}

void Lieferant::pDelete()
{
  QSqlQuery query;

  if (fWahl) return;
  if (UtilQDelete())
  {
    pFelder();
    query.exec("START TRANSACTION");
    query.exec("DELETE FROM lieferant WHERE bez='" + sNameKurz + "'");
    wAdresse->del("L",sNameKurz);
    wBank->del("L",sNameKurz);
    query.exec("COMMIT TRANSACTION");
    pSetWahl(true);
  }
  return;
}

void Lieferant::closeEvent(QCloseEvent *e)
{
  if (fWahl)
    e->accept();
  else
  {
    pCancel();
    e->ignore();
  }
}

void Lieferant::keyPressEvent(QKeyEvent *e)
{
  if (e->key()==Qt::Key_Escape)
  {
    if (fWahl)
       close();
    else
       pCancel();
    e->ignore();
  }
  e->accept();
}

QString Lieferant::angelegt()
{
  return(sNeuBez);
}
