#include <QtGui>
#include <QtSql>

#include "lssuche.h"
#include "lieferant.h"
#include "utils.h"

LsSuche::LsSuche(bool flag_anlegen, QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  QGridLayout *layout = new QGridLayout;

  setWindowTitle("Lieferantensuche");
  sWahl="";

  bOk.setText("&Ok");
  bOk.setAutoDefault(true);
  bCancel.setText("&Abbruch");
  bCancel.setAutoDefault(false);
  bAnlegen.setText("A&nlegen");
  bAnlegen.setAutoDefault(false);
  wName.setFixedSize(525,25);
  wPlz.setFixedSize(95,25);
  l=new QLabel("Nachname");
  layout->addWidget(l,1,1);
  layout->addWidget(&wName,1,2,1,5);
  l=new QLabel("PLZ.");
  layout->addWidget(l,2,1);
  layout->addWidget(&wPlz,2,2);
  if (flag_anlegen) layout->addWidget(&bAnlegen,2,4);
  layout->addWidget(&bOk,2,5);
  layout->addWidget(&bCancel,2,6);
  layout->setColumnStretch(3,1);
  layout->setColumnStretch(7,1);

  wModel.setColumnCount(3);
  wListe.setModel(&wModel);
  wListe.setEnabled(true);
  wListe.setFixedSize(750,300);
  pInitListe();
  layout->addWidget(&wListe,3,1,5,6);

  setLayout(layout);

  connect(&bCancel,SIGNAL(clicked()),this,SLOT(reject()));
  connect(&bOk,SIGNAL(clicked()),this,SLOT(pSuche()));
  connect(&bAnlegen,SIGNAL(clicked()),this,SLOT(pAnlegen()));
  connect(&wListe,SIGNAL(clicked(QModelIndex)),this,SLOT(pAuswahl(QModelIndex)));
}

LsSuche::~LsSuche()
{
  return;
}

int LsSuche::exec(QString suchString)
{
  int r;
  int flag_suche=0;
  QString s1=suchString.simplified().toUpper();
  QChar   wAnfang=s1[0];

  sWahl="";
  wName.setText("");
  wPlz.setText("");
  if (s1.right(1) == "?")
  {
    r=s1.length();
    if (wAnfang.isDigit()) { wPlz.setText(s1.left(r-1)); flag_suche=2; }
    else                   { wName.setText(s1.left(r-1)); flag_suche=1; }
  }
  else if (wAnfang == 'N') { wName.setText(s1.mid(1)); flag_suche=1; }
  else if (wAnfang == 'P') { wPlz.setText(s1.mid(1)); flag_suche=2; }
  if (flag_suche) pSuche();
  r=QDialog::exec();
  return(r);
}

void LsSuche::pSuche()
{
  QStandardItem *wItem;
  QSqlQuery      query;
  QString   s,sName,sPlz,sNameVgl,sPlz1,sPlz2;
  bool      flag_plz,flag_name;
  int       i,wAnz;
  
  pInitListe();
  flag_name=flag_plz=false;
  sName=wName.text().toUpper().replace("'","`").simplified();
  sPlz=wPlz.text().toUpper().replace("'","`").simplified();
  if (sName.length()>0)
  {
    flag_name=true;
    sNameVgl=SuchName(sName,"");
  }
  if (sPlz.length()>0)
  {
    flag_plz=true;
    sPlz1=sPlz;
    sPlz2=sPlz1 + "\xff";
  }
  s="SELECT key,name1,name2,plz,ort,suchname FROM adresse WHERE ";
  if (flag_plz)
     s=s+"plz >= '" + sPlz1 + "' AND plz <= '" + sPlz2 + "' AND ";
  s=s + "art = 'L' ORDER BY";
  if (!flag_plz ||
      (flag_plz && sPlz1.length() < sName.length()))
     s=s+" suchname,name1,name2";
  else if (flag_plz)
     s=s+" plz,name1,name2";
  else
     s=s+" art,key";
  query.exec(s);
  
  for (i=wAnz=0; query.next(); i++)
  {
    if (flag_name && !query.value(5).toString().contains(sNameVgl)) continue;

    wItem=new QStandardItem(query.value(0).toString());
    wItem->setEditable(false),
    wModel.setItem(wAnz,0,wItem);
    wItem=new QStandardItem(query.value(1).toString() + ", " +
                            query.value(2).toString());
    wItem->setEditable(false),
    wModel.setItem(wAnz,1,wItem);
    wItem=new QStandardItem(query.value(3).toString() + " " +
                            query.value(4).toString());
    wItem->setEditable(false),
    wModel.setItem(wAnz,2,wItem);
    wAnz++;
  }

  wModel.setRowCount(wAnz);
  return;
}

void LsSuche::pAuswahl(QModelIndex i)
{
  int wZeile = i.row();

  sWahl=wModel.item(wZeile,0)->text();
  accept();
}

QString LsSuche::auswahl()
{
  return(sWahl);
}

void LsSuche::pInitListe()
{
  wModel.clear();
  wModel.setHorizontalHeaderLabels(QString("Bez.,Name, PLZ/Ort").split(","));
  wListe.setColumnWidth(0,95);
  wListe.setColumnWidth(1,300);
  wListe.setColumnWidth(2,300);
}

void LsSuche::pAnlegen()
{
  int r;
  Lieferant pDiag(true);
  if (pDiag.exec() == QDialog::Accepted)
  {
    sWahl=pDiag.angelegt();
    accept();
  }
}
