#include <QtSql>
#include <QtGui>
#include "wadresse.h"
#include "wbank.h"
#include "utils.h"
#include "midimport.h"

MIDImport::MIDImport(QWidget *parent) : QDialog(parent)
{
  QGridLayout *wLayout;
  int y;

  setWindowTitle("Import Mitgliederdatei");
  wLayout = new QGridLayout();
  wDatei.setFlags(true,true);
  bOk.setText("&Ok");
  bCancel.setText("&Abbruch");
  lDisp.setFixedSize(100,25);
  y=wDatei.FuellLayout(wLayout,1,1);
  wDatei.setFlags(true,true);
  wLayout->addWidget(&bOk,y,1);
  wLayout->addWidget(&bCancel,y,2,1,2);
  wLayout->addWidget(&lDisp,y,4,1,2);
  setLayout(wLayout);

  connect(&bOk,SIGNAL(clicked()),this,SLOT(pImport()));
  connect(&bCancel,SIGNAL(clicked()),this,SLOT(reject()));
}

MIDImport::~MIDImport()
{
  return;
}

void MIDImport::pImport()
{
  QSqlQuery   query,query2;
  QString     s,s2,cMid;
  WAdresse    wAdresse(1,NULL);
  WBank       wBank(NULL);
  QStringList wGlList;
  int         i,z1,anzRead,anzNew,anzDel,anzUpd;
  int imid, iintnr, isex,ination,itel,igebdat,izahlart,ibeitrag;
  int ieintritt,iaustritt,iktoinhaber,iktonummer,iktoblz,iktobank,iktobic;
  int iktoiban,ivname,inname,istrasse,iplz,iort,iland,imail;
  int iovb,ikvb,ibzvb,ilvb,ibezahlt,ierfasst;
  QString smid,sintnr,sgliederung,ssex,snation,stel,sgebdat,szahlart,sbeitrag;
  QString seintritt,saustritt,sktoinhaber,sktonummer,sktoblz,sktobank,sktobic;
  QString sktoiban,svname,snname,sstrasse,splz,sort,sland,smail;
  QString sovb,skvb,sbzvb,slvb,sbezahlt,swohngliederung;

  if (!wDatei.isValid()) return;

  imid=iintnr=isex=ination=itel=igebdat=izahlart=ibeitrag=-1;
  ieintritt=iaustritt=iktoinhaber=iktonummer=iktoblz=iktobank=iktobic=-1;
  iktoiban=ivname=inname=istrasse=iplz=iort=iland=imail=-1;
  iovb=ikvb=ibzvb=ilvb=ibezahlt=ierfasst=-1;
  anzRead=anzNew=anzDel=anzUpd=0;

  if (!wDatei.startLine()) return;

  for (i=0; !wDatei.eol(); i++)
  {
    s=SuchName(wDatei.readField(),"");
    if      (s=="INTERNEKONTAKTID")                            iintnr=i;
    else if (s=="INTERNALCONTACTID")                           iintnr=i;
    else if (s=="EXTERNEIDFENTIFIKATIONMATCHTOCONTACT")        imid=i;
    else if (s=="MITGLIEDSNR")                                 imid=i;
    else if (s=="MITGLIEDSNUMER")                              imid=i;
    else if (s=="EXTERNALIDENTIFIERMATCHTOCONTACT")            imid=i;
    else if (s=="VORNAME")                                     ivname=i;
    else if (s=="FIRSTNAME")                                   ivname=i;
    else if (s=="NACHNAME")                                    inname=i;
    else if (s=="LASTNAME")                                    inname=i;
    else if (s=="GEBURTSDATUM")                                igebdat=i;
    else if (s=="GEBDATUM")                                    igebdat=i;
    else if (s=="BIRTHDATE")                                   igebdat=i;
    else if (s=="GENDER")                                      isex=i;
    else if (s=="STRASE")                                      istrasse=i;
    else if (s=="STRETADRES")                                  istrasse=i;
    else if (s=="PLZ")                                         iplz=i;
    else if (s=="POSTALCODE")                                  iplz=i;
    else if (s=="STADT")                                       iort=i;
    else if (s=="CITI")                                        iort=i;
    else if (s=="LAND")                                        iland=i;
    else if (s=="COUNTI")                                      iland=i;
    else if (s=="STATSBURGERSCHAFT")                           ination=i;
    else if (s=="TELEFON")                                     itel=i;
    else if (s=="PHONE")                                       itel=i;
    else if (s=="EMAIL")                                       imail=i;
    else if (s=="EINZUGSERMACHTIGUNG")                         izahlart=i;
    else if (s=="KONTOINHABER")                                iktoinhaber=i;
    else if (s=="KONTONUMER")                                  iktonummer=i;
    else if (s=="BANKLEITZAHL")                                iktoblz=i;
    else if (s=="KREDITINSTITUT")                              iktobank=i;
    else if (s=="GESAMTBEITRAG")                               ibeitrag=i;
    else if (s=="EINTRITSDATUM")                               ieintritt=i;
    else if (s=="AUSTRITSDATUM")                               iaustritt=i;
    else if (s=="GEMEINDE")                                    iovb=i;
    else if (s=="LANDKREIS")                                   ikvb=i;
    else if (s=="REGIERUNGSBEZIRK")                            ibzvb=i;
    else if (s=="BUNDESLAND")                                  ilvb=i;
    else if (s=="DATENSATZANGELEGTAM")                         ierfasst=i;
  }

  if (imid<0 || inname<0)
  {
    QMessageBox::critical(NULL,"Fehler bei Import","Datei hat falsches Format");
    return;
  }
  if (ieintritt<0) ieintritt=ierfasst;

  if (iktoblz>=0 && iktonummer>=0)
  {
    if (iktobic<0)  iktobic=9999;
    if (iktoiban<0) iktoiban=9999;
    if (iktoinhaber<0) iktoinhaber=9999;
  }
  if (iktobic>=0 && iktoiban>=0)
  {
    if (iktonummer<0)  iktonummer=9999;
    if (iktoblz<0)     iktoblz=9999;
    if (iktobank<0)    iktobank=9999;
    if (iktoinhaber<0) iktoinhaber=9999;
  }

  wGlList.append(UtilSelbst());
  for (i=0; i<wGlList.size(); i++)
  {
    query.exec("SELECT namekurz FROM verband WHERE teilvon='" +
              wGlList[i] + "'");
    while(query.next())
    {
      if (!wGlList.contains(query.value(0).toString()))
         wGlList.append(query.value(0).toString());
    }
    query.finish();
  }

  query.exec("CREATE TEMPORARY TABLE zwimid (mid int, PRIMARY KEY(mid ASC))");
  query.exec("INSERT INTO zwimid (mid) SELECT mid FROM mitglied WHERE mid<'1000000' AND austritt>'" + wDatei.creationDate().toString("yyyy-MM-dd") + "'");

  z1=0;
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
  while(wDatei.startLine())
  {
    anzRead++;
    for (i=0; !wDatei.eol(); i++)
    {
      s=wDatei.readField().simplified().replace("'","`");
      if (i==imid)        smid=QString::number(s.toInt());
      if (i==iintnr)      sintnr=QString::number(s.toInt());
      if (i==isex)        {
                             s=s.toUpper().left(1);
                             if (s=="M" || s=="W")
                                ssex=s;
                             else
                                ssex="";
                          }
      if (i==ination)     snation=s;
      if (i==itel)        stel=s;
      if (i==igebdat)     sgebdat=wDatei.toDate(s,QDate(1900,1,1)).toString("yyyy-MM-dd");
      if (i==izahlart)    {
                            s=s.toUpper();
                            if (s=="YES" || s=="JA")
                               szahlart="L";
                            else
                               szahlart="V";
                          }
      if (i==ibeitrag)    sbeitrag=QString::number((qint64)round(s.toDouble() *
                                                   (double)100.0l));
      if (i==ibezahlt)    sbezahlt=wDatei.toDate(s,QDate(1900,1,1)).toString("yyyy-MM-dd");
      if (i==ieintritt)   seintritt=wDatei.toDate(s,QDate(1900,1,1)).toString("yyyy-MM-dd");
      if (i==iaustritt)   saustritt=wDatei.toDate(s,QDate(2200,1,1)).toString("yyyy-MM-dd");
      if (i==iktoinhaber) sktoinhaber=s;
      if (i==iktonummer)  sktonummer=s;
      if (i==iktoblz)     sktoblz=s;
      if (i==iktobank)    sktobank=s;
      if (i==iktobic)     sktobic=s;
      if (i==iktoiban)    sktoiban=s;
      if (i==ivname)      svname=s;
      if (i==inname)      snname=s;
      if (i==istrasse)    sstrasse=s;
      if (i==iplz)        splz=s;
      if (i==iort)        sort=s;
      if (i==iland)       sland=s;
      if (i==imail)       smail=s;
      if (i==ilvb)        slvb=s.toUpper();
      if (i==ibzvb)       sbzvb=s.toUpper();
      if (i==ikvb)        skvb=s.toUpper();
      if (i==iovb)        sovb=s.toUpper();
    }
    z1++;
    if (z1>5)
    {
      lDisp.setText(smid);
      QCoreApplication::processEvents();
      z1=0;
    }
    swohngliederung="";
    if      (iovb>=0  && sovb!="" && wGlList.contains(sovb))  sgliederung=sovb;
    else if (ikvb>=0  && skvb!="" && wGlList.contains(skvb))  sgliederung=skvb;
    else if (ibzvb>=0 && sbzvb!="" && wGlList.contains(sbzvb))sgliederung=sbzvb;
    else if (ilvb>=0  && slvb!="" && wGlList.contains(slvb))  sgliederung=slvb;
    else
    {
      sgliederung=UtilSelbst();
      if      (iovb>=0  && sovb!="")  swohngliederung=sovb;
      else if (ikvb>=0  && skvb!="")  swohngliederung=skvb;
      else if (ibzvb>=0 && sbzvb!="") swohngliederung=sbzvb;
      else if (ilvb>=0  && slvb!="")  swohngliederung=slvb;
    }
    if (szahlart=="L" && (sktoblz=="" || sktonummer=="") &&
        (sktobic=="" || sktoiban=="") )
       szahlart="V";
    if (szahlart=="L" && sktoinhaber=="")
       sktoinhaber=svname + " " + snname;
    cMid=smid.sprintf("%08i",smid.toInt());
    wAdresse.read("M",cMid);
    wBank.read("M",cMid);
    query.exec("START TRANSACTION");
    s="";
    if (iintnr>=0)      s=s+",intnr='"          + sintnr          + "'";
                        s=s+",gliederung='"     + sgliederung     + "'";
                        s=s+",wohngliederung='" + swohngliederung + "'";
    if (isex>=0)        s=s+",sex='"            + ssex            + "'";
    if (ination>=0)     s=s+",nation='"         + snation         + "'";
    if (itel>=0)        s=s+",tel='"            + stel            + "'";
    if (igebdat>=0)     s=s+",gebdat='"         + sgebdat         + "'";
    // if (izahlart>=0)    s=s+",zahlart='"    + szahlart    + "'";
    // if (ibeitrag>=0)    s=s+",beitrag='"    + sbeitrag    + "'";
    // if (ibezahlt>=0)    s=s+",bezahlt='"    + sbezahlt    + "'";
    if (ieintritt>=0)   s=s+",eintritt='"   + seintritt   + "'";
    if (iaustritt>=0)   s=s+",austritt='"   + saustritt   + "'";
    else                s=s+",austritt='2200-01-01'";
    query.exec("UPDATE mitglied SET " + s.mid(1) + "WHERE mid='" + smid + "'");
    if (query.numRowsAffected()>0)
       anzUpd++;
    else
    {
      s="mid";
      s2="'" + smid + "'";
      if (iintnr>=0)      s=s+",intnr";
                          s=s+",gliederung,wohngliederung";
      if (isex>=0)        s=s+",sex";
      if (ination>=0)     s=s+",nation";
      if (itel>=0)        s=s+",tel";
      if (igebdat>=0)     s=s+",gebdat";
      if (izahlart>=0)    s=s+",zahlart";
      if (ibeitrag>=0)    s=s+",beitrag";
      s=s+",eintritt";
      s=s+",austritt";
      s=s+",bezahlt";
      s=s+",zahlstatus";
      s=s+",statusdatum";
      if (iintnr>=0)      s2=s2+",'" + sintnr + "'";
                          s2=s2+",'" + sgliederung +
                               "','" + swohngliederung + "'";
      if (isex>=0)        s2=s2+",'" + ssex + "'";
      if (ination>=0)     s2=s2+",'" + snation + "'";
      if (itel>=0)        s2=s2+",'" + stel + "'";
      if (igebdat>=0)     s2=s2+",'" + sgebdat + "'";
      if (izahlart>=0)    s2=s2+",'" + szahlart + "'";
      if (ibeitrag>=0)    s2=s2+",'" + sbeitrag + "'";
      if (ieintritt>=0)   s2=s2+",'" + seintritt + "'";
      else                s2=s2+",'" + wDatei.creationDate().toString("yyyy-MM-dd") + "'";
      if (iaustritt>=0)   s2=s2+",'" + saustritt + "'";
      else                s2=s2+",'2200-01-01'";
      if (ibezahlt>=0)    s2=s2+",'" + sbezahlt + "'";
      else if (ieintritt>=0) s2=s2+",'" + seintritt + "'";
      else                   s2=s2+",'" + wDatei.creationDate().toString("yyyy-MM-dd") + "'";
      s2=s2+",'N','" + wDatei.creationDate().toString("yyyy-MM-dd") + "'";
      query.exec("INSERT INTO mitglied (" + s + ") VALUES (" + s2 + ")");
      if (query.numRowsAffected()>0)
         anzNew++;
    }
    if (ivname>=0)   wAdresse.Name2()->setText(svname);
    if (inname>=0)   wAdresse.Name1()->setText(snname);
    if (istrasse>=0) wAdresse.Strasse()->setText(sstrasse);
    if (iplz>=0)     wAdresse.Plz()->setText(splz);
    if (iort>=0)     wAdresse.Ort()->setText(sort);
    if (iland>=0)    wAdresse.Land()->setText(sland);
    if (imail>=0)    wAdresse.Mail()->setText(smail);
    if (iktoinhaber>=0) wBank.KontoInh()->setText(sktoinhaber);
    if (iktonummer>=0)  wBank.KontoNr()->setText(sktonummer);
    if (iktoblz>=0)     wBank.KontoBlz()->setText(sktoblz);
    if (iktobank>=0)    wBank.KontoBank()->setText(sktobank);
    if (iktobic>=0)     wBank.KontoBic()->setText(sktobic);
    if (iktoiban>=0)    wBank.KontoIban()->setText(sktoiban);
    wAdresse.write("M",cMid); wAdresse.clear();
    wBank.write("M",cMid);    wBank.clear();
    query.exec("DELETE FROM zwimid WHERE mid='" + smid + "'");
    query.exec("COMMIT TRANSACTION");
  }
  lDisp.setText("");
  query.exec("SELECT count(*) FROM zwimid");
  if (query.next())
  {
    anzDel=query.value(0).toInt();
    query.finish();
  }
  QApplication::restoreOverrideCursor();
  if (anzRead<1)
  {
    QMessageBox::critical(NULL,"Fehler bei Import","Keine Daten gefunden");
  }
  else
  {
    s=QString::number(anzRead) + " Stze gelesen";
    if (anzNew>0) s=s+"\n" + QString::number(anzNew) + " Stze neu";
    if (anzUpd>0) s=s+"\n" + QString::number(anzUpd) + " Stze aktualisiert";
    if (anzDel>0) s=s+"\n" + QString::number(anzDel) + " Stze nicht mehr vorhanden";
    if (anzDel>0)
    {
      s=s+"\nNicht mehr vorhandene Mitglieder als ausgetreten markieren?";
      if (QMessageBox::question(NULL,"Import abgeschlossen",s,
          QMessageBox::Yes,QMessageBox::No)
          == QMessageBox::Yes)
      {
        z1=0;
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
        query.exec("SELECT mid FROM zwimid");
        while(query.next())
        {
          if (z1>9) { QCoreApplication::processEvents(); z1=0; }
          query2.exec("UPDATE mitglied SET austritt='" +
                      wDatei.creationDate().toString("yyyy-MM-dd") +
                      "' WHERE mid = '" + query.value(0).toString() + "'");
        }
        QApplication::restoreOverrideCursor();
      }
    }
    else
    {
      QMessageBox::information(NULL,"Import abgeschlossen",s);
    }
  }
  query.exec("DROP TABLE zwimid");
  accept();
  return;
}
