#include <QtGui>
#include <QtSql>

#include "mitglieder.h"
#include "zuwenddruck.h"
#include "druckfkt.h"
#include "utils.h"

Mitglieder::Mitglieder(bool flag_anlegen, QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  int    y;

  setWindowTitle("Mitglieder");
  fAnlegen = flag_anlegen;
  iNeuNummer = -1;

  QVBoxLayout *topLayout = new QVBoxLayout();
  QHBoxLayout *bezLayout = new QHBoxLayout();
  QHBoxLayout *buttonLayout = new QHBoxLayout();
  QGridLayout *adrLayout = new QGridLayout();
  QGridLayout *bankLayout = new QGridLayout();
  QTabWidget  *wTabWidget = new QTabWidget();
  QWidget     *wTabAdr    = new QWidget();
  QWidget     *wTabBank   = new QWidget();

  wTabWidget->addTab(wTabAdr,"A&dresse");
  wTabWidget->addTab(wTabBank,"&Verwaltung");

  wMid=new WMitglied(false,this);
  wIntern=new QLineEdit(this); wIntern->setFixedSize(95,25);
  wIntern->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
  wSex=new QComboBox(this);
  wGebdat=new QDateEdit(); wGebdat->setDisplayFormat("dd.MM.yyyy");
  wAdresse=new WAdresse(1,this);
  wTel=new QLineEdit(this);    wTel->setFixedSize(300,25);
  wNation=new QLineEdit(this);    wTel->setFixedSize(300,25);
  wZahlart=new QComboBox(this);
  wBank   =new WBank(this);
  wBeitrag = new QDoubleSpinBox;
  wBeitrag->setDecimals(2);
  wBeitrag->setMinimum(-99999999.99);
  wBeitrag->setMaximum( 99999999.99);
  wBeitrag->setSuffix(" EUR");
  wBeitrag->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
  wBeitrag->setFixedSize(150,25);
  wGliederung    =new WGliederung(this); wGliederung->setFixedSize(350,25);
  wWohnGliederung=new WGliederung(this); wWohnGliederung->setFixedSize(350,25);
  wBezahlt=new QDateEdit(); wBezahlt->setDisplayFormat("dd.MM.yyyy");
  wBezahlt->setFixedSize(wBezahlt->sizeHint());
  wEintritt=new QDateEdit(); wEintritt->setDisplayFormat("dd.MM.yyyy");
  wEintritt->setFixedSize(wEintritt->sizeHint());
  wAustritt=new QDateEdit(); wAustritt->setDisplayFormat("dd.MM.yyyy");
  wAustritt->setFixedSize(wAustritt->sizeHint());
  wZahlStatus=new QComboBox(this);
  wStatusDatum=new QDateEdit(); wStatusDatum->setDisplayFormat("dd.MM.yyyy");
  wStatusDatum->setFixedSize(wAustritt->sizeHint());

  wSex->insertItem(0,"");
  wSex->insertItem(1,"Mnnlich");
  wSex->insertItem(2,"Weiblich");
  wSex->insertItem(3,"Firma");
  wSex->setFixedSize(wSex->sizeHint());
  wZahlart->insertItem(0,"");
  wZahlart->insertItem(1,"Vorkasse");
  wZahlart->insertItem(2,"Lastschrift");
  wZahlart->insertItem(3,"Lastschrift geplatzt");
  wZahlart->setFixedSize(wZahlart->sizeHint());
  wZahlStatus->insertItem(0,"");
  wZahlStatus->insertItem(1,"Neu angelegt");
  wZahlStatus->insertItem(2,"Rechnung geschrieben");
  wZahlStatus->insertItem(3,"1. Mahnung");
  wZahlStatus->insertItem(4,"2. Mahnung");
  wZahlStatus->insertItem(5,"3. Mahnung");
  wZahlStatus->insertItem(6,"Lastschrift gestartet");
  wZahlStatus->insertItem(7,"Gesperrt");
  wZahlStatus->setFixedSize(wZahlStatus->sizeHint());

  bOk=new QPushButton("&Speichern");
  bCancel=new QPushButton("&Abbruch");
  bOk->setAutoDefault(false);
  bCancel->setAutoDefault(false);
  if (!fAnlegen)
  {
     bDel=new QPushButton("&Lschen");
     bDel->setAutoDefault(false);
  }
  bZuwend=new QPushButton("&Zuwendungsbest.");
  bZuwend->setAutoDefault(false);
  bMitglied=new QPushButton("-> &Mitglied");
  bMitglied->setAutoDefault(false);

  wListe = new QTableView;
  wModel = new QStandardItemModel;
  wModel->setColumnCount(4);
  wListe->setModel(wModel);
  wListe->setEnabled(true);
  wListe->setFixedSize(800,200);

  topLayout->addLayout(bezLayout);
  topLayout->addWidget(wTabWidget);
  topLayout->addLayout(buttonLayout);
  topLayout->addWidget(wListe);

  l=new QLabel("Mitgliedsnummer");
  bezLayout->addWidget(l);
  bezLayout->addWidget(wMid);
  l=new QLabel("Intern");
  bezLayout->addWidget(l);
  bezLayout->addWidget(wIntern);
  bezLayout->addStretch(1);

  l=new QLabel("Geschlecht");
  adrLayout->addWidget(l,1,1);
  adrLayout->addWidget(wSex,1,2);
  l=new QLabel("Geburtsdatum");
  adrLayout->addWidget(l,1,3);
  adrLayout->addWidget(wGebdat,1,4);
  wAdresse->FuellLayout(adrLayout,2,1);
  l=new QLabel("Telefon");
  adrLayout->addWidget(l,8,1);
  adrLayout->addWidget(wTel,8,2,1,3);
  l=new QLabel("Nationalitt");
  adrLayout->addWidget(l,9,1);
  adrLayout->addWidget(wNation,9,2,1,3);
  adrLayout->setRowStretch(99,1);
  adrLayout->setColumnStretch(99,1);

  y=1;
  l=new QLabel("Gliederung");
  bankLayout->addWidget(l,y,1);
  bankLayout->addWidget(wGliederung,y,2,1,3);
  y++;
  l=new QLabel("- Wohnort");
  bankLayout->addWidget(l,y,1);
  bankLayout->addWidget(wWohnGliederung,y,2,1,3);
  y++;
  l=new QLabel("Mitgliedsbeitrag");
  bankLayout->addWidget(l,y,1);
  bankLayout->addWidget(wBeitrag,y,2);
  l=new QLabel("Bezahlt");
  bankLayout->addWidget(l,y,3);
  bankLayout->addWidget(wBezahlt,y,4);
  y++;
  l=new QLabel("Eintritt");
  bankLayout->addWidget(l,y,1);
  bankLayout->addWidget(wEintritt,y,2);
  l=new QLabel("Austritt");
  bankLayout->addWidget(l,y,3);
  bankLayout->addWidget(wAustritt,y,4);
  y++;
  l=new QLabel("Zahlungsart");
  bankLayout->addWidget(l,y,1);
  bankLayout->addWidget(wZahlart,y,2);
  l=new QLabel("Datum");
  bankLayout->addWidget(l,y,3);
  bankLayout->addWidget(wStatusDatum,y,4);
  l=new QLabel("Zahlungsstatus");
  bankLayout->addWidget(l,y,5);
  bankLayout->addWidget(wZahlStatus,y,6);
  y++;
  wBank->FuellLayout(bankLayout,y,1);
  bankLayout->setColumnStretch(4,1);
  bankLayout->setRowStretch(99,1);
  bankLayout->setColumnStretch(99,1);

  buttonLayout->addWidget(bOk);
  buttonLayout->addWidget(bCancel);
  if (!fAnlegen) buttonLayout->addWidget(bDel);
  buttonLayout->addStretch(1);
  buttonLayout->addWidget(bZuwend);
  buttonLayout->addWidget(bMitglied);

  wTabAdr->setLayout(adrLayout);
  wTabBank->setLayout(bankLayout);
  setLayout(topLayout);

  pSetWahl(true);

  connect(wMid,SIGNAL(finished()),this,SLOT(pRead()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(pCancel()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pWrite()));
  connect(bZuwend,SIGNAL(clicked()),this,SLOT(pZuwend()));
  connect(bMitglied,SIGNAL(clicked()),this,SLOT(pMitglied()));
  if (!fAnlegen)
     connect(bDel,SIGNAL(clicked()),this,SLOT(pDelete()));
}

Mitglieder::~Mitglieder()
{
  delete wMid;
  delete wIntern;
  delete wSex;
  delete wGebdat;
  delete wAdresse;
  delete wTel;
  delete wNation;
  delete wZahlart;
  delete wBank;
  delete wBeitrag;
  delete wGliederung,
  delete wWohnGliederung;
  delete wBezahlt;
  delete wEintritt;
  delete wAustritt;
  delete wZahlStatus;
  delete wStatusDatum;
  delete bOk;
  delete bCancel;
  if (!fAnlegen) delete bDel;
  delete bZuwend;
  delete bMitglied;
  delete wListe;
  delete wModel;
  return;
}

void Mitglieder::pSetWahl(bool flag_wahl)
{
  bool flag_wahl1;
  fWahl=flag_wahl;

  wMid->setEnabled(flag_wahl);
  iMid=wMid->getInt();
  if (iMid>999999) flag_wahl1=true;
  else             flag_wahl1=flag_wahl;

  wAdresse->setEnabled(!flag_wahl);
  wBank->setEnabled(!flag_wahl);
  wIntern->setEnabled(!flag_wahl);
  wTel->setEnabled(!flag_wahl);
  wSex->setEnabled(!flag_wahl);
  wGebdat->setEnabled(!flag_wahl);
  wGliederung->setEnabled(!flag_wahl);
  wWohnGliederung->setEnabled(!flag_wahl);
  wNation->setEnabled(!flag_wahl);

  wZahlart->setEnabled(!flag_wahl1);
  wZahlStatus->setEnabled(!flag_wahl1);
  wStatusDatum->setEnabled(!flag_wahl1);
  wBeitrag->setEnabled(!flag_wahl1);
  wBezahlt->setEnabled(!flag_wahl1);
  wEintritt->setEnabled(!flag_wahl1);
  wAustritt->setEnabled(!flag_wahl1);

  bOk->setEnabled(!flag_wahl);
  bCancel->setEnabled(!flag_wahl);
  bZuwend->setEnabled(!flag_wahl);
  if (!fAnlegen) 
    bDel->setEnabled(!flag_wahl);
  if (!flag_wahl && iMid>999999 && !fAnlegen)
  {
    bMitglied->setEnabled(true);
    bMitglied->show();
  }
  else
  {
    bMitglied->setEnabled(false);
    bMitglied->hide();
  }

  if (flag_wahl)
  {
    wAdresse->clear();
    wBank->clear();
    wMid->clear();
    wIntern->clear();
    wTel->clear();
    wNation->clear();
    wGliederung->setText("");
    wWohnGliederung->setText("");
    wZahlart->setCurrentIndex(0);
    wZahlStatus->setCurrentIndex(0);
    wSex->setCurrentIndex(0);
    wBeitrag->setValue(0);
    wGebdat->setDate(QDate(1900,1,1));
    wBezahlt->setDate(QDate(1900,1,1));
    wEintritt->setDate(QDate(1900,1,1));
    wStatusDatum->setDate(QDate::currentDate());
    wAustritt->setDate(QDate(2200,1,1));
  }
  pFuellListe();

  if (flag_wahl)
    wMid->setFocus();
  else if (wAdresse->isEmpty())
    wIntern->setFocus();
  else
    bCancel->setFocus();
}

void Mitglieder::pFelder()
{
  iMid=wMid->getInt();
  cMid=wMid->getKey();
  sMid=QString::number(iMid);
  iIntern=wIntern->text().simplified().toInt(); sIntern=QString::number(iIntern);
  iBeitrag=(qint64)round(wBeitrag->value() * (double)(100.0l));
  sBeitrag=QString::number(iBeitrag);
  sTel=wTel->text().replace("'","`").simplified();
  sGliederung=wGliederung->text().replace("'","`").simplified();
  sWohnGliederung=wWohnGliederung->text().replace("'","`").simplified();
  if (sWohnGliederung==sGliederung) sWohnGliederung="";
  sNation=wNation->text().replace("'","`").simplified();
  sGebdat=wGebdat->date().toString("yyyy-MM-dd");
  sBezahlt=wBezahlt->date().toString("yyyy-MM-dd");
  sEintritt=wEintritt->date().toString("yyyy-MM-dd");
  sAustritt=wAustritt->date().toString("yyyy-MM-dd");
  sStatusDatum=wStatusDatum->date().toString("yyyy-MM-dd");
  switch(wSex->currentIndex())
  {
    case 1 : sSex="M"; break;
    case 2 : sSex="W"; break;
    case 3 : sSex="U"; break;
    default: sSex=" "; break;
  }
  switch(wZahlart->currentIndex())
  {
    case 1 : sZahlart="V"; break;
    case 2 : sZahlart="L"; break;
    case 3 : sZahlart="G"; break;
    default: sZahlart=" "; break;
  }
  switch(wZahlStatus->currentIndex())
  {
    case 1 : sZahlStatus="N"; break;
    case 2 : sZahlStatus="R"; break;
    case 3 : sZahlStatus="1"; break;
    case 4 : sZahlStatus="2"; break;
    case 5 : sZahlStatus="3"; break;
    case 6 : sZahlStatus="L"; break;
    case 7 : sZahlStatus="X"; break;
    default: sZahlStatus=" "; break;
  }
  return;
}

void Mitglieder::pRead()
{
  QSqlQuery query;

  if (!fWahl) return;
  pFelder();
  if (iMid<1 || iMid>9999999) return;
  wMid->setText(sMid);

  if (iMid<1000000)
  {
    wZahlStatus->setCurrentIndex(1);
    wStatusDatum->setDate(QDate::currentDate());
    wEintritt->setDate(QDate::currentDate());
    wBezahlt->setDate(QDate::currentDate());
    wBeitrag->setValue(36.0l);
  }
  query.exec("SELECT intnr,gliederung,sex,nation,tel,gebdat,zahlart,beitrag,eintritt,austritt,bezahlt,zahlstatus,statusdatum,wohngliederung FROM mitglied WHERE mid='" + sMid + "'");
  if (query.next())
  {
    if (iMid>9999999) { query.finish(); return; }
    wIntern->setText(query.value(0).toString());
    wGliederung->setText(query.value(1).toString());
    if      (query.value(2).toString() == "M") wSex->setCurrentIndex(1);
    else if (query.value(2).toString() == "W") wSex->setCurrentIndex(2);
    else if (query.value(2).toString() == "U") wSex->setCurrentIndex(3);
    else                                       wSex->setCurrentIndex(0);
    wNation->setText(query.value(3).toString());
    wTel->setText(query.value(4).toString());
    wGebdat->setDate(QDate::fromString(query.value(5).toString(),"yyyy-MM-dd"));
    if      (query.value(6).toString() == "V") wZahlart->setCurrentIndex(1);
    else if (query.value(6).toString() == "L") wZahlart->setCurrentIndex(2);
    else if (query.value(6).toString() == "G") wZahlart->setCurrentIndex(3);
    else                                       wZahlart->setCurrentIndex(0);
    wBeitrag->setValue(query.value(7).toDouble() / (double)100.0l);
    wEintritt->setDate(QDate::fromString(query.value(8).toString(),"yyyy-MM-dd"));
    wAustritt->setDate(QDate::fromString(query.value(9).toString(),"yyyy-MM-dd"));
    wBezahlt->setDate(QDate::fromString(query.value(10).toString(),"yyyy-MM-dd"));
    if      (query.value(11).toString() == "N") wZahlStatus->setCurrentIndex(1);
    else if (query.value(11).toString() == "R") wZahlStatus->setCurrentIndex(2);
    else if (query.value(11).toString() == "1") wZahlStatus->setCurrentIndex(3);
    else if (query.value(11).toString() == "2") wZahlStatus->setCurrentIndex(4);
    else if (query.value(11).toString() == "3") wZahlStatus->setCurrentIndex(5);
    else if (query.value(11).toString() == "L") wZahlStatus->setCurrentIndex(6);
    else if (query.value(11).toString() == "X") wZahlStatus->setCurrentIndex(7);
    else                                        wZahlStatus->setCurrentIndex(0);
    wStatusDatum->setDate(QDate::fromString(query.value(12).toString(),"yyyy-MM-dd"));
    wWohnGliederung->setText(query.value(13).toString());
  }
  wAdresse->read("M",cMid);
  wBank->read("M",cMid);
  
  pSetWahl(false);
}

void Mitglieder::pCancel()
{
  if (fWahl || fAnlegen)
    reject();
  else
  {
    pSetWahl(true);
  }
}

void Mitglieder::pWrite()
{
  QSqlQuery query;
  QString q;

  if (fWahl) return;
  pFelder();
  if (wAdresse->Name1()->text().simplified().length()<1) return;
  iNeuNummer=iMid;

  query.exec("START TRANSACTION");

  if (fAnlegen)
  {
    query.exec("SELECT mid FROM mitglied ORDER BY mid DESC LIMIT 1");
    if (query.next())
       iMid=query.value(0).toInt() + 1;
    else
       iMid=1000000;
    query.finish();
    if (iMid<1000000) iMid=1000000;
    sMid=QString::number(iMid);
    wMid->setText(sMid);
    cMid=wMid->getKey();
  }
  iNeuNummer=iMid;

  query.exec("UPDATE mitglied SET intnr='" + sIntern +
             "',gliederung='"     + sGliederung +
             "',wohngliederung='" + sWohnGliederung +
             "',sex='"            + sSex +
             "',nation='"         + sNation +
             "',tel='"            + sTel +
             "',gebdat='"         + sGebdat +
             "',zahlart='"        + sZahlart +
             "',beitrag='"        + sBeitrag +
             "',eintritt='"       + sEintritt +
             "',austritt='"       + sAustritt +
             "',bezahlt='"        + sBezahlt +
             "',zahlstatus='"     + sZahlStatus +
             "',statusdatum='"    + sStatusDatum +
             "' WHERE mid='" + sMid + "'");
  if (query.numRowsAffected()<1)
  {
    query.exec("INSERT INTO mitglied (mid,intnr,gliederung,wohngliederung,sex,nation,tel,gebdat,zahlart,beitrag,eintritt,austritt,bezahlt,zahlstatus,statusdatum) VALUES ('" +
               sMid + "','" +
               sIntern + "','" +
               sGliederung + "','" +
               sWohnGliederung + "','" +
               sSex + "','" +
               sNation + "','" +
               sTel + "','" +
               sGebdat + "','" +
               sZahlart + "','" +
               sBeitrag + "','" +
               sEintritt + "','" +
               sAustritt + "','" +
               sBezahlt + "','" +
               sZahlStatus + "','" +
               sStatusDatum + "')");
  }
  wAdresse->write("M",cMid);
  wBank->write("M",cMid);
  query.exec("COMMIT TRANSACTION");

  pSetWahl(true);
  if (fAnlegen) accept();
}

void Mitglieder::pDelete()
{
  QSqlQuery query;
  int       r;

  if (fWahl) return;
  if (UtilQDelete())
  {
    pFelder();
    query.exec("SELECT count(*) FROM spende WHERE mid='" + sMid + "'");
    if (query.next())
    {
      r=query.value(0).toInt();
      query.finish();
      if (r>0)
      {
        if (QMessageBox::question(NULL,"Lschen",
            "Achtung: Es existieren bereits Zahlungen. Trotzdem lschen?",
            QMessageBox::Ok|QMessageBox::Cancel,QMessageBox::Cancel)!=
            QMessageBox::Ok)
          return;
      }
    }
    query.exec("START TRANSACTION");
    pDelete1();
    query.exec("COMMIT TRANSACTION");
    pSetWahl(true);
  }
}

void Mitglieder::pDelete1()
{
  QSqlQuery query;
  wAdresse->del("M",cMid);
  wBank->del("M",cMid);
  query.exec("DELETE FROM mitglied WHERE mid='" + sMid + "'");
}

void Mitglieder::closeEvent(QCloseEvent *e)
{
  if (fWahl)
    e->accept();
  else
  {
    pCancel();
    e->ignore();
  }
}

void Mitglieder::keyPressEvent(QKeyEvent *e)
{
  if (e->key()==Qt::Key_Escape)
  {
    if (fWahl)
       close();
    else
       pCancel();
    e->ignore();
  }
  e->accept();
}

int Mitglieder::exec()
{
  int r;

  iNeuNummer=-1;
  if (fAnlegen)
  {
    wMid->setText("");
    wAdresse->clear();
    wBank->clear();
    pSetWahl(false);
    wSex->setFocus();
  }
  r=QDialog::exec();
  return(r);
}

int Mitglieder::angelegt()
{
  return(iNeuNummer);
}

void Mitglieder::pFuellListe()
{
  QSqlQuery query,query1,query2;
  QStandardItem *wItem;
  QString       smk;
  int  i,n;
  double vz;

  iMid=wMid->getInt();

  smk=UtilMIDKonto(-1);
  wModel->clear();
  wModel->setHorizontalHeaderLabels(QString("Datum,Kost.,Text,Betrag").split(","));
  query.exec("SELECT jdatum,jlfd,art,quittung FROM spende WHERE mid='" +
             QString::number(iMid) +
             "' ORDER BY mid DESC, jdatum DESC LIMIT 20");
  for (i=n=0; query.next() && i<20; i++)
  {
    query1.exec("SELECT kost,butext FROM journdat WHERE datum='" +
                query.value(0).toString() + "' AND lfd='" +
                query.value(1).toString() + "' LIMIT 1");
    if (query1.next())
    {
      query2.exec("SELECT konto,cent FROM kgkto WHERE datum='" +
                  query.value(0).toString() + "' AND lfd='" +
                  query.value(1).toString() + "' AND kg='1' LIMIT 1");
      if (query2.next())
      {
        wItem=new QStandardItem(query.value(0).toDate().toString("dd.MM.yyyy"));
        wItem->setEditable(false);
        wItem->setSelectable(false);
        wModel->setItem(n,0,wItem);
        wItem=new QStandardItem(query1.value(0).toString());
        wItem->setEditable(false);
        wItem->setSelectable(false);
        wModel->setItem(n,1,wItem);
        wItem=new QStandardItem(query1.value(1).toString());
        wItem->setEditable(false);
        wItem->setSelectable(false);
        wModel->setItem(n,2,wItem);
        if (query2.value(0).toString()>="1000" &&
            query2.value(0).toString()<="1200")
           vz=1.0;
        else if (query2.value(0).toString().left(smk.length())==smk)
           vz=-1.0;
        else if (query2.value(0).toString()>="5000" &&
                 query2.value(0).toString()<="5299")
           vz=-1.0;
        else
           vz=1.0;
        wItem=new QStandardItem(QString::number(query2.value(1).toDouble() * vz / 100.0l,'F',2));
        wItem->setEditable(false);
        wItem->setSelectable(false);
        wItem->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        wModel->setItem(n,3,wItem);
        n++;
        query2.finish();
      }
      query1.finish();
    }
  }
  query.finish();

  wModel->setRowCount(n);
  wListe->setColumnWidth(0,100);
  wListe->setColumnWidth(1,150);
  wListe->setColumnWidth(2,400);
  wListe->setColumnWidth(3,120);
}

void Mitglieder::pMitglied()
{
  QSqlQuery query,query2,query3;
  int       iNeu;
  QString   sNeu,cNeu,s;

  pFelder();
  QInputDialog wDiag(this);
  wDiag.setInputMode(QInputDialog::IntInput);
  wDiag.setLabelText("Mitgliedsnummer");
  wDiag.setIntMinimum(1);
  wDiag.setIntMaximum(999999);
  if (wDiag.exec()!=QDialog::Accepted) return;
  iNeu=wDiag.intValue();
  sNeu=QString::number(iNeu);
  cNeu=cNeu.sprintf("%08i",iNeu);
  query.exec("SELECT intnr,gliederung,sex,nation,tel,gebdat,zahlart,beitrag,eintritt,austritt,bezahlt,zahlstatus,statusdatum,wohngliederung FROM mitglied WHERE mid='" + sNeu + "'");
  if (query.next())
  {
    if (QMessageBox::question(this,"Mitglieder",
         "Mitglied " + sMid + " existiert bereits. Daten bernehmen?",
         QMessageBox::Ok|QMessageBox::Cancel,QMessageBox::Cancel)
         != QMessageBox::Ok)
    {
      query.finish();
      return;
    }
    if (query.value(0).toString().length()>0)
        wIntern->setText(query.value(0).toString());
    if (query.value(1).toString().length()>0)
        wGliederung->setText(query.value(1).toString());
    if      (query.value(2).toString() == "M") wSex->setCurrentIndex(1);
    else if (query.value(2).toString() == "W") wSex->setCurrentIndex(2);
    else if (query.value(2).toString() == "U") wSex->setCurrentIndex(3);
    else                                       wSex->setCurrentIndex(0);
    if (query.value(3).toString().length()>0)
       wNation->setText(query.value(3).toString());
    if (query.value(4).toString().length()>0)
       wTel->setText(query.value(4).toString());
    if (query.value(5).toDate() > QDate(1900,1,1))
       wGebdat->setDate(query.value(5).toDate());
    if      (query.value(6).toString() == "V") wZahlart->setCurrentIndex(1);
    else if (query.value(6).toString() == "L") wZahlart->setCurrentIndex(2);
    else if (query.value(6).toString() == "G") wZahlart->setCurrentIndex(3);
    if (query.value(7).toDouble()>0)
       wBeitrag->setValue(query.value(7).toDouble() / (double)100.0l);
    if (query.value(8).toDate() > QDate(1900,1,1))
       wEintritt->setDate(query.value(8).toDate());
    if (query.value(9).toDate() > QDate(1900,1,1))
       wAustritt->setDate(query.value(9).toDate());
    if (query.value(10).toDate() > QDate(1900,1,1))
       wBezahlt->setDate(query.value(10).toDate());
    if      (query.value(11).toString() == "N") wZahlStatus->setCurrentIndex(1);
    else if (query.value(11).toString() == "R") wZahlStatus->setCurrentIndex(2);
    else if (query.value(11).toString() == "1") wZahlStatus->setCurrentIndex(3);
    else if (query.value(11).toString() == "2") wZahlStatus->setCurrentIndex(4);
    else if (query.value(11).toString() == "3") wZahlStatus->setCurrentIndex(5);
    else if (query.value(11).toString() == "L") wZahlStatus->setCurrentIndex(6);
    else if (query.value(11).toString() == "X") wZahlStatus->setCurrentIndex(7);
    if (query.value(12).toDate() > QDate(1900,1,1))
       wStatusDatum->setDate(query.value(12).toDate());
    if (query.value(13).toString().length()>0)
        wWohnGliederung->setText(query.value(1).toString());

    query.finish();
  }
  query.exec("SELECT name1,name2,strasse,plz,ort,land,mail from adresse WHERE art='M' AND key='" +
              cNeu + "'");
  if (query.next())
  {
    if (query.value(0).toString().length()>0)
       wAdresse->Name1()->setText(query.value(0).toString());
    if (query.value(1).toString().length()>0)
       wAdresse->Name2()->setText(query.value(1).toString());
    if (query.value(2).toString().length()>0)
       wAdresse->Strasse()->setText(query.value(2).toString());
    if (query.value(3).toString().length()>0)
       wAdresse->Plz()->setText(query.value(3).toString());
    if (query.value(4).toString().length()>0)
       wAdresse->Ort()->setText(query.value(4).toString());
    if (query.value(5).toString().length()>0)
       wAdresse->Land()->setText(query.value(5).toString());
    if (query.value(6).toString().length()>0)
       wAdresse->Mail()->setText(query.value(6).toString());
    query.finish();
  }
  query.exec("SELECT inhaber,nummer,blz,bank,bic,iban FROM bankverbindung WHERE art='M' AND key='" +
              cNeu + "'");
  if (query.next())
  {
    if (query.value(0).toString().length()>0)
       wBank->KontoInh()->setText(query.value(0).toString());
    if (query.value(1).toString().length()>0)
       wBank->KontoNr()->setText(query.value(1).toString());
    if (query.value(2).toString().length()>0)
       wBank->KontoBlz()->setText(query.value(2).toString());
    if (query.value(3).toString().length()>0)
       wBank->KontoBank()->setText(query.value(3).toString());
    if (query.value(4).toString().length()>0)
       wBank->KontoBic()->setText(query.value(4).toString());
    if (query.value(5).toString().length()>0)
       wBank->KontoIban()->setText(query.value(5).toString());
    query.finish();
  }
  query.exec("START TRANSACTION");
  pDelete1();
  query.exec("UPDATE spende SET mid='" + sNeu + "' WHERE mid='" + sMid + "'");
  query.exec("SELECT datum,lfd FROM kgkto WHERE konto='" + UtilMIDKonto(iMid) +
             "'");
  while(query.next())
  {
    query2.exec("SELECT butext FROM journdat WHERE datum='" +
                query.value(0).toString() + "' AND lfd='" +
                query.value(1).toString() + "'");
    if (query2.next())
    {
      if (query2.value(0).toString().contains("MID " + sMid + " "))
         s=query2.value(0).toString().replace("MID " + sMid + " ",
                                              "MID " + sNeu + " ");
      else
         s="MID " + sNeu + " " + query2.value(0).toString();
      query3.exec("UPDATE journdat SET butext='" + s + "' WHERE datum='" +
                query.value(0).toString() + "' AND lfd='" +
                query.value(1).toString() + "'");
      query2.finish();
    }
  }
  query.exec("UPDATE kgkto SET konto='" + UtilMIDKonto(iNeu) +
             "' WHERE konto='" + UtilMIDKonto(iMid) + "'");
  query.exec("UPDATE zahllauf SET mid='" + sNeu + "' WHERE mid='" + sMid + "'");
  query.exec("COMMIT TRANSACTION");
  wMid->setText(sNeu);
  pWrite();
  wMid->setText(sNeu);
  pRead();
}

void Mitglieder::pZuwend()
{
  int       wAnzSeiten,wAnzZuwend,i;
  QSqlQuery query,query2;
  int       b;
  bool      flag_nochmal;

  pFelder();
  Zuwenddruck wDiag(iMid,this);
  wDiag.exec();
}
