#include <QtGui>
#include <QtCore>
#include <QtSql>

#include "setup.h"

Setup::Setup(QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  int    z;
  QSqlQuery   query;
  QVBoxLayout *topLayout    = new QVBoxLayout;
  QHBoxLayout *buttonLayout = new QHBoxLayout;
  QGridLayout *allgLayout   = new QGridLayout;
  QGridLayout *mailLayout   = new QGridLayout;
  QGridLayout *spendeLayout = new QGridLayout;
  QTabWidget  *wTabWidget   = new QTabWidget;
  QWidget     *wTabAllg     = new QWidget;
  QWidget     *wTabMail     = new QWidget;
  QWidget     *wTabSpende   = new QWidget;
  QIntValidator *wValMm = new QIntValidator(0,999,this);
  QIntValidator *wValPt = new QIntValidator(1,36,this);

  setWindowTitle("Setup");
  wTabWidget->addTab(wTabAllg,"A&llgemein");
  wTabWidget->addTab(wTabMail,"&Mail/Brief");
  wTabWidget->addTab(wTabSpende,"&Zuwendungsbest.");

  wGliederung = new WGliederung(0);
  wGliederung->setFixedSize(500,25);
  lAbgrAktiv=new QLabel();
  lAbgrPassiv=new QLabel();
  lBankGeb=new QLabel();
  lPrefMitglied=new QLabel();
  lPrefGliederung=new QLabel();
  lPrefLief=new QLabel();
  wAbgrAktiv = new WKonto(lAbgrAktiv,true,this);
  wAbgrPassiv = new WKonto(lAbgrPassiv,true,this);
  wBankGeb = new WKonto(lBankGeb,true,this);
  wPrefMitglied = new WKonto(lPrefMitglied,true,this);
  wAddMitglied = new QLineEdit();
  wAddMitglied->setFixedSize(95,25);
  wAddMitglied->setValidator(new QIntValidator(0,9999999,this));
  wPrefGliederung = new WKonto(lPrefGliederung,true,this);
  wPrefLief = new WKonto(lPrefLief,true,this);
  wBeitragAb = new QComboBox();
  wBeitragAb->insertItem(0,"Monat nach Grndung");
  wBeitragAb->insertItem(1,"Jahr nach Grndung");
  wBeitragAb->insertItem(2,"Kein eigener Einzug");
  wBeitragAb->setFixedSize(wBeitragAb->sizeHint());

  wMailHost   = new QLineEdit(this); wMailHost->setFixedSize(300,25);
  wMailPort   = new QLineEdit(this); wMailPort->setFixedSize(100,25);
  wMailPort->setValidator(new QIntValidator(1,65535,this));
  wMailUser   = new QLineEdit(this); wMailUser->setFixedSize(300,25);
  wMailPasswd = new QLineEdit(this); wMailPasswd->setFixedSize(300,25);
  wMailPasswd->setEchoMode(QLineEdit::Password);
  wMailAbs    = new QLineEdit(this); wMailAbs->setFixedSize(300,25);
  wMailName   = new QLineEdit(this); wMailName->setFixedSize(300,25);
  wMailAdrL   = new QLineEdit(this); wMailAdrL->setFixedSize(50,25);
  wMailAdrT   = new QLineEdit(this); wMailAdrT->setFixedSize(50,25);
  wMailAdrPt  = new QLineEdit(this); wMailAdrPt->setFixedSize(50,25);
  wMailL      = new QLineEdit(this); wMailL->setFixedSize(50,25);
  wMailT      = new QLineEdit(this); wMailT->setFixedSize(50,25);
  wMailPt     = new QLineEdit(this); wMailPt->setFixedSize(50,25);
  wMailR      = new QLineEdit(this); wMailR->setFixedSize(50,25);
  wMailB      = new QLineEdit(this); wMailB->setFixedSize(50,25);
  wMailAdrL->setValidator(wValMm);
  wMailAdrT->setValidator(wValMm);
  wMailL->setValidator(wValMm);
  wMailT->setValidator(wValMm);
  wMailR->setValidator(wValMm);
  wMailB->setValidator(wValMm);
  wMailAdrPt->setValidator(wValPt);
  wMailPt->setValidator(wValPt);

  wSpendeAdr1 = new QLineEdit(this); wSpendeAdr1->setFixedSize(400,25);
  wSpendeAdr2 = new QLineEdit(this); wSpendeAdr2->setFixedSize(400,25);
  wSpendeAdr3 = new QLineEdit(this); wSpendeAdr3->setFixedSize(400,25);
  wSpendeAbs  = new QLineEdit(this); wSpendeAbs->setFixedSize(500,25);
  wSpendeOrt  = new QLineEdit(this); wSpendeOrt->setFixedSize(300,25);
  wSpendeFunkt= new QLineEdit(this); wSpendeFunkt->setFixedSize(500,25);
  wSpendeMin  = new QDoubleSpinBox;
  wSpendeMin->setDecimals(2);
  wSpendeMin->setMinimum(-99999999.99);
  wSpendeMin->setMaximum( 99999999.99);
  wSpendeMin->setSuffix(" EUR");
  wSpendeMin->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
  wSpendeMin->setFixedSize(150,25);
  wSpendeMin->setValue(50.0l);

  bOk = new QPushButton("&Speichern");
  bCancel = new QPushButton("&Abbruch");
  bOk->setAutoDefault(false);
  bCancel->setAutoDefault(false);

  query.exec("SELECT key,wert FROM parameter");
  while(query.next())
  {
    if (query.value(0).toString()=="General/Gliederung")
       wGliederung->setText(query.value(1).toString());
    else if (query.value(0).toString()=="General/BeitragAb")
    {
      if (query.value(1).toString()=="J")
         wBeitragAb->setCurrentIndex(1);
      else if (query.value(1).toString()=="X")
         wBeitragAb->setCurrentIndex(2);
      else
         wBeitragAb->setCurrentIndex(0);
    }
    else if (query.value(0).toString()=="Konten/AbgrAktiv")
       wAbgrAktiv->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Konten/AbgrPassiv")
       wAbgrPassiv->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Konten/BankGeb")
       wBankGeb->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Konten/PrefMitglied")
       wPrefMitglied->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Konten/AddMitglied")
       iAddMitglied=query.value(1).toInt();
    else if (query.value(0).toString()=="Konten/PrefGliederung")
       wPrefGliederung->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Konten/PrefLief")
       wPrefLief->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/Host")
       wMailHost->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/Port")
       wMailPort->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/User")
       wMailUser->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/Passwd")
       wMailPasswd->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/Abs")
       wMailAbs->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/Name")
       wMailName->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/AdrL")
       wMailAdrL->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/AdrT")
       wMailAdrT->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/AdrPt")
       wMailAdrPt->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/L")
       wMailL->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/T")
       wMailT->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/R")
       wMailR->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/B")
       wMailB->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Mail/Pt")
       wMailPt->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Spende/Adr1")
       wSpendeAdr1->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Spende/Adr2")
       wSpendeAdr2->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Spende/Adr3")
       wSpendeAdr3->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Spende/Abs")
       wSpendeAbs->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Spende/Ort")
       wSpendeOrt->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Spende/Funkt")
       wSpendeFunkt->setText(query.value(1).toString());
    else if (query.value(0).toString()=="Spende/Min")
       wSpendeMin->setValue(query.value(1).toDouble() / (double)100.0l);
  }
  query.finish();
  if (wMailPort->text().length()<1) wMailPort->setText("25");
  if (wMailAdrL->text().length()<1) wMailAdrL->setText("12");
  if (wMailAdrT->text().length()<1) wMailAdrT->setText("55");
  if (wMailAdrL->text().length()<1) wMailAdrL->setText("25");
  if (wMailAdrPt->text().length()<1) wMailAdrPt->setText("12");
  if (wMailT->text().length()<1) wMailT->setText("95");
  if (wMailL->text().length()<1) wMailL->setText("25");
  if (wMailR->text().length()<1) wMailR->setText("20");
  if (wMailB->text().length()<1) wMailB->setText("20");
  if (wMailPt->text().length()<1) wMailPt->setText("12");

  wAddMitglied->setText(QString::number(iAddMitglied));

  topLayout->addWidget(wTabWidget);
  topLayout->addLayout(buttonLayout);

  z=1;
  l=new QLabel("Gliederung");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wGliederung,z,2,1,2);
  z++;
  l=new QLabel("Abgrenzung Aktiv");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wAbgrAktiv,z,2); allgLayout->addWidget(lAbgrAktiv,z,3);
  z++;
  l=new QLabel("Abgrenzung Passiv");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wAbgrPassiv,z,2); allgLayout->addWidget(lAbgrPassiv,z,3);
  z++;
  l=new QLabel("Bankgebhren");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wBankGeb,z,2); allgLayout->addWidget(lBankGeb,z,3);
  z++;
  l=new QLabel("Kontenprefix Gliederung");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wPrefGliederung,z,2);allgLayout->addWidget(lPrefGliederung,z,3);
  z++;
  l=new QLabel("Kontenprefix Mitglieder");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wPrefMitglied,z,2); allgLayout->addWidget(lPrefMitglied,z,3);
  z++;
  l=new QLabel("Addition Mitglieder");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wAddMitglied,z,2);
  z++;
  l=new QLabel("Kontenprefix Lieferanten");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wPrefLief,z,2); allgLayout->addWidget(lPrefLief,z,3);
  z++;
  l=new QLabel("Beitragsanteil Gliederungen");
  allgLayout->addWidget(l,z,1);
  allgLayout->addWidget(wBeitragAb,z,2);
  z++;

  z=1;
  l=new QLabel("Mailausgang:");
  mailLayout->addWidget(l,z,1);
  z++;
  l=new QLabel("Mailserver");
  mailLayout->addWidget(l,z,1);
  mailLayout->addWidget(wMailHost,z,2,1,5);
  l=new QLabel("Port");
  mailLayout->addWidget(l,z,7);
  mailLayout->addWidget(wMailPort,z,8);
  z++;
  l=new QLabel("Auth. User");
  mailLayout->addWidget(l,z,1);
  mailLayout->addWidget(wMailUser,z,2,1,5);
  l=new QLabel("Passwort");
  mailLayout->addWidget(l,z,7);
  mailLayout->addWidget(wMailPasswd,z,8);
  z++;
  l=new QLabel("Absender");
  mailLayout->addWidget(l,z,1);
  mailLayout->addWidget(wMailAbs,z,2,1,5);
  l=new QLabel("Name");
  mailLayout->addWidget(l,z,7);
  mailLayout->addWidget(wMailName,z,8);
  z++;
  l=new QLabel("Briefpapier:");
  mailLayout->addWidget(l,z,1);
  z++;
  l=new QLabel("Adressfeld");
  mailLayout->addWidget(l,z,1);
  l=new QLabel("links");
  mailLayout->addWidget(l,z,2);
  mailLayout->addWidget(wMailAdrL,z,3);
  l=new QLabel("oben");
  mailLayout->addWidget(l,z,4);
  mailLayout->addWidget(wMailAdrT,z,5);
  l=new QLabel("Schrift");
  mailLayout->addWidget(l,z,7);
  mailLayout->addWidget(wMailAdrPt,z,8);
  z++;
  l=new QLabel("Textfeld");
  mailLayout->addWidget(l,z,1);
  l=new QLabel("links");
  mailLayout->addWidget(l,z,2);
  mailLayout->addWidget(wMailL,z,3);
  l=new QLabel("oben");
  mailLayout->addWidget(l,z,4);
  mailLayout->addWidget(wMailT,z,5);
  l=new QLabel("Schrift");
  mailLayout->addWidget(l,z,7);
  mailLayout->addWidget(wMailPt,z,8);
  z++;
  l=new QLabel("rechts");
  mailLayout->addWidget(l,z,2);
  mailLayout->addWidget(wMailR,z,3);
  l=new QLabel("unten");
  mailLayout->addWidget(l,z,4);
  mailLayout->addWidget(wMailB,z,5);
  z++;

  mailLayout->setColumnStretch(6,1);
  mailLayout->setRowStretch(z,1);

  z=1;
  l=new QLabel("Adr. 1.Zeile");
  spendeLayout->addWidget(l,z,1);
  spendeLayout->addWidget(wSpendeAdr1,z,2);
  z++;
  l=new QLabel("Adr. 2.Zeile");
  spendeLayout->addWidget(l,z,1);
  spendeLayout->addWidget(wSpendeAdr2,z,2);
  z++;
  l=new QLabel("Adr. 3.Zeile");
  spendeLayout->addWidget(l,z,1);
  spendeLayout->addWidget(wSpendeAdr3,z,2);
  z++;
  l=new QLabel("Absender");
  spendeLayout->addWidget(l,z,1);
  spendeLayout->addWidget(wSpendeAbs,z,2);
  z++;
  l=new QLabel("Ort");
  spendeLayout->addWidget(l,z,1);
  spendeLayout->addWidget(wSpendeOrt,z,2);
  z++;
  l=new QLabel("Funktion");
  spendeLayout->addWidget(l,z,1);
  spendeLayout->addWidget(wSpendeFunkt,z,2);
  z++;
  l=new QLabel("Min.Betrag");
  spendeLayout->addWidget(l,z,1);
  spendeLayout->addWidget(wSpendeMin,z,2);
  z++;

  spendeLayout->setRowStretch(z,1);
  spendeLayout->setColumnStretch(99,1);

  buttonLayout->addWidget(bOk);
  buttonLayout->addWidget(bCancel);
  buttonLayout->addStretch(1);

  wTabAllg->setLayout(allgLayout);
  wTabMail->setLayout(mailLayout);
  wTabSpende->setLayout(spendeLayout);
  setLayout(topLayout);

  connect(bCancel,SIGNAL(clicked()),this,SLOT(pCancel()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pWrite()));
}

Setup::~Setup()
{
  delete wGliederung;
  delete lAbgrAktiv;
  delete lAbgrPassiv;
  delete lBankGeb;
  delete lPrefMitglied;
  delete lPrefGliederung;
  delete lPrefLief;
  delete wAbgrAktiv;
  delete wAbgrPassiv;
  delete wBankGeb;
  delete wPrefMitglied;
  delete wAddMitglied;
  delete wPrefGliederung;
  delete wPrefLief;
  delete wBeitragAb;
  delete wMailHost;
  delete wMailPort;
  delete wMailUser;
  delete wMailPasswd;
  delete wMailAbs;
  delete wMailName;
  delete wMailAdrL;
  delete wMailAdrT;
  delete wMailAdrPt;
  delete wMailL;
  delete wMailT;
  delete wMailPt;
  delete wMailR;
  delete wMailB;
  delete wSpendeAdr1;
  delete wSpendeAdr2;
  delete wSpendeAdr3;
  delete wSpendeAbs;
  delete wSpendeOrt;
  delete wSpendeFunkt;
  delete wSpendeMin;
  delete bOk;
  delete bCancel;
  return;
}

void Setup::pFelder()
{
  sGliederung=wGliederung->text().replace("'","`").simplified();
  sAbgrAktiv=wAbgrAktiv->text().replace("'","`").toUpper().simplified();
  sAbgrPassiv=wAbgrPassiv->text().replace("'","`").toUpper().simplified();
  sBankGeb=wBankGeb->text().replace("'","`").toUpper().simplified();
  sPrefMitglied=wPrefMitglied->text().replace("'","`").toUpper().simplified();
  sPrefGliederung=wPrefGliederung->text().replace("'","`").toUpper().simplified();
  sPrefGliederung=wPrefGliederung->text().replace("'","`").toUpper().simplified();
  sPrefLief=wPrefLief->text().replace("'","`").toUpper().simplified();
  switch(wBeitragAb->currentIndex())
  {
     case 0 : sBeitragAb="M"; break;
     case 1 : sBeitragAb="J"; break;
     case 2 : sBeitragAb="X"; break;
     default: sBeitragAb="X"; break;
  }
  iAddMitglied=wAddMitglied->text().toInt();
  sMailHost=wMailHost->text().replace("'","`").simplified();
  sMailPort=wMailPort->text().replace("'","`").simplified();
  sMailUser=wMailUser->text().replace("'","`").simplified();
  sMailPasswd=wMailPasswd->text().replace("'","`").simplified();
  sMailAbs=wMailAbs->text().replace("'","`").simplified();
  sMailName=wMailName->text().replace("'","`").simplified();
  iMailAdrL=wMailAdrL->text().toInt();
  iMailAdrT=wMailAdrT->text().toInt();
  iMailAdrPt=wMailAdrPt->text().toInt();
  iMailL=wMailL->text().toInt();
  iMailT=wMailT->text().toInt();
  iMailR=wMailR->text().toInt();
  iMailB=wMailB->text().toInt();
  iMailPt=wMailPt->text().toInt();
  sSpendeAdr1 =wSpendeAdr1->text().replace("'","`").simplified();
  sSpendeAdr2 =wSpendeAdr2->text().replace("'","`").simplified();
  sSpendeAdr3 =wSpendeAdr3->text().replace("'","`").simplified();
  sSpendeAbs  =wSpendeAbs->text().replace("'","`").simplified();
  sSpendeOrt  =wSpendeOrt->text().replace("'","`").simplified();
  sSpendeFunkt=wSpendeFunkt->text().replace("'","`").simplified();
  iSpendeMin=(qint64)round(wSpendeMin->value() * (double)(100.0l));
  sSpendeMin=QString::number(iSpendeMin);
}

void Setup::pCancel()
{
  close();
}

void Setup::pWrite()
{
  QSqlQuery query;
  pFelder();
  query.exec("START TRANSACTION");
  query.exec("DELETE FROM parameter WHERE key='General/Gliederung'");
  query.exec("DELETE FROM parameter WHERE key='General/BeitragAb'");
  query.exec("DELETE FROM parameter WHERE key>='Konten/' AND key < 'Konten0'");
  query.exec("DELETE FROM parameter WHERE key>='Mail/' AND key < 'Mail0'");
  query.exec("DELETE FROM parameter WHERE key>='Spende/' AND key < 'Spende0'");

  query.exec("INSERT INTO parameter (key,wert) VALUES ('General/Gliederung','"
              + sGliederung + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('General/BeitragAb','"
              + sBeitragAb + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Konten/AbgrAktiv','"
              + sAbgrAktiv + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Konten/AbgrPassiv','"
              + sAbgrPassiv + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Konten/BankGeb','"
              + sBankGeb + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Konten/PrefGliederung','"
              + sPrefGliederung + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Konten/PrefMitglied','"
              + sPrefMitglied + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Konten/AddMitglied','"
              + QString::number(iAddMitglied) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Konten/PrefLief','"
              + sPrefLief + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/Host','"
              + sMailHost + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/Port','"
              + sMailPort + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/User','"
              + sMailUser + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/Passwd','"
              + sMailPasswd + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/Abs','"
              + sMailAbs + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/Name','"
              + sMailName + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/AdrL','"
              + QString::number(iMailAdrL) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/AdrT','"
              + QString::number(iMailAdrT) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/AdrPt','"
              + QString::number(iMailAdrPt) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/L','"
              + QString::number(iMailL) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/T','"
              + QString::number(iMailT) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/R','"
              + QString::number(iMailR) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/B','"
              + QString::number(iMailB) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Mail/Pt','"
              + QString::number(iMailPt) + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Spende/Adr1','"
              + sSpendeAdr1 + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Spende/Adr2','"
              + sSpendeAdr2 + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Spende/Adr3','"
              + sSpendeAdr3 + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Spende/Abs','"
              + sSpendeAbs + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Spende/Ort','"
              + sSpendeOrt + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Spende/Funkt','"
              + sSpendeFunkt + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('Spende/Min','"
              + sSpendeMin + "')");
  query.exec("COMMIT TRANSACTION");

  UtilInit(true);
  close();
}
