#include <QtSql>

#include "spende.h"
#include "utils.h"

Spende::Spende(QWidget *parent) : QDialog(parent)
{
  QLabel      *l;
  int         z;

  fNeu=true;
  fAufteilen=true;
  iArtRead=-1;
  iTotal=0;
  fBeitrDat=true;
  bOk=new QPushButton("&Speichern");
  bCancel=new QPushButton("&Abbruch");
  bOk->setAutoDefault(false);
  bCancel->setAutoDefault(false);
  QVBoxLayout *topLayout = new QVBoxLayout();
  QGridLayout *mainLayout = new QGridLayout();
  QHBoxLayout *buttonLayout = new QHBoxLayout();
  wMitglied = new WMitglied(true);
  wGliederung = new WGliederung();
  wArt = new QComboBox();
  wArt->insertItem(0,"Zweckgebundene Geldspende");  // 'Z'
  wArt->insertItem(1,"Allgemeine Geldspende");      // 'G'
  wArt->insertItem(2,"Aufwandsspende");             // 'A'
  wArt->insertItem(3,"Sachspende");                 // 'S'
  wArt->insertItem(4,"Mitgliedsbeitrag");           // 'M'
  wArt->insertItem(5,"Mandatstrgerbeitrag");       // 'T'
  wArt->setFixedSize(wArt->sizeHint());
  wProzInt=pProzBox();
  wProzBuv=pProzBox();
  wProzLv =pProzBox();
  wProzBzv=pProzBox();
  wProzKv =pProzBox();
  wProzOv =pProzBox();
  wQuittiert = new QDateEdit();
  wQuittiert->setDisplayFormat("dd.MM.yyyy");
  wQuittiert->setFixedSize(wQuittiert->sizeHint());
  wBeitrVon = new QDateEdit(); wBeitrVon->setDate(QDate(1900,1,1));
  wBeitrVon->setDisplayFormat("dd.MM.yyyy");
  wBeitrVon->setFixedSize(wQuittiert->sizeHint());
  wBeitrVon->setEnabled(false);
  wBeitrBis = new QDateEdit(); wBeitrBis->setDate(QDate(1900,1,1));
  wBeitrBis->setDisplayFormat("dd.MM.yyyy");
  wBeitrBis->setFixedSize(wQuittiert->sizeHint());
  wGebuehr = new QDoubleSpinBox();
  wGebuehr->setDecimals(2);
  wGebuehr->setMinimum(-999.99);
  wGebuehr->setMaximum( 999.99);
  wGebuehr->setSuffix(" EUR");
  wGebuehr->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
  wGebuehr->setFixedSize(120,25);

  wEbene=UtilSelbstEbene();

  lDisName = new QLabel("");

  topLayout->addLayout(mainLayout);
  topLayout->addLayout(buttonLayout);

  z=0;
  l=new QLabel("MID/Spender");
  mainLayout->addWidget(l,z,1);
  mainLayout->addWidget(wMitglied,z,2);
  l=new QLabel("Bezahlt bis");
  mainLayout->addWidget(l,z,3);
  mainLayout->addWidget(wBeitrVon,z,5);
  z++;
  mainLayout->addWidget(lDisName,z,1,1,3);
  z++;
  l=new QLabel("Gliederung");
  mainLayout->addWidget(l,z,1);
  mainLayout->addWidget(wGliederung,z,2,1,4);
  z++;
  l=new QLabel("Art");
  mainLayout->addWidget(l,z,1);
  mainLayout->addWidget(wArt,z,2,1,2);
  mainLayout->addWidget(wBeitrBis,z,5);
  z++;

  l=new QLabel("Anteile:");
  mainLayout->addWidget(l,z,1);
  l=new QLabel("Bankgebhr");
  mainLayout->addWidget(l,z,3);
  mainLayout->addWidget(wGebuehr,z,5);
  z++;
  l=new QLabel("Ortsverband");
  mainLayout->addWidget(l,z,1);
  mainLayout->addWidget(wProzOv,z,2);
  l=new QLabel("Kreisverband");
  mainLayout->addWidget(l,z,3);
  mainLayout->addWidget(wProzKv,z,5);
  z++;
  l=new QLabel("Bezirksverband");
  mainLayout->addWidget(l,z,1);
  mainLayout->addWidget(wProzBzv,z,2);
  l=new QLabel("Landesverband");
  mainLayout->addWidget(l,z,3);
  mainLayout->addWidget(wProzLv,z,5);
  z++;
  l=new QLabel("Bundesverband");
  mainLayout->addWidget(l,z,1);
  mainLayout->addWidget(wProzBuv,z,2);
  l=new QLabel("International");
  mainLayout->addWidget(l,z,3);
  mainLayout->addWidget(wProzInt,z,5);
  z++;

  l=new QLabel("Bescheinigung");
  mainLayout->addWidget(l,z,1);
  mainLayout->addWidget(wQuittiert,z,2);
  z++;

  mainLayout->setColumnStretch(4,1);

  buttonLayout->addWidget(bOk);
  buttonLayout->addWidget(bCancel);
  buttonLayout->addStretch(1);

  clear();
  setLayout(topLayout);
  connect(bOk,SIGNAL(clicked()),this,SLOT(accept()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(reject()));
  connect(wMitglied,SIGNAL(finished()),this,SLOT(pDisName()));
  connect(wArt,SIGNAL(currentIndexChanged(int)),this,SLOT(pArtChg()));
}

Spende::~Spende()
{
  return;
}

void Spende::pFelder()
{
  iMid=wMitglied->text().simplified().toInt();
  iProzInt=(qint64)round(wProzInt->value() * (double)(100.0l));
  iProzBuv=(qint64)round(wProzBuv->value() * (double)(100.0l));
  iProzLv =(qint64)round(wProzLv->value() * (double)(100.0l));
  iProzBzv=(qint64)round(wProzBzv->value() * (double)(100.0l));
  iProzKv =(qint64)round(wProzKv->value() * (double)(100.0l));
  iProzOv =(qint64)round(wProzOv->value() * (double)(100.0l));
  sGliederung=wGliederung->text().replace("'","`").simplified();
  if (iMid<1) sMid="";
  else        sMid=QString::number(iMid);
  sQuittiert=wQuittiert->date().toString("yyyy-MM-dd");
  sProzInt=QString::number(iProzInt);
  sProzBuv=QString::number(iProzBuv);
  sProzLv =QString::number(iProzLv );
  sProzBzv=QString::number(iProzBzv);
  sProzKv =QString::number(iProzKv );
  sProzOv =QString::number(iProzOv );
  dBeitrVon=wBeitrVon->date();
  dBeitrBis=wBeitrBis->date();
  switch(wArt->currentIndex())
  {
    case 0: sArt="Z"; break;
    case 1: sArt="G"; break;
    case 2: sArt="A"; break;
    case 3: sArt="S"; break;
    case 4: sArt="M"; break;
    case 5: sArt="T"; break;
    default: sArt="Z"; break;
  }
}

void Spende::read(QDate jDatum, int jLfd)
{
  QSqlQuery query;

  if (jLfd<1) { clear(); return; }
  iArtRead=-1;
  fNeu=true;
  iMid=-1;
  query.exec("SELECT mid,art,gliederung,quittung,prozint,prozbuv,prozlv,prozbzv,prozkv,prozov FROM spende WHERE jdatum='" +
               jDatum.toString("yyyy-MM-dd") +
               "' AND jlfd='" + QString::number(jLfd) + "'");
  if (query.next())
  {
    fNeu=false;
    wMitglied->setText(query.value(0).toString());
    sArt=query.value(1).toString();
    if      (sArt=="Z") iArtRead=0;
    else if (sArt=="G") iArtRead=1;
    else if (sArt=="A") iArtRead=2;
    else if (sArt=="S") iArtRead=3;
    else if (sArt=="M") iArtRead=4;
    else                iArtRead=0;
    wArt->setCurrentIndex(iArtRead);
    wGliederung->setText(query.value(2).toString());
    wQuittiert->setDate(query.value(3).toDate());
    wProzInt->setValue((double)query.value(4).toInt() / 100.0);
    wProzBuv->setValue((double)query.value(5).toInt() / 100.0);
    wProzLv->setValue((double)query.value(6).toInt() / 100.0);
    wProzBzv->setValue((double)query.value(7).toInt() / 100.0);
    wProzKv->setValue((double)query.value(8).toInt() / 100.0);
    wProzOv->setValue((double)query.value(9).toInt() / 100.0);
    query.finish();
    pFelder();
  }
  pDisName();
}

void Spende::clear()
{
  iArtRead=-1;
  fNeu=true;
  fBeitrDat=true;
  wQuittiert->setDate(QDate(1900,1,1));
  wMitglied->setText("");
  wGliederung->setText("");
  wArt->setCurrentIndex(0);
  lDisName->setText("");
  wProzInt->setValue(0);
  wProzBuv->setValue(0);
  wProzLv->setValue(0);
  wProzBzv->setValue(0);
  wProzKv->setValue(0);
  wProzOv->setValue(0);
  wGebuehr->setValue(0);
  fMIDBuchung=false;
  iProzInt=0;
  iProzBuv=0;
  iProzLv=0;
  iProzBzv=0;
  iProzKv=0;
  iProzOv=0;
  sKtoK="";
  sKtoB="";
  sKtoAnonK="";
  sKtoAnonB="";
  sKtoJurK="";
  sKtoJurB="";
  sKostK="";
  sKostB="";
  sKostAnonK="";
  sKostAnonB="";
  sKostJurK="";
  sKostJurB="";
  lDisName->setText("");
  pFelder();
  pBeitrDat();
}

void Spende::write(QDate jDatum, int jLfd)
{
  QSqlQuery query;
  QString   wSortname;
  QString   s;

  pFelder();

  if (iMid<1)
  {
    query.exec("DELETE FROM spende WHERE jdatum='" +
               jDatum.toString("yyyy-MM-dd") + "' AND jlfd='" +
               QString::number(jLfd) + "'");
  }
  else
  {
    query.exec("SELECT name1,name2 FROM adresse WHERE art='M' AND key='" +
                wMitglied->getKey() + "'");
    if (query.next())
    {
      wSortname=query.value(0).toString() + ", " + query.value(1).toString() +
                ", " + wMitglied->getKey();
      query.finish();
    }
    else
       wSortname="";
    query.exec("UPDATE spende SET mid='" + sMid +
               "',sortname='" + wSortname +
               "',art='" + sArt +
               "',gliederung='" + sGliederung +
               "',quittung='" + sQuittiert +
               "',prozint='" + sProzInt +
               "',prozbuv='" + sProzBuv +
               "',prozlv='" + sProzLv +
               "',prozbzv='" + sProzBzv +
               "',prozkv='" + sProzKv +
               "',prozov='" + sProzOv +
               "' WHERE jdatum='" + jDatum.toString("yyyy-MM-dd") +
               "' AND jlfd='" + QString::number(jLfd) + "'");
    if (query.numRowsAffected()<1)
    {
      query.exec("INSERT INTO spende (jdatum,jlfd,mid,sortname,art,gliederung,quittung,prozint,prozbuv,prozlv,prozbzv,prozkv,prozov) VALUES ('" +
               jDatum.toString("yyyy-MM-dd") + "','" +
               QString::number(jLfd) + "','" + sMid + "','" +
               wSortname + "','" + sArt + "','" + sGliederung + "','" +
               sQuittiert + "','" + sProzInt + "','" + sProzBuv + "','" +
               sProzLv + "','" + sProzBzv + "','" + sProzKv  + "','" +
               sProzOv + "')");
               
    }
  }
  if (iMid>0 && iMid<1000000 && sArt=="M" && dBeitrVon!=dBeitrBis)
  {
    s="bezahlt='" + dBeitrBis.toString("yyyy-MM-dd") + "'";
    query.exec("SELECT zahlart,zahlstatus,statusdatum FROM mitglied WHERE mid='"
               + sMid + "' LIMIT 1");
    if (query.next())
    {
      if (query.value(1).toString() > "!")
      {
         s=s+",zahlstatus=' ', statusdatum='" +
           QDate::currentDate().toString("yyyy-MM-dd") + "'";
      }
      if (query.value(0).toString() == "L" && iTotal<0)
         s=s+",zahlart='G'";
      if (query.value(0).toString() == "G" && iTotal>0)
         s=s+",zahlart='L'";
      query.finish();
    }
    query.exec("UPDATE mitglied SET " + s + " WHERE mid='" + sMid + "'");
  }
  fNeu=false;
  pBeitrDat();
}

int Spende::exec(qint64 total)
{
  int r;

  iTotal=total;
  wMitglied->setText(sMid);
  wGliederung->setText(sGliederung);
  wQuittiert->setDate(QDate::fromString(sQuittiert,"yyyy-MM-dd"));
  wProzInt->setValue((double)iProzInt / 100.0);
  wProzBuv->setValue((double)iProzBuv / 100.0);
  wProzLv->setValue((double)iProzLv / 100.0);
  wProzBzv->setValue((double)iProzBzv / 100.0);
  wProzKv->setValue((double)iProzKv / 100.0);
  wProzOv->setValue((double)iProzOv / 100.0);
  if (iMid<1)
     wMitglied->setFocus();
  else
     bCancel->setFocus();
  if      (sArt=="Z") wArt->setCurrentIndex(0);
  else if (sArt=="G") wArt->setCurrentIndex(1);
  else if (sArt=="A") wArt->setCurrentIndex(2);
  else if (sArt=="S") wArt->setCurrentIndex(3);
  else if (sArt=="M") wArt->setCurrentIndex(4);
  else if (sArt=="T") wArt->setCurrentIndex(5);
  else                wArt->setCurrentIndex(0);
  pArtChg();
  wBeitrVon->setDate(dBeitrVon);
  wBeitrBis->setDate(dBeitrBis);
  r=QDialog::exec();
  if (r==QDialog::Accepted) fAufteilen=true;
  return(r);
}

int Spende::exec()
{
  return(exec(0));
}

void Spende::accept()
{
  pFelder();
  QDialog::accept();
}

void Spende::reject()
{
  wMitglied->setText(sMid);
  pDisName();
  QDialog::reject();
}

void Spende::pDisName()
{
  QSqlQuery query;
  int       nr;

  nr=wMitglied->getInt();
  if (nr<1)
    lDisName->setText("");
  else
  {
    query.exec("SELECT name1,name2,plz,ort FROM adresse WHERE art='M' AND key='" +
                wMitglied->getKey() + "'");
    if (query.next())
    {
      if (nr<10000000)
         lDisName->setText(QString::number(nr) + " " +
                   query.value(0).toString() + ", " +
                   query.value(1).toString() + ", " +
                   query.value(2).toString() + "  " +
                   query.value(3).toString());
      else
      lDisName->setText("");
      query.finish();
      if (wGliederung->text()=="")
      {
        query.exec("SELECT gliederung FROM mitglied WHERE mid='" +
                   QString::number(nr) + "'");
        if (query.next())
        {
           wGliederung->setText(query.value(0).toString());
           query.finish();
        }
      }
    }
    else
      lDisName->setText("");
  }
  pBeitrDat();
}

void Spende::pArtChg()
{
  QSqlQuery query;
  qint64    iProzRest;

  if (wArt->currentIndex()==iArtRead) return;
  switch(wArt->currentIndex())
  {
    case 0: sArt="Z"; break;
    case 1: sArt="G"; break;
    case 2: sArt="A"; break;
    case 3: sArt="S"; break;
    case 4: sArt="M"; break;
    case 5: sArt="T"; break;
    default: sArt="Z"; break;
  }
  wProzInt->setValue(0.0);
  wProzBuv->setValue(0.0);
  wProzLv->setValue(0.0);
  wProzBzv->setValue(0.0);
  wProzKv->setValue(0.0);
  wProzOv->setValue(0.0);

  iProzInt=0;
  iProzBuv=0;
  iProzLv=0;
  iProzBzv=0;
  iProzKv=0;
  iProzOv=0;
  sKtoK="";
  sKtoB="";
  sKtoAnonK="";
  sKtoAnonB="";
  sKtoJurK="";
  sKtoJurB="";
  sKostK="";
  sKostB="";
  sKostAnonK="";
  sKostAnonB="";
  sKostJurK="";
  sKostJurB="";
  fMIDBuchung=false;
  query.exec("SELECT prozint,prozbuv,prozlv,prozbzv,prozkv,prozov,midbuchung,kontokasse,kontobank,kontoanonkasse,kontoanonbank,kontojurkasse,kontojurbank,kostkasse,kostbank,kostanonkasse,kostanonbank,kostjurkasse,kostjurbank FROM spendenparam WHERE art='" + sArt + "'");
  if (query.next())
  {
    iProzInt=query.value(0).toLongLong();
    iProzBuv=query.value(1).toLongLong();
    iProzLv =query.value(2).toLongLong();
    iProzBzv=query.value(3).toLongLong();
    iProzKv =query.value(4).toLongLong();
    iProzOv =query.value(5).toLongLong();
    if (query.value(6).toString()=="J") fMIDBuchung=true;
    sKtoK=query.value(7).toString();
    sKtoB=query.value(8).toString();
    sKtoAnonK=query.value(9).toString();
    sKtoAnonB=query.value(10).toString();
    sKtoJurK=query.value(11).toString();
    sKtoJurB=query.value(12).toString();
    sKostK=query.value(13).toString();
    sKostB=query.value(14).toString();
    sKostAnonK=query.value(15).toString();
    sKostAnonB=query.value(16).toString();
    sKostJurK=query.value(17).toString();
    sKostJurB=query.value(18).toString();
    query.finish();
  }
  iProzRest=10000-iProzInt-iProzBuv-iProzLv-iProzBzv-iProzKv-iProzOv;
  if (iProzRest>0)
  {
    switch(wEbene)
    {
      case 0 : iProzBuv += iProzRest; break;
      case 1 : iProzLv  += iProzRest; break;
      case 2 : iProzBzv += iProzRest; break;
      case 3 : iProzKv  += iProzRest; break;
      case 4 : iProzOv  += iProzRest; break;
      default: iProzBuv += iProzRest; break;
    }
  }
  wProzInt->setValue((double)iProzInt/100.0l);
  wProzBuv->setValue((double)iProzBuv/100.0l);
  wProzLv->setValue((double)iProzLv/100.0l);
  wProzBzv->setValue((double)iProzBzv/100.0l);
  wProzKv->setValue((double)iProzKv/100.0l);
  wProzOv->setValue((double)iProzOv/100.0l);
  pBeitrDat();
}

QString Spende::disName()
{
  return(lDisName->text());
}

QDoubleSpinBox* Spende::pProzBox()
{
  QDoubleSpinBox *wBox;
  wBox=new QDoubleSpinBox();
  wBox->setDecimals(2);
  wBox->setMinimum(0.00);
  wBox->setMaximum(100.00);
  wBox->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
  wBox->setFixedSize(100,25);
  wBox->setSuffix(" %");
  wBox->setValue(0);
  return(wBox);
}

void Spende::pAufteilen(bool flag_bar, qint64 total1)
{
  QString zwiKonto[6];
  qint64  zwiProz[12][6];
  QString sAktGliederung,sAktTeilvon,sAktBukto;
  QDate   wAktGegruendet;
  QSqlQuery query;
  int     wAnon;
  int     wJahr,wYE,wYA;
  int     midMonStart,midMonEnd;
  int i,j;
  int iAktEbene,iAltEbene,iEigenEbene;
  double dm,d1;
  qint64 total,wGeb,wRest;

  total=total1;
  wGeb =round(wGebuehr->value() * 100.0l);
  if      (total<0) total=total+wGeb;
  else if (total>0) total=total+wGeb;
  else              wGeb=0;

  query.exec("SELECT ebene FROM verband WHERE namekurz='" + UtilSelbst() + "'");
  if (query.next()) { iEigenEbene=query.value(0).toInt(); query.finish(); }
  else              iEigenEbene=0;
  query.exec("SELECT ebene FROM verband WHERE namekurz='" + sGliederung + "'");
  if (query.next()) { iAktEbene=query.value(0).toInt(); query.finish(); }
  else              iAktEbene=0;
  wJahr=QDate::currentDate().year();

  if (iMid<1)
  {
    wAnon=1;
    sText="Anonyme ";
  }
  else
  {
    if (iMid<1000000)
       sText="MID " + QString::number(iMid) + " ";
    else
       sText="";
    query.exec("SELECT name1,name2,plz,ort FROM adresse WHERE art='M' AND key='" +
                wMitglied->getKey() + "'");
    if (query.next())
    {
      if (query.value(2).toString()=="ZZZZZ")
      {
         wAnon=1;
         if (sText=="") sText="Anonyme ";
      }
      else
      {
         wAnon=0;
         sText=sText+query.value(0).toString();
         if (query.value(1).toString().length()>0)
            sText=sText+", " + query.value(1).toString();
         sText=sText+" ";
      }
      query.finish();
      if (wAnon==0)
      {
        query.exec("SELECT sex FROM mitglied WHERE mid='" +
                    QString::number(iMid) + "'");
        if (query.next())
        {
           if (query.value(0).toString()=="U") wAnon=2;
           query.finish();
        }
      }
    }
    else
    {
      wAnon=1;
      if (sText=="") sText="Anonyme ";
    }
  }
  if (sArt=="M")
     sText=sText+"Mitgliedsbeitrag %Jahr%";
  else
  {
     sText=sText+"Spende";
     if (sGliederung!="")
     {
       sText=sText+" ";
       if      (iAktEbene==1) sText=sText+"LV ";
       else if (iAktEbene==2) sText=sText+"BZV ";
       else if (iAktEbene==3) sText=sText+"KV ";
       else if (iAktEbene==4) sText=sText+"OV ";
       sText=sText+sGliederung;
     }
  }

  if (flag_bar)
  {
    switch(wAnon)
    {
      case 0: sKost=sKostK;     sGKto=sKtoK;     break;
      case 1: sKost=sKostAnonK; sGKto=sKtoAnonK; break;
      case 2: sKost=sKostJurK;  sGKto=sKtoJurK;  break;
    }
  }
  else
  {
    switch(wAnon)
    {
      case 0: sKost=sKostB;     sGKto=sKtoB;     break;
      case 1: sKost=sKostAnonB; sGKto=sKtoAnonB; break;
      case 2: sKost=sKostJurB;  sGKto=sKtoJurB;  break;
    }
  }
  
  if (fMIDBuchung)
  {
    sHauptKto=UtilMIDKonto(iMid);
  }
  else
    sHauptKto=sGKto;

  for (i=0; i<6; i++)
  {
    auftBetrag[i]=0;
    auftKonto[i]="";
    auftKost[i]="";
  }
  for (j=0; j<12; j++)
  {
    zwiProz[j][0]=0;
    zwiProz[j][1]=iProzBuv+iProzInt;
    zwiProz[j][2]=iProzLv;
    zwiProz[j][3]=iProzBzv;
    zwiProz[j][4]=iProzKv;
    zwiProz[j][5]=iProzOv;
  }

  midMonStart=1;
  midMonEnd=12;
  if (sArt=="M" && iMid>0 && iMid<1000000)
  {
    if (total1<0)
       wJahr=dBeitrBis.addDays(1).year();
    else if (dBeitrBis >= dBeitrVon)
       wJahr=dBeitrBis.year();
    else
       wJahr=dBeitrVon.addDays(1).year();
    query.exec("SELECT eintritt,austritt FROM mitglied WHERE mid='" +
               QString::number(iMid) + "'");
    if (query.next())
    {
      wYE=query.value(0).toDate().year();
      wYA=query.value(1).toDate().year();
      if (wYA<wJahr || wYE>wJahr)
        midMonStart=midMonEnd=1;
      else
      {
        if (wYE<wJahr) midMonStart=1;
        else           midMonStart=query.value(0).toDate().month();
        if (wYA>wJahr) midMonEnd=12;
        else           midMonEnd=query.value(1).toDate().month();
      }
      query.finish();
    }
  }
  midMonStart--;
  midMonEnd--;
  if (midMonEnd<midMonStart) midMonEnd=midMonStart;

  sAktGliederung=sGliederung;
  if (sAktGliederung.length()<1) sAktGliederung=UtilSelbst();
  query.exec("SELECT gegruendet,ebene,teilvon,bukto FROM verband WHERE namekurz='" +
             sAktGliederung + "'");
  if (!query.next() && sAktGliederung!=UtilSelbst())
  {
     sAktGliederung=UtilSelbst();
     query.exec("SELECT gegruendet,ebene,teilvon,bukto FROM verband WHERE namekurz='" +
               sAktGliederung + "'");
     if (!query.next()) return;
  }
  iAltEbene=6;
  while(true)
  {
    wAktGegruendet=query.value(0).toDate();
    iAktEbene=query.value(1).toInt()+1;
    sAktTeilvon=query.value(2).toString();
    sAktBukto=query.value(3).toString();
    query.finish();
    if (iAktEbene<0 || iAktEbene>5) iAktEbene=0;

    for (i=iAltEbene-1; i>iAktEbene; i--)
    {
      zwiKonto[i]="";
      for (j=0; j<12; j++)
      {
        zwiProz[j][iAktEbene]+=zwiProz[j][i];
        zwiProz[j][i]=0;
      }
    }
    zwiKonto[i]=sAktBukto;

    if (iAktEbene>1 && sArt=="M")
    {
      if      (wAktGegruendet.year()<wJahr) j=0;
      else if (wAktGegruendet.year()>wJahr) j=12;
      else if (UtilBeitragAb()     ==1    ) j=12;
      else                                  j=wAktGegruendet.month();
      for (j--; j>=0; j--)
      {
        zwiProz[j][iAktEbene-1]+=zwiProz[j][iAktEbene];
        zwiProz[j][iAktEbene]  =0;
      }
    }

    if (iAktEbene<2) break;
    sAktGliederung=sAktTeilvon;
    if (sAktGliederung!="")
    {
      query.exec("SELECT gegruendet,ebene,teilvon,bukto FROM verband WHERE namekurz='" +
                 sAktGliederung + "'");
      if (!query.next()) sAktGliederung="";
    }
    if (sAktGliederung=="")
    {
      query.exec("SELECT gegruendet,ebene,teilvon,bukto,namekurz FROM verband WHERE ebene='0'");
      if (!query.next()) break;
      sAktGliederung=query.value(4).toString();
    }
    iAltEbene=iAktEbene;
  }
  query.finish();

  dm=(double)total/(double)100.l/(double)(midMonEnd-midMonStart+1);
  wRest=total;
  iAktEbene=iEigenEbene+1;

  for (d1=0.0l, j=midMonStart; j<=midMonEnd; j++)
  {
    d1=d1+dm*(double)(zwiProz[j][iAktEbene])/(double)10000.0l;
    zwiProz[j][iAktEbene]=0;
  }
  auftBetrag[0]=round(d1*100.0l);
  wRest=wRest-auftBetrag[0];
  if (auftBetrag[0]==0)
     iAktEbene=0;
  else
  {
    iAktEbene=1;
    auftKonto[0]=sGKto;
    auftKost[0]=sKost;
  }

  for (i=0; i<6; i++)
  {
    for (d1=0.0l, j=midMonStart; j<=midMonEnd; j++)
    {
      d1=d1+dm*(double)(zwiProz[j][i])/(double)10000.0l;
    }
    auftBetrag[iAktEbene]=round(d1*100.0l);
    wRest=wRest-auftBetrag[iAktEbene];
    if (auftBetrag[iAktEbene]!=0)
    {
      auftKonto[iAktEbene]=UtilGlKonto(zwiKonto[i]);
      auftKost[iAktEbene]=sKost;
      iAktEbene++;
    }
  }
  for (i=j=0; i<iAktEbene; i++)
  {
    if (auftKonto[i]!=sHauptKto)
    {
      if (j<i)
      {
        auftBetrag[j]=auftBetrag[i];
        auftKonto[j]=auftKonto[i];
        auftKost[j]=auftKost[i];
      }
      j++;
    }
  }
  if (wGeb!=0 && j<6)
  {
    auftBetrag[j]=wGeb*-1;
    auftKonto[i]=UtilBankGebKonto();
    auftKost[j]=sKost;
    j++;
  }
  for (; j<6; j++)
  {
    auftBetrag[j]=0;
    auftKonto[j]="";
    auftKost[j]="";
  }
  auftBetrag[0]+=wRest;
}

void Spende::pBeitrDat()
{
  QSqlQuery query;
  int       m;

  m=wMitglied->getInt();
  if (m<1 || m>999999)
  {
    wBeitrVon->setDate(QDate(1900,1,1));
    wBeitrBis->setDate(QDate(1900,1,1));
    wBeitrBis->setEnabled(false);
    return;
  }
  query.exec("SELECT bezahlt FROM mitglied WHERE mid='" +
             QString::number(m) + "'");
  if (!query.next())
  {
    wBeitrVon->setDate(QDate(1900,1,1));
    wBeitrBis->setDate(QDate(1900,1,1));
    wBeitrBis->setEnabled(false);
    return;
  }
  wBeitrVon->setDate(query.value(0).toDate());
  if (sArt=="M" && fBeitrDat)
  {
    if (fNeu)
    {
      if (iTotal<0)
      {
        if (wBeitrVon->date().month()==1)
           wBeitrBis->setDate(QDate(wBeitrVon->date().year() - 1,1,1).addDays(-1));
        else
           wBeitrBis->setDate(QDate(wBeitrVon->date().year()    ,1,1).addDays(-1));
      }
      else
      {
        if (wBeitrVon->date().month()==12)
           wBeitrBis->setDate(QDate(wBeitrVon->date().year() + 1,12,31));
        else
           wBeitrBis->setDate(QDate(wBeitrVon->date().year()    ,12,31));
      }
    }
    else
    {
      wBeitrBis->setDate(wBeitrVon->date());
    }
    wBeitrBis->setEnabled(true);
  }
  else if (fBeitrDat)
  {
    wBeitrBis->setDate(wBeitrVon->date());
    wBeitrBis->setEnabled(false);
  }
  query.finish();
}

QString Spende::pVariablen(qint64 total, QString s)
{
  QString s1;
  int     i;

  if (sArt=="M" && iMid>0 && iMid<1000000)
  {
     if (total<0)
        i=dBeitrBis.addDays(1).year();
     else if (dBeitrBis >= dBeitrVon)
        i=dBeitrBis.year();
     else
        i=dBeitrVon.addDays(1).year();
  }
  else
     i=QDate::currentDate().year();
  s1=s.replace("%Jahr%",QString::number(i));

  if (iMid>0 && iMid<1000000)
      i=iMid;
  else
      iMid=0;
  s1=s.replace("%MID%",QString::number(i));
  return(s1);
}

void Spende::aufteilung(bool flag_bar, int i,qint64 total, qint64 *teilBetrag,
                        QString *teilKonto, QString *teilKost)
{
  if (fAufteilen)
  {
    pFelder();
    pAufteilen(flag_bar,total);
    fAufteilen=false;
  }
  if (i==-1)
  {
    *teilBetrag=total;
    *teilKost=pVariablen(total, sKost);
    *teilKonto=sHauptKto;
  }
  else if (i==-2)
  {
    *teilBetrag=total;
    *teilKonto="";
    *teilKost=pVariablen(total,sText);
  }
  else if (i<-2 || i>5)
  {
    *teilBetrag=0;
    *teilKonto="";
    *teilKost="";
  }
  else
  {
    *teilBetrag=auftBetrag[i];
    *teilKonto=auftKonto[i];
    *teilKost=pVariablen(total,auftKost[i]);
  }
}

void Spende::setParam(int mid, char art, QDate bisDatum)
{
  int i;

  wMitglied->setText(QString::number(mid));
  pDisName();
  if      (art=='Z') i=0;
  else if (art=='G') i=1;
  else if (art=='A') i=2;
  else if (art=='S') i=3;
  else if (art=='M') i=4;
  else               i=0;
  wArt->setCurrentIndex(i);
  pArtChg();
  if (art=='M')
  {
    fBeitrDat=false;
    wBeitrBis->setDate(bisDatum);
  }
  fAufteilen=true;
}
