#include <QSqlQuery>

#include "spendenexport.h"
#include "utils.h"

SpendenExport::SpendenExport(QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  QGridLayout *layout = new QGridLayout;
  int    y;

  setWindowTitle("Spendenbuch Export");
  wDatei.setFlags(false,false);

  bExport.setText("&Export");
  bCancel.setText("&Abbruch");
  wVonDat.setDate(UtilGJVon());
  wVonDat.setDisplayFormat("dd.MM.yyyy");
  wVonDat.setFixedSize(wVonDat.sizeHint());
  wBisDat.setDate(QDate::currentDate());
  wBisDat.setDisplayFormat("dd.MM.yyyy");
  wBisDat.setFixedSize(wBisDat.sizeHint());
  wBestGl.setText("Bestimmte Gliederung");
  wGliederung.setEnabled(false);
  wGliederung.setText(UtilSelbst());
  wBestArt.insertItem(0,"Beitrge+Spenden");
  wBestArt.insertItem(1,"nur Beitrge");
  wBestArt.insertItem(2,"nur Spenden");
  wBestArt.setFixedSize(wBestArt.sizeHint());
  wBestKost.setFixedSize(200,25);

  y=1;
  l=new QLabel("Datum von");
  layout->addWidget(l,y,1);
  layout->addWidget(&wVonDat,y,2);
  l=new QLabel("bis");
  layout->addWidget(l,y,3);
  layout->addWidget(&wBisDat,y,4);
  y++;
  l=new QLabel("Best. Art");
  layout->addWidget(l,y,1);
  layout->addWidget(&wBestArt,y,2);
  l=new QLabel("Best.Kost.");
  layout->addWidget(l,y,3);
  layout->addWidget(&wBestKost,y,4);
  y++;
  layout->addWidget(&wBestGl,y,1,1,2);
  layout->addWidget(&wGliederung,y,3,1,2);
  y++;
  y=wDatei.FuellLayout(layout,y,1);
  layout->addWidget(&bExport,y,1);
  layout->addWidget(&bCancel,y,2);

  setLayout(layout);

  connect(&bExport,SIGNAL(clicked()),this,SLOT(pExport()));
  connect(&bCancel,SIGNAL(clicked()),this,SLOT(reject()));
  connect(&wBestGl,SIGNAL(stateChanged(int)),this,SLOT(pBestGl()));
}

SpendenExport::~SpendenExport()
{
}

void SpendenExport::pExport()
{
  if (!wDatei.isValid()) return;

  QDate wVon=wVonDat.date();
  QDate wBis=wBisDat.date();
  QDate wGJVon=UtilGJVonDatum(wVon);
  QSqlQuery query1,query2;
  QString   smk,sBestKost;
  double    wEur;
  double    vz;
  int       iMid;
  QString   cMid,wArt;
  QStringList wGlList;
  bool        flag_gliederung,flag_kost;
  int         i;

  smk=UtilMIDKonto(-1);

  flag_gliederung=false;
  if (wBestGl.checkState()==Qt::Checked &&
      wGliederung.text().simplified().length()>0)
  {
    flag_gliederung=true;
    wGlList.append(wGliederung.text());
    for (i=0; i<wGlList.size(); i++)
    {
      query1.exec("SELECT namekurz FROM verband WHERE teilvon='" +
                 wGlList[i] + "'");
      while(query1.next())
      {
        if (!wGlList.contains(query1.value(0).toString()))
           wGlList.append(query1.value(0).toString());
      }
     query1.finish();
    }
  }
  if (wBestKost.text().simplified().length()>0)
  {
    flag_kost=true;
    sBestKost=wBestKost.text().simplified().toUpper();
  }
  else
    flag_kost=false;

  wDatei.write("Datum",true,false);
  wDatei.write("EUR",true,false);
  wDatei.write("Art",true,false);
  wDatei.write("MID",true,false);
  wDatei.write("Vorname",true,false);
  wDatei.write("Nachname",true,false);
  wDatei.write("Strasse",true,false);
  wDatei.write("PLZ",true,false);
  wDatei.write("Ort",true,false);
  wDatei.newLine();

  query1.exec("SELECT jdatum,jlfd,mid,art,gliederung FROM spende WHERE jdatum>='" +
              wVon.toString("yyyy-MM-dd") + "' AND jdatum<='" +
              wBis.toString("yyyy-MM-dd") + "' ORDER BY sortname,jdatum");
  while(query1.next())
  {
    iMid=query1.value(2).toInt();
    cMid=cMid.sprintf("%08i",iMid);
    wArt=query1.value(3).toString();
    if ((wBestArt.currentIndex() == 1 && wArt!="M") ||
        (wBestArt.currentIndex() == 2 && wArt=="M"))
        continue;
    if (flag_gliederung && !wGlList.contains(query1.value(4).toString()))
       continue;
    if (flag_kost)
    {
      query2.exec("SELECT kost FROM journdat WHERE datum='" +
                query1.value(0).toString() + "' AND lfd='" +
                query1.value(1).toString() + "' LIMIT 1");
      if (!query2.next())
         continue;
      if (query2.value(0).toString().toUpper() != sBestKost)
      {
         query2.finish();
         continue;
      }
      query2.finish();
    }
    wEur=0;
    query2.exec("SELECT konto,cent FROM kgkto WHERE datum='" +
                query1.value(0).toString() + "' AND lfd='" +
                query1.value(1).toString() + "' AND kg='1' LIMIT 1");
    if(query2.next())
    {
      if (query2.value(0).toString()>="1000" &&
          query2.value(0).toString()<="1200")
         vz=1.0;
      else if (query2.value(0).toString().left(smk.length())==smk)
         vz=-1.0;
      else if (query2.value(0).toString()>="5000" &&
               query2.value(0).toString()<="5299")
         vz=-1.0;
      else
         vz=1.0;
      wEur=vz*query2.value(1).toDouble()/(double)100.0l;
      query2.finish();
    }
    wDatei.write(query1.value(0).toDate().toString("yyyy-MM-dd"),false,false);
    wDatei.write(QLocale::system().toString(wEur,'f',2),true,false);
    if (wArt=="M")
       wDatei.write("Mitgliedsbeitrag",true,false);
    else if (wArt=="Z")
       wDatei.write("Zweckgebundene Spende",true,false);
    else if (wArt=="G")
       wDatei.write("Geldspende",true,false);
    else if (wArt=="A")
       wDatei.write("Spende durch Aufwandsverzicht",true,false);
    else if (wArt=="S")
       wDatei.write("Sachspende",true,false);
    else
       wDatei.write("Zweckgebundene Spende",true,false);
    if (iMid<1000000)
       wDatei.write(QString::number(iMid),false,false);
    else
       wDatei.write("",false,false);

    query2.exec("SELECT name1,name2,strasse,plz,ort FROM adresse WHERE art='M' AND key='" +
                 cMid + "' LIMIT 1");
    if (query2.next())
    {
      wDatei.write(query2.value(1).toString(),true,false);
      wDatei.write(query2.value(0).toString(),true,false);
      wDatei.write(query2.value(2).toString(),true,false);
      wDatei.write(query2.value(3).toString(),true,false);
      wDatei.write(query2.value(4).toString(),true,true);
    }
    else
    {
       wDatei.write("",true,false);
       wDatei.write("",true,false);
       wDatei.write("",true,false);
       wDatei.write("",true,false);
       wDatei.write("",true,true);
    }
    wDatei.newLine();
  }
  wDatei.closeFile();
  QMessageBox::information(this,"Spendenexport","Fertig");
}

void SpendenExport::pBestGl()
{
  if (wBestGl.checkState() == Qt::Checked)
     wGliederung.setEnabled(true);
  else
     wGliederung.setEnabled(false);
}
