#include <QtGui>
#include <QSqlQuery>

#include "textbaustein.h"
#include "utils.h"

Textbaustein::Textbaustein(bool flag_anlegen, QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  int    y,x;
  QGridLayout *layout = new QGridLayout;

  fAnlegen=flag_anlegen;
  sNeuBez="";
  setWindowTitle("Textbausteine");
  wBez = new QLineEdit(this);
  wBez->setMaxLength(15);
  wBez->setFixedSize(150,25);
  wBeschr = new QLineEdit(this);
  wBeschr->setMaxLength(80);
  wBeschr->setFixedSize(500,25);
  wSubject = new QLineEdit(this);
  wSubject->setMaxLength(80);
  wSubject->setFixedSize(500,25);
  wInhalt = new QTextEdit(this);
  wInhalt->setAcceptRichText(true);
  bOk = new QPushButton("&Speichern",this);
  bCancel = new QPushButton("&Abbruch",this);
  bOk->setAutoDefault(false);
  bCancel->setAutoDefault(false);
  if (!fAnlegen)
  {
    bDel = new QPushButton("&Lschen",this);
    bDel->setAutoDefault(false);
  }
  wListe = new QTableView(this);
  wModel = new QStandardItemModel();
  wModel->setColumnCount(2);
  wListe->setModel(wModel);
  wListe->setEnabled(true);

  y=1;
  l=new QLabel("Bezeichnung");
  layout->addWidget(l,y,1);
  layout->addWidget(wBez,y,2);
  y++;
  l=new QLabel("Beschreibung");
  layout->addWidget(l,y,1);
  layout->addWidget(wBeschr,y,2,1,6);
  y++;
  l=new QLabel("Betreff");
  layout->addWidget(l,y,1);
  layout->addWidget(wSubject,y,2,1,6);
  y++;
  layout->addWidget(wInhalt,y,1,1,7);
  y++;
  if (fAnlegen) x=5;
  else          x=4;
  layout->addWidget(bOk,y,x); x++;
  layout->addWidget(bCancel,y,x); x++;
  if (!fAnlegen) layout->addWidget(bDel,y,x);
  y++;
  layout->addWidget(wListe,y,1,1,7);
  layout->setColumnStretch(3,1);

  setLayout(layout);

  pSetWahl(true);

  connect(wBez,SIGNAL(returnPressed()),this,SLOT(pRead()));
  connect(wBez,SIGNAL(editingFinished()),this,SLOT(pRead()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(pCancel()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pWrite()));
  if (!fAnlegen)
     connect(bDel,SIGNAL(clicked()),this,SLOT(pDelete()));
  connect(wListe,SIGNAL(clicked(QModelIndex)),this,SLOT(pAuswahl(QModelIndex)));

  pFuellListe();
}

Textbaustein::~Textbaustein()
{
  delete wBez;
  delete wBeschr;
  delete wSubject;
  delete wInhalt;
  delete bOk;
  delete bCancel;
  delete wListe;
  delete wModel;
  if (!fAnlegen) delete bDel;
  return;
}

void Textbaustein::pFuellListe()
{
  int i;
  QStandardItem *wItem;
  QSqlQuery     query;

  wModel->clear();
  wModel->setHorizontalHeaderLabels(QString("Bez.,Beschreibung").split(","));
  query.exec("SELECT bez,beschreibung FROM textbaustein ORDER BY bez");
  for (i=0; query.next(); i++)
  {
    wItem=new QStandardItem(query.value(0).toString());
    wItem->setEditable(false);
    wModel->setItem(i,0,wItem);
    wItem=new QStandardItem(query.value(1).toString());
    wItem->setEditable(false);
    wModel->setItem(i,1,wItem);
  }
  wModel->setRowCount(i);
  wListe->setColumnWidth(0,150);
  wListe->setColumnWidth(1,400);
}

void Textbaustein::pAuswahl(QModelIndex i)
{
  int wZeile = i.row();
  if (fWahl)
  {
    wBez->setText(wModel->item(wZeile,0)->text());
    pRead();
  }
}

void Textbaustein::pSetWahl(bool flag_wahl)
{
  fWahl=flag_wahl;
  bOk->setEnabled(!flag_wahl);
  bCancel->setEnabled(!flag_wahl);
  if (!fAnlegen) bDel->setEnabled(!flag_wahl);
  wBez->setEnabled(flag_wahl);
  wBeschr->setEnabled(!flag_wahl);
  wSubject->setEnabled(!flag_wahl);
  wInhalt->setEnabled(!flag_wahl);
  if (flag_wahl)
  {
    wBez->setText("");
    wBeschr->setText("");
    wSubject->setText("");
    wInhalt->setPlainText("");
  }
  if (flag_wahl)
     wBez->setFocus();
  else if (wBeschr->text().length()<1)
     wBeschr->setFocus();
  else
     bCancel->setFocus();
}

void Textbaustein::pFelder()
{
  sBez=wBez->text().replace("'","`").simplified().toUpper();
  sBeschr=wBeschr->text().replace("'","`").simplified();
  sSubject=wSubject->text().replace("'","`").simplified();
  sInhalt=wInhalt->toPlainText().replace("'","`");
}

void Textbaustein::pRead()
{
  QSqlQuery query;

  pFelder();
  wBez->setText(sBez);
  if (!fWahl || sBez.length()<1) return;

  query.exec("SELECT beschreibung,subject,inhalt FROM textbaustein WHERE bez='" + sBez + "'");
  if (query.next())
  {
    wBeschr->setText(query.value(0).toString());
    wSubject->setText(query.value(1).toString());
    wInhalt->setPlainText(query.value(2).toString());
    query.finish();
  }
  pSetWahl(false);
}

void Textbaustein::pCancel()
{
  if (fWahl || fAnlegen)
    reject();
  else
  {
    pSetWahl(true);
  }
}

void Textbaustein::pWrite()
{
  QSqlQuery query;
  QString q;

  pFelder();
  if (fWahl || sBez.length()<1) return;
  query.exec("UPDATE textbaustein SET beschreibung='" + sBeschr +
             "',subject='" + sSubject + + "',inhalt='" + sInhalt +
             "' WHERE bez = '" + sBez + "'");
  if (query.numRowsAffected()<1)
  {
    query.exec("INSERT INTO textbaustein (bez,beschreibung,subject,inhalt) VALUES ('" +
          sBez + "','" + sBeschr + "','" + sSubject + "','" + sInhalt + "')");
  }
  if (fAnlegen)
  {
    sNeuBez=sBez;
    accept();
    return;
  }
  pSetWahl(true);
  pFuellListe();
}

void Textbaustein::pDelete()
{
  QSqlQuery query;

  if (fWahl || fAnlegen) return;
  if (UtilQDelete())
  {
    pFelder();
    query.exec("DELETE FROM textbaustein WHERE bez='" + sBez + "'");
    pSetWahl(true);
    pFuellListe();
  }
}

void Textbaustein::closeEvent(QCloseEvent *e)
{
  if (fWahl)
    e->accept();
  else
  {
    pCancel();
    e->ignore();
  }
}

void Textbaustein::keyPressEvent(QKeyEvent *e)
{
  if (e->key()==Qt::Key_Escape)
  {
    if (fWahl)
       close();
    else
       pCancel();
    e->ignore();
  }
  e->accept();
}

QString Textbaustein::angelegt()
{
  return(sNeuBez);
}
