#include <QtGui>
#include <QtSql>

#include "utils.h"
#include "wkonto.h"
#include "midsuche.h"
#include "lssuche.h"
#include "kplsuche.h"
#include "gliederungswahl.h"

extern int flag_db;

WKonto::WKonto(QLabel *Text, bool flag_anlegen, QWidget *parent) :
               QLineEdit(parent)
{
  lText=Text;

  fAnlegen=flag_anlegen;
  setFixedSize(150,25);
  setWhatsThis("Hier wird ein Buchungskonto eingegeben. Dabei gibt es folgende Suchmglichkeiten:\nMName: Suche nach einem Mitglied mit dem Nachnamen 'Name'\nMnnnnn: Suche nach einem mitglied mit der Postleitzahl nnnnn\nLName : Such nach einem Lieferanten mit diesem Namen\nGName : Suche nach einer Gliederung mit diesem Namen\nKText : Suche im Kontanplan nach diesem Text");
  connect(this,SIGNAL(textChanged(const QString)),this,SLOT(pChanged()));
  connect(this,SIGNAL(returnPressed()),this,SLOT(pEnter()));
}

void WKonto::setLabel(QLabel *Text)
{
  lText=Text;
}

void WKonto::pChanged()
{
  QString s;
  if (lText!=NULL && flag_db)
  {
    s=text();
    if (s.length()<1)
       lText->setText("");
    else
       lText->setText(UtilKtoText(s));
  }
}

QString WKonto::text()
{
  QString s=QLineEdit::text().simplified().toUpper().replace("'","`");

  if (s.length()<2 || !s[0].isDigit() || !s[1].isDigit())
     return("");
  else
     return(s);
}

void WKonto::pEnter()
{
  int       r;
  QString   s=QLineEdit::text().simplified().toUpper().replace("'","`");
  QSqlQuery query;

  flag_wahl=false;

  if (s.left(1) == "M" && s.right(1)!="?")
  {
    MIDSuche pMIDSuche(fAnlegen);
    r=pMIDSuche.exec(s);
    if (r==QDialog::Accepted)
    {
      setText(UtilMIDKonto(pMIDSuche.auswahl()));
      flag_wahl=true;
    }
    return;
  }
  if (s.left(1) == "L" && s.right(1)!="?")
  {
    LsSuche pLsSuche(fAnlegen);
    r=pLsSuche.exec(s.mid(1) + "?");
    if (r==QDialog::Accepted)
    {
      setText(UtilLsKonto(pLsSuche.auswahl()));
      flag_wahl=true;
    }
    return;
  }
  if (s.left(1) == "G" && s.right(1)!="?")
  {
    GliederungsWahl pGliederungsWahl(4);
    r=pGliederungsWahl.exec();
    if (r==QDialog::Accepted)
    {
      s=pGliederungsWahl.text();
      query.exec("SELECT bukto FROM verband WHERE namekurz='" + s + "'");
      if (query.next())
      {
        setText(UtilGlKonto(query.value(0).toString()));
        flag_wahl=true;
        query.finish();
      }
    }
    return;
  }
  if (s.left(1) == "K" || s.right(1)=="?")
  {
    KplSuche pKplSuche(fAnlegen);
    if (s.right(1) == "?")
       r=pKplSuche.exec(s.left(s.length()-1));
    else
       r=pKplSuche.exec(s.mid(1));
    if (r==QDialog::Accepted)
    {
      setText(pKplSuche.auswahl());
      flag_wahl=true;
    }
    return;
  }
}

void WKonto::focusOutEvent(QFocusEvent *e)
{
  QString s=QLineEdit::text().simplified().toUpper().replace("'","`");
  if (e->reason()==Qt::TabFocusReason ||
      e->reason()==Qt::BacktabFocusReason)
  {
    if (s.left(1)=="M" || s.left(1)=="L" || s.left(1)=="G" || s.left(1)=="K" ||
        s.right(1)=="?")
    {
      pEnter();
      if (flag_wahl)
      {
        QLineEdit::focusOutEvent(e);
      }
      else
      {
        e->accept();
        setFocus();
      }
      return;
    }
  }
  QLineEdit::focusOutEvent(e);
}
