#include <QtGui>
#include <QtSql>

#include "zahlbearb.h"
#include "utils.h"
#include "smtp.h"

ZahlungBearb::ZahlungBearb(QString art, QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  int    y;
  QVBoxLayout *topLayout = new QVBoxLayout;
  QGridLayout *layout = new QGridLayout;
  QHBoxLayout *buttonLayout = new QHBoxLayout;

  sArt=art;

  setWindowTitle("Bearbeitung der Zahlungen");
  lArt=new QLabel();
  if      (art=="L") lArt->setText("Lastschriftankndigung");
  else if (art=="l") lArt->setText("Lastschriftankndigung Neumitglieder");
  else if (art=="R") lArt->setText("Vorauskasse");
  else if (art=="r") lArt->setText("Vorauskasse Neumitglieder");
  else if (art=="D") lArt->setText("DTAUS-Datei");
  else if (art=="G") lArt->setText("Geplatzte Lastschrift");
  else if (art=="1") lArt->setText("1. Mahnung");
  else if (art=="2") lArt->setText("2. Mahnung");
  else if (art=="3") lArt->setText("3. Mahnung");
  wMid = new WMitglied(false,this);
  lName = new QLabel(this);
  lName->setFixedSize(300,25);
  if (sArt == "D")
  {
     lBaustein=lBetrag=NULL;
     wForm=NULL;
     wBetrag=new QDoubleSpinBox(this);
     wBetrag->setDecimals(2);
     wBetrag->setMinimum(0.00);
     wBetrag->setMaximum(99999999.99);
     wBetrag->setSuffix(" EUR");
     wBetrag->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
     wBetrag->setFixedSize(150,25);
  }
  else
  {
    wForm = new QComboBox(this);
    wForm->insertItem(0,"Email");
    wForm->insertItem(1,"Brief");
    wForm->setFixedSize(wForm->sizeHint());
    lBaustein = new QLabel(this);
    lBaustein->setFixedSize(150,25);
    wBetrag=NULL;
    lBetrag = new QLabel(this);
    lBetrag->setFixedSize(150,25);
  }
  wSperren = new QCheckBox("Sperren",this);
  if (sArt == "D")
     bTest=NULL;
  else
     bTest=new QPushButton("&Testen");
  bOk = new QPushButton("&Speichern");
  bCancel = new QPushButton("&Abbruch");
  bOk->setAutoDefault(false);
  bCancel->setAutoDefault(false);

  wListe = new QTableView;
  wModel = new QStandardItemModel();
  wModel->setColumnCount(4);
  wListe->setModel(wModel);
  wListe->setEnabled(true);
  wListe->setFixedSize(670,200);

  y=1;
  layout->addWidget(lArt,y,1,1,4);
  y++;
  l=new QLabel("Mitglied");
  layout->addWidget(l,y,1);
  layout->addWidget(wMid,y,2);
  layout->addWidget(lName,y,3,1,2);
  y++;
  if (sArt=="D")
  {
    l=new QLabel("Betrag");
    layout->addWidget(l,y,1);
    layout->addWidget(wBetrag,y,2);
    y++;
  }
  else
  {
    l=new QLabel("Betrag");
    layout->addWidget(l,y,1);
    layout->addWidget(lBetrag,y,2);
    y++;
    l=new QLabel("Mail/Brief");
    layout->addWidget(l,y,1);
    layout->addWidget(wForm,y,2);
    l=new QLabel("Textbaustein");
    layout->addWidget(l,y,3);
    layout->addWidget(lBaustein,y,4);
    y++;
  }
  layout->setColumnStretch(5,1);

  buttonLayout->addWidget(wSperren);
  buttonLayout->addStretch(1);
  if (bTest!=NULL) buttonLayout->addWidget(bTest);
  buttonLayout->addWidget(bOk);
  buttonLayout->addWidget(bCancel);
  topLayout->addLayout(layout);
  topLayout->addLayout(buttonLayout);
  topLayout->addWidget(wListe);
  setLayout(topLayout);

  pFuellListe();
  pSetWahl(true);

  connect(wMid,SIGNAL(finished()),this,SLOT(pRead()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(pCancel()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pWrite()));
  connect(wListe,SIGNAL(clicked(QModelIndex)),this,SLOT(pAuswahl(QModelIndex)));
  if (wForm!=NULL)
     connect(wForm,SIGNAL(currentIndexChanged(int)),this,SLOT(pBaustein(int)));
  if (bTest!=NULL)
     connect(bTest,SIGNAL(clicked()),this,SLOT(pTest()));
}

ZahlungBearb::~ZahlungBearb()
{
  delete wMid;
  delete wSperren;
  delete lArt;
  delete lName;
  delete wModel;
  delete wListe;
  if (lBetrag  !=NULL) delete lBetrag;
  if (lBaustein!=NULL) delete lBaustein;
  if (wForm    !=NULL) delete wForm;
  if (wBetrag  !=NULL) delete wBetrag;
  if (bTest    !=NULL) delete bTest;
  return;
}

void ZahlungBearb::pSetWahl(bool flag_wahl)
{
  fWahl=flag_wahl;
  wMid->setEnabled(flag_wahl);
  bOk->setEnabled(!flag_wahl);
  bCancel->setEnabled(!flag_wahl);
  wSperren->setEnabled(!flag_wahl);
  if (wForm!=NULL)
  {
     if (flag_wahl || sMail.length()<1 || sBausteinM.length()<1 ||
         sBausteinB.length()<1)
        wForm->setEnabled(false);
     else
        wForm->setEnabled(true);
  }
  if (wBetrag!=NULL) wBetrag->setEnabled(!flag_wahl);
  if (bTest!=NULL) bTest->setEnabled(!flag_wahl);
  if (flag_wahl)
  {
    wMid->setText("");
    wSperren->setChecked(Qt::Unchecked);
    lName->setText("");
    if (wBetrag!=NULL)   wBetrag->setValue(0);
    if (wForm!=NULL)     wForm->setCurrentIndex(0);
    if (lBaustein!=NULL) lBaustein->setText("");
    if (lBetrag!=NULL)   lBetrag->setText("");
    iCent=0;
    wMid->setFocus();
  }
  else
     bCancel->setFocus();
}

void ZahlungBearb::pFelder()
{
  sMid=QString::number(wMid->getInt());
  if (wSperren->checkState() == Qt::Checked)
     sSperren="J";
  else
     sSperren="";
  if (wForm==NULL)
     sForm="D";
  else if (wForm->currentIndex()==0)
     sForm="M";
  else
     sForm="B";
  if (wBetrag==NULL)
     sBetrag="";
  else
     sBetrag=QString::number((qint64)round(wBetrag->value()+(double)(100.0l)));
}

void ZahlungBearb::pRead()
{
  QSqlQuery query,query2;
  QString   s;

  if (!fWahl) return;
  pFelder();
  if (sMid.length()<1) return;

  query.exec("SELECT mid,art,sperren,form,bausteinm,bausteinb,cent,done FROM zahllauf WHERE mid='" + sMid + "'");
  if (!query.next() || query.value(1).toString()!=sArt) return;
  iCent=query.value(6).toLongLong();

  s=s.sprintf("%08i",query.value(0).toInt());
  query2.exec("SELECT name1,name2,plz,ort,mail FROM adresse WHERE art='M' AND key='" +
              s + "'");
  if (query2.next())
  {
    s=query2.value(0).toString();
    if (query2.value(1).toString().length()>0)
    s=s+", " + query2.value(1).toString();
    if (query2.value(2).toString().length()>0 ||
        query2.value(3).toString().length()>0)
       s=s+", " + query2.value(2).toString() + " " +
         query2.value(3).toString();
    sMail=query2.value(4).toString();
  }
  else
     s=sMail="";

  lName->setText(s);
  query2.finish();

  if (query.value(2).toString() == "J")
     wSperren->setChecked(Qt::Checked);
  else
     wSperren->setChecked(Qt::Unchecked);
  if (sArt=="D")
  {
    wBetrag->setValue((double)(query.value(6).toLongLong()) / 100.0l);
  }
  else
  {
    if (query.value(3).toString() == "M")
       wForm->setCurrentIndex(0);
    else
       wForm->setCurrentIndex(1);
    lBetrag->setText(QLocale::system().toString((double)query.value(6).toLongLong()/100.0,'f',2) + " EUR");
  }
  sBausteinM=query.value(4).toString();
  sBausteinB=query.value(5).toString();

  if (sArt!="D" && lBaustein!=NULL)
  {
    fWahl=false;
    pBaustein(wForm->currentIndex());
  }
  
  query.finish();
  pSetWahl(false);
}

void ZahlungBearb::pCancel()
{
  if (fWahl)
    reject();
  else
  {
    pSetWahl(true);
  }
}

void ZahlungBearb::pWrite()
{
  QSqlQuery query;
  QString q;

  pFelder();
  if (fWahl) return;

  if (sArt=="D")
    query.exec("UPDATE zahllauf SET manuell='J',sperren='" + sSperren +
                "',cent='" + sBetrag + "' WHERE mid='" + sMid + "'");
  else
    query.exec("UPDATE zahllauf SET manuell='J',sperren='" + sSperren +
                "',form='" + sForm + "' WHERE mid='" + sMid + "'");

  pFuellListe();
  pSetWahl(true);
}

void ZahlungBearb::pFuellListe()
{
  int i;
  QStandardItem *wItem;
  QSqlQuery query,query2;
  QString   s;

  wModel->clear();
  wModel->setHorizontalHeaderLabels(QString("MID,Name,Betrag,Status").split(","));
  query.exec("SELECT mid,cent,sperren,done FROM zahllauf WHERE art='" + sArt +
             "' ORDER BY mid");
  for (i=0; query.next(); i++)
  {
    s=s.sprintf("%08i",query.value(0).toInt());
    query2.exec("SELECT name1,name2,plz,ort FROM adresse WHERE art='M' AND key='" +
                s + "'");
    if (query2.next())
    {
      s=query2.value(0).toString();
      if (query2.value(1).toString().length()>0)
         s=s+", " + query2.value(1).toString();
      if (query2.value(2).toString().length()>0 ||
          query2.value(3).toString().length()>0)
         s=s+", " + query2.value(2).toString() + " " +
           query2.value(3).toString();
    }
    else
       s="";
    wItem=new QStandardItem(query.value(0).toString());
    wItem->setEditable(false);
    wModel->setItem(i,0,wItem);
    wItem=new QStandardItem(s);
    wItem->setEditable(false);
    wModel->setItem(i,1,wItem);
    wItem=new QStandardItem(QLocale::system().toString((double)query.value(1).toLongLong()/100.0,'f',2));
    wItem->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
    wItem->setEditable(false);
    wModel->setItem(i,2,wItem);
    if (query.value(2).toString() == "J")
       wItem=new QStandardItem("Gesperrt");
    else if (query.value(3).toString() == "J")
       wItem=new QStandardItem("O.K.");
    else
       wItem=new QStandardItem("");
    wItem->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
    wItem->setEditable(false);
    wModel->setItem(i,3,wItem);
  }
  wModel->setRowCount(i);
  wListe->setColumnWidth(0,75);
  wListe->setColumnWidth(1,350);
  wListe->setColumnWidth(2,100);
  wListe->setColumnWidth(3,80);
}

void ZahlungBearb::pAuswahl(QModelIndex i)
{
  int wZeile = i.row();

  if (fWahl)
  {
    wMid->setText(wModel->item(wZeile,0)->text());
    pRead();
  }
}

void ZahlungBearb::closeEvent(QCloseEvent *e)
{
  if (fWahl)
    e->accept();
  else
  {
    pCancel();
    e->ignore();
  }
}

void ZahlungBearb::keyPressEvent(QKeyEvent *e)
{
  if (e->key()==Qt::Key_Escape)
  {
    if (fWahl)
       close();
    else
       pCancel();
    e->ignore();
  }
  e->accept();
}

void ZahlungBearb::pBaustein(int i)
{
  if (fWahl)
      lBaustein->setText("");
  else if (i>0)
  {
    if (sBausteinM.length()>0 && sBausteinB.length()<1 && sMail.length()>0)
       wForm->setCurrentIndex(0);
    else
       lBaustein->setText(sBausteinB);
  }
  else
  {
    if (sBausteinB.length()>0 && (sMail.length()<1 || sBausteinM.length()<1))
       wForm->setCurrentIndex(1);
    else
       lBaustein->setText(sBausteinM);
  }
}

void ZahlungBearb::pTest()
{
  bool ok;
  QString wAdr;
  Smtp    wSmtp;

  if (bTest==NULL) return;
  pFelder();
  if (sForm!="M")
  {
     QMessageBox::critical(this,"Test der Zahlung","Derzeit ist nur Email implementiert");
     return;
  }
  wAdr=QInputDialog::getText(this,"Test der Zahlung","Email Adresse",
                                QLineEdit::Normal,"",&ok);
  if (!ok) return;
  
  ok=wSmtp.setText(sBausteinM,"M",wMid->getKey(),wMid->getInt(),
                   QDate::currentDate().year(),iCent);
  if (ok)
  {
    wSmtp.setTo(wAdr);
    ok=wSmtp.sendMail();
  }
  if (ok)
    QMessageBox::information(this,"Test der Zahlung","Email wurde verschickt");
  else
    QMessageBox::critical(this,"Test der Zahlung","Email konnte nicht verschickt werden");
  return;
}
