#include <QtSql>
#include <QtGui>

#include "zahlbrief.h"
#include "utils.h"
#include "smtp.h"

ZahlungBrief::ZahlungBrief(QWidget *parent) : QDialog(parent)
{
  QGridLayout *layout;
  QLabel      *l;
  int y;

  setWindowTitle("Zahlungen - Briefe");
  layout=new QGridLayout();

  fLauf=fAbbruch=false;

  wCheckRech    = new QCheckBox("&Vorkasse",this);
  wCheckRechNeu = new QCheckBox("Vorkasse &Neumitglieder",this);
  wCheckLast    = new QCheckBox("&Lastschrifthinweis",this);
  wCheckLastNeu = new QCheckBox("Lastschrifthinweis N&eumitglieder",this);
  wCheckPlatz   = new QCheckBox("&Geplatzte Lastschrift",this);
  wCheckMahn1   = new QCheckBox("&1. Mahnung",this);
  wCheckMahn2   = new QCheckBox("&2. Mahnung",this);
  wCheckMahn3   = new QCheckBox("&3. Mahnung",this);
  wCheckDup     = new QCheckBox("&Bereits erstellte Briefe neu drucken",this);
  lMid          = new QLabel(this);
  bOk     = new QPushButton("&OK",this);
  bCancel = new QPushButton("&Abbruch",this);

  y=1;
  layout->addWidget(wCheckRech,y,1,1,3); y++;
  layout->addWidget(wCheckRechNeu,y,1,1,3); y++;
  layout->addWidget(wCheckLast,y,1,1,3); y++;
  layout->addWidget(wCheckLastNeu,y,1,1,3); y++;
  layout->addWidget(wCheckPlatz,y,1,1,3); y++;
  layout->addWidget(wCheckMahn1,y,1,1,3); y++;
  layout->addWidget(wCheckMahn2,y,1,1,3); y++;
  layout->addWidget(wCheckMahn3,y,1,1,3); y++;
  layout->addWidget(wCheckDup,y,1,1,3); y++;
  layout->addWidget(bOk,y,1);
  layout->addWidget(bCancel,y,2);
  layout->addWidget(lMid,y,3);
  // layout->setColumnStretch(3,1);

  wCheckRech->setCheckState(Qt::Checked);
  wCheckRechNeu->setCheckState(Qt::Checked);
  wCheckLast->setCheckState(Qt::Checked);
  wCheckLastNeu->setCheckState(Qt::Checked);
  wCheckPlatz->setCheckState(Qt::Checked);
  wCheckMahn1->setCheckState(Qt::Checked);
  wCheckMahn2->setCheckState(Qt::Checked);
  wCheckMahn3->setCheckState(Qt::Checked);
  wCheckDup->setCheckState(Qt::Unchecked);

  setLayout(layout);

  connect(bOk,SIGNAL(clicked()),this,SLOT(pLauf()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(reject()));
  connect(wCheckDup,SIGNAL(stateChanged(int)),this,SLOT(pDupWarn()));
}

ZahlungBrief::~ZahlungBrief()
{
  delete wCheckRech;
  delete wCheckRechNeu;
  delete wCheckLast;
  delete wCheckLastNeu;
  delete wCheckPlatz;
  delete wCheckMahn1;
  delete wCheckMahn2;
  delete wCheckMahn3;
  delete wCheckDup;
  delete bOk;
  delete bCancel;
  return;
}

void ZahlungBrief::reject()
{
  if (fLauf)
      fAbbruch=true;
  else
      QDialog::reject();
}

void ZahlungBrief::pDupWarn()
{
  if (wCheckDup->checkState()==Qt::Checked)
  {
    QMessageBox::warning(this,"Zahlungen - Briefe","Achtung: Wenn diese Option aktiviert ist, ist es nicht ausgeschlossen, dass Briefe doppelt gedruckt werden!");
  }
}

void ZahlungBrief::closeEvent(QCloseEvent *e)
{
  if (fLauf) fAbbruch=true;
  else       e->accept();
}
void ZahlungBrief::pFelder()
{
  fRech=fRechNeu=fLast=fLastNeu=fPlatz=fMahn1=fMahn2=fMahn3=fDup=false;
  if (wCheckRech->checkState()==Qt::Checked)    fRech=true;
  if (wCheckRechNeu->checkState()==Qt::Checked) fRechNeu=true;
  if (wCheckLast->checkState()==Qt::Checked)    fLast=true;
  if (wCheckLastNeu->checkState()==Qt::Checked) fLastNeu=true;
  if (wCheckPlatz->checkState()==Qt::Checked)   fPlatz=true;
  if (wCheckMahn1->checkState()==Qt::Checked)   fMahn1=true;
  if (wCheckMahn2->checkState()==Qt::Checked)   fMahn2=true;
  if (wCheckMahn3->checkState()==Qt::Checked)   fMahn3=true;
  if (wCheckDup->checkState()==Qt::Checked)     fDup=true;
}

void ZahlungBrief::pSetLauf(bool flag_lauf)
{
  fLauf=flag_lauf;
  wCheckRech->setEnabled(!fLauf);
  wCheckRechNeu->setEnabled(!fLauf);
  wCheckLast->setEnabled(!fLauf);
  wCheckLastNeu->setEnabled(!fLauf);
  wCheckPlatz->setEnabled(!fLauf);
  wCheckMahn1->setEnabled(!fLauf);
  wCheckMahn2->setEnabled(!fLauf);
  wCheckMahn3->setEnabled(!fLauf);
  bOk->setEnabled(!fLauf);
}

void ZahlungBrief::pLauf()
{
  QSqlQuery query,query2;
  QFont     wFontAdr,wFontSubj,wFontText;
  QString   s,s1,cMid;
  Smtp      wSmtp;
  int       iMid;
  int       wPosX,wPosY,wStepY,wSeite;
  int       iRech,iRechNeu,iLast,iLastNeu,iPlatz,iMahn1,iMahn2,iMahn3;
  int       iAdrL,iAdrT,iAdrPt,iTextL,iTextT,iTextR,iTextB,iTextPt;
  QString   sText,sSubject,sKonto,sBank,sBlz;
  QDate     dHeute = QDate::currentDate();

  pFelder();
  pSetLauf(true);
  fAbbruch=false;
  iRech=iRechNeu=iLast=iLastNeu=iPlatz=iMahn1=iMahn2=iMahn3=0;

  iAdrL=iAdrT=iAdrPt=iTextL=iTextT=iTextR=iTextB=iTextPt=0;
  query.exec("SELECT wert FROM parameter WHERE key='Mail/AdrL'");
  if (query.next()) iAdrL=query.value(0).toInt();
  query.exec("SELECT wert FROM parameter WHERE key='Mail/AdrT'");
  if (query.next()) iAdrT=query.value(0).toInt();
  query.exec("SELECT wert FROM parameter WHERE key='Mail/AdrPt'");
  if (query.next()) iAdrPt=query.value(0).toInt();
  query.exec("SELECT wert FROM parameter WHERE key='Mail/L'");
  if (query.next()) iTextL=query.value(0).toInt();
  query.exec("SELECT wert FROM parameter WHERE key='Mail/R'");
  if (query.next()) iTextR=query.value(0).toInt();
  query.exec("SELECT wert FROM parameter WHERE key='Mail/T'");
  if (query.next()) iTextT=query.value(0).toInt();
  query.exec("SELECT wert FROM parameter WHERE key='Mail/B'");
  if (query.next()) iTextB=query.value(0).toInt();
  query.exec("SELECT wert FROM parameter WHERE key='Mail/Pt'");
  if (query.next()) iTextPt=query.value(0).toInt();

  QPrinter *wDrucker = new QPrinter(QPrinter::HighResolution);
  wDrucker->setOrientation(QPrinter::Portrait);
  QPrintDialog wDiag(wDrucker,this);
  if (wDiag.exec() != QDialog::Accepted) return;
  wDrucker=wDiag.printer();
  wDrucker->setFullPage(true);
  QPainter *wPainter = new QPainter();

  wFontAdr.setFamily("Helvetica");
  wFontAdr.setPointSize(iAdrPt);
  wFontSubj.setFamily("Helvetica");
  wFontSubj.setPointSize(iTextPt);
  wFontSubj.setWeight(QFont::Bold);
  wFontText.setFamily("Helvetica");
  wFontText.setPointSize(iTextPt);

  iAdrL=iAdrL  * wDrucker->logicalDpiX()  * 10 / 254;
  iAdrT=iAdrT  * wDrucker->logicalDpiY()  * 10 / 254;
  iTextL=iTextL * wDrucker->logicalDpiX() * 10 / 254;
  iTextR=iTextR * wDrucker->logicalDpiX() * 10 / 254;
  iTextT=iTextT * wDrucker->logicalDpiY() * 10 / 254;
  iTextB=iTextB * wDrucker->logicalDpiY() * 10 / 254;

  wSeite=0;
  query.exec("SELECT mid,art,bausteinb,cent,done,sperren FROM zahllauf WHERE form='B' ORDER BY mid");
  while(query.next() && !fAbbruch)
  {
    QCoreApplication::processEvents();
    s=query.value(1).toString();
    iMid=query.value(0).toInt();
    cMid=cMid.sprintf("%08i",iMid);

    if      (s=="L" && iLast   ==0) iLast   =1;
    else if (s=="l" && iLastNeu==0) iLastNeu=1;
    else if (s=="R" && iRech   ==0) iRech   =1;
    else if (s=="r" && iRechNeu==0) iRechNeu=1;
    else if (s=="G" && iPlatz  ==0) iPlatz  =1;
    else if (s=="1" && iMahn1  ==0) iMahn1  =1;
    else if (s=="2" && iMahn2  ==0) iMahn2  =1;
    else if (s=="3" && iMahn3  ==0) iMahn3  =1;

    if (query.value(5).toString()=="J") continue;
    if (query.value(4).toString()=="J" && !fDup) continue;

    query2.exec("SELECT subject,inhalt FROM textbaustein WHERE bez='" +
               query.value(2).toString() + "'");
    if (!query2.next()) continue;
    sSubject=query2.value(0).toString();
    sText=query2.value(1).toString();
    query2.finish();

    if ((s=="L" && fLast)    ||
        (s=="l" && fLastNeu) ||
        (s=="R" && fRech)    ||
        (s=="r" && fRechNeu) ||
        (s=="G" && fPlatz)   ||
        (s=="1" && fMahn1)   ||
        (s=="2" && fMahn2)   ||
        (s=="3" && fMahn3)     )
    {
      lMid->setText(query.value(0).toString());

      query2.exec("select nummer,blz,bank FROM bankverbindung WHERE art='M' AND key='" +
                  cMid + "'");
      if (query2.next())
      {
        sKonto="XXXXX" + query2.value(0).toString().right(4);
        sBlz=query2.value(1).toString();
        sBank=query2.value(2).toString();
        query2.finish();
      }
      else
      {
        sKonto="";
        sBlz="";
        sBank="";
      }
      query2.exec("select name2,name1,strasse,plz,ort,land,mail FROM adresse WHERE art='M' AND key='" +
                  cMid + "'");
      if (!query2.next()) continue;
      sSubject=sSubject.replace("%vname%",query2.value(0).toString())
                       .replace("%nname%",query2.value(1).toString())
                       .replace("%mid%",QString::number(iMid))
                       .replace("%strasse%",query2.value(2).toString())
                       .replace("%plz%",query2.value(3).toString())
                       .replace("%ort%",query2.value(4).toString())
                       .replace("%land%",query2.value(5).toString())
                       .replace("%jahr%",QString::number(dHeute.year()))
                       .replace("%betrag%",
                QLocale::system().toString((double)query.value(3).toLongLong() /
                                           (double)100.0,'f',2))
                       .replace("%kontonr%",sKonto)
                       .replace("%kontoblz%",sBlz)
                       .replace("%kontobank%",sBank);
      sText=sText.replace("%vname%",query2.value(0).toString())
                 .replace("%nname%",query2.value(1).toString())
                 .replace("%mid%",QString::number(iMid))
                 .replace("%strasse%",query2.value(2).toString())
                 .replace("%plz%",query2.value(3).toString())
                 .replace("%ort%",query2.value(4).toString())
                 .replace("%land%",query2.value(5).toString())
                 .replace("%jahr%",QString::number(dHeute.year()))
                 .replace("%betrag%",
                QLocale::system().toString((double)query.value(3).toLongLong() /
                                           (double)100.0,'f',2))
                 .replace("%kontonr%",sKonto)
                 .replace("%kontoblz%",sBlz)
                 .replace("%kontobank%",sBank);

      if (wSeite>0) wDrucker->newPage();
      else          wPainter->begin(wDrucker);
      wSeite++;
      wPainter->setFont(wFontAdr);
      wStepY=wPainter->fontMetrics().height();
      wPosY=iAdrT+wStepY;
      wStepY=wStepY*3/2;
      wPosX=iAdrL;
      s=query2.value(0).toString();
      if (s.length()>0) s=s+" ";
      s=s+query2.value(1).toString();
      wPainter->drawText(wPosX,wPosY,s);
      wPosY+=wStepY;
      wPainter->drawText(wPosX,wPosY,query2.value(2).toString());
      wPosY+=wStepY;
      wPainter->drawText(wPosX,wPosY,query2.value(3).toString() + " " +
                                     query2.value(4).toString());

      wPainter->setFont(wFontSubj);
      wStepY=wPainter->fontMetrics().height();
      wPosX=iTextL;
      wPosY=iTextT+wStepY;
      wPainter->drawText(wPosX,wPosY,sSubject);
      wPosY=wPosY+2*wStepY;

      wPainter->setFont(wFontText);
      wPainter->drawText(iTextL,wPosY,
                         wDrucker->pageRect().width() - iTextR - iTextL,
                         wDrucker->pageRect().height() - iTextB - wPosY,
                         Qt::TextWordWrap|Qt::AlignTop, sText);
      query2.finish();
      query2.exec("UPDATE zahllauf SET done='J' WHERE mid='" +
                   query.value(0).toString() + "'");
    }
  }
  lMid->setText("");
  query.finish();
  if (wSeite>0) wPainter->end();
  pSetLauf(false);
  if (fAbbruch)
     QMessageBox::information(this,"Zahlungen Brief","Der Lauf wurde abgebrochen");
  else
  {
     query.exec("SELECT wert FROM parameter WHERE key='Status/Zahlungen'");
     if (query.next())
     {
       s=query.value(0).toString();
       query.finish();
       if (iRech   ==1 && s.length()> 2) s=s.mid(0, 2) + "D" + s.mid(3);
       if (iRechNeu==1 && s.length()> 4) s=s.mid(0, 4) + "D" + s.mid(5);
       if (iLast   ==1 && s.length()> 6) s=s.mid(0, 6) + "D" + s.mid(7);
       if (iLastNeu==1 && s.length()> 8) s=s.mid(0, 8) + "D" + s.mid(9);
       if (iPlatz  ==1 && s.length()>10) s=s.mid(0,10) + "D" + s.mid(11);
       if (iMahn1  ==1 && s.length()>12) s=s.mid(0,12) + "D" + s.mid(13);
       if (iMahn2  ==1 && s.length()>14) s=s.mid(0,14) + "D" + s.mid(15);
       if (iMahn3  ==1 && s.length()>16) s=s.mid(0,16) + "D" + s.mid(17);
       query.exec("UPDATE parameter SET wert='" + s +
                  "' WHERE key='Status/Zahlungen'");
     }
     QMessageBox::information(this,"Zahlungen Briefe",
                  QString::number(wSeite) + " Seiten wurden gedruckt");
  }
  accept();
}
