#include <QtGui>

#include "utils.h"
#include "zahldtaus.h"

ZahlungDtaus::ZahlungDtaus(QWidget *parent)
{
  int y;
  QLabel *l;
  QSqlQuery query;

  fTempTable=fInhalt=fFehler=fErstellen=false;

  QGridLayout *layout = new QGridLayout;

  setWindowTitle("DTAUS Datei generieren");
  dBudat=QDate(1900,1,1);
  wBudat=NULL;
  query.exec("SELECT wert FROM parameter WHERE key='Zahlungen/KontoDtaus'");
  if (query.next())
  {
    if (query.value(0).toString().length()>1)
    {
      dBudat=QDate::currentDate();
    }
    query.finish();
  }

  wEName=new QLineEdit(this);
  wEName->setFixedSize(230,25);  wEName->setMaxLength(27);
  wEKonto=new QLineEdit(this);
  wEKonto->setFixedSize(110,25); wEKonto->setMaxLength(12);
  wEBlz=new QLineEdit(this);
  wEBlz->setFixedSize(75,25);    wEBlz->setMaxLength(8);
  wCheckDup=new QCheckBox("&Bereits erstellte Zahlungen neu erstellen",this);
  wDateiname=new QLineEdit(this);
  wDateiname->setFixedSize(400,25);
  bDatei=new QPushButton("&Whlen",this);
  bVorbereiten=new QPushButton("&Vorbereiten",this);
  bInhalt=new QPushButton("&Inhaltsliste",this);
  lAnzOk=new QLabel(this);
  lAnzOk->setFixedSize(75,24);
  lAnzOk->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
  lAnzFehler=new QLabel(this);
  lAnzFehler->setFixedSize(75,24);
  lAnzFehler->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
  bFehler=new QPushButton("&Fehlerliste",this);
  bErstellen=new QPushButton("&Datei erstellen",this);
  if (dBudat.year()>1900)
  {
    wBudat=new QDateEdit(this);
    wBudat->setDate(dBudat);
    wBudat->setFixedSize(wBudat->sizeHint());
  }
  bOk=new QPushButton("&Ok",this);
  bCancel=new QPushButton("&Abbrechen",this);

  y=1;
  l=new QLabel("Eigener Name");
  layout->addWidget(l,y,1);
  layout->addWidget(wEName,y,2,1,2);
  y++;
  l=new QLabel("Konto, BLZ");
  layout->addWidget(l,y,1);
  layout->addWidget(wEKonto,y,2);
  layout->addWidget(wEBlz,y,3);
  y++;
  layout->addWidget(wCheckDup,y,1,1,3);
  y++;
  l=new QLabel("Dateiname");
  layout->addWidget(l,y,1);
  layout->addWidget(wDateiname,y,2,1,3);
  layout->addWidget(bDatei,y,5);
  y++;
  l=new QLabel("1. Schritt");
  layout->addWidget(l,y,1);
  layout->addWidget(bVorbereiten,y,2);
  y++;
  l=new QLabel("2. Schritt");
  layout->addWidget(l,y,1);
  layout->addWidget(bInhalt,y,2);
  layout->addWidget(lAnzOk,y,3);
  y++;
  l=new QLabel("3. Schritt");
  layout->addWidget(l,y,1);
  layout->addWidget(bFehler,y,2);
  layout->addWidget(lAnzFehler,y,3);
  y++;
  l=new QLabel("4. Schritt");
  layout->addWidget(l,y,1);
  layout->addWidget(bErstellen,y,2);
  if (wBudat!=NULL) layout->addWidget(wBudat,y,3);
  y++;
  layout->addWidget(bOk,y,1);
  layout->addWidget(bCancel,y,2);
  y++;

  layout->setColumnStretch(4,1);

  setLayout(layout);

  connect(bDatei,SIGNAL(clicked()),this,SLOT(pDateiWahl()));
  connect(bInhalt,SIGNAL(clicked()),this,SLOT(pInhalt()));
  connect(bFehler,SIGNAL(clicked()),this,SLOT(pFehlerliste()));
  connect(bErstellen,SIGNAL(clicked()),this,SLOT(pDateiErst()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pAbschluss()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(reject()));
  connect(bVorbereiten,SIGNAL(clicked()),this,SLOT(pTempTable()));
  connect(wCheckDup,SIGNAL(stateChanged(int)),this,SLOT(pDupWarn()));

  pReadParam();
  if      (wEName->text().length()<2) wEName->setFocus();
  else if (wEKonto->text().length()<2) wEKonto->setFocus();
  else if (wEBlz->text().length()!=8) wEBlz->setFocus();
  else if (wDateiname->text().length()<2) wDateiname->setFocus();
  else                                    bCancel->setFocus();

  pSetEnabled();
}

ZahlungDtaus::~ZahlungDtaus()
{
  QSqlQuery query;

  if (fTempTable) query.exec("DROP TABLE zwidtaus");
  if (wBudat!=NULL) delete wBudat;
  delete wEName;
  delete wEKonto;
  delete wEBlz;
  delete wDateiname;
  delete bDatei;
  delete wCheckDup;
  delete bVorbereiten;
  delete bInhalt;
  delete bFehler;
  delete bErstellen;
  delete bOk;
  delete bCancel;
  return;
}

void ZahlungDtaus::pSetEnabled()
{
  if (!fTempTable)
  {
    wEName->setEnabled(true);
    wEKonto->setEnabled(true);
    wEBlz->setEnabled(true);
    wCheckDup->setEnabled(true);
    bVorbereiten->setEnabled(true);
    bInhalt->setEnabled(false);
    bFehler->setEnabled(false);
    bErstellen->setEnabled(false);
    bOk->setEnabled(false);
  }
  else
  {
    wEName->setEnabled(false);
    wEKonto->setEnabled(false);
    wEBlz->setEnabled(false);
    wCheckDup->setEnabled(false);
    bVorbereiten->setEnabled(false);
    bInhalt->setEnabled(iOk>0);
    bFehler->setEnabled(iFehler>0);
    bErstellen->setEnabled(iOk>0 && fInhalt);
    if (wBudat!=NULL) wBudat->setEnabled(fErstellen);
    bOk->setEnabled(fErstellen);
  }
}

void ZahlungDtaus::pSaveParam()
{
  QSqlQuery query;

  query.exec("START TRANSACTION");
  query.exec("DELETE FROM parameter WHERE key>='dtaus/' AND key<'dtaus/zzz'");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('dtaus/name','" +
              sEName + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('dtaus/konto','" +
              sEKonto + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('dtaus/blz','" +
              sEBlz + "')");
  query.exec("INSERT INTO parameter (key,wert) VALUES ('dtaus/datei','" +
              sDateiname + "')");
  query.exec("COMMIT TRANSACTION");
}

void ZahlungDtaus::pReadParam()
{
  QSqlQuery query;

  query.exec("SELECT key,wert  FROM parameter WHERE key>='dtaus/' AND key<'dtaus/zzz'");
  while(query.next())
  {
    if (query.value(0).toString() == "dtaus/name")
       wEName->setText(query.value(1).toString());
    else if (query.value(0).toString() == "dtaus/konto")
       wEKonto->setText(query.value(1).toString());
    else if (query.value(0).toString() == "dtaus/blz")
       wEBlz->setText(query.value(1).toString());
    else if (query.value(0).toString() == "dtaus/datei")
       wDateiname->setText(query.value(1).toString());
  }
}

void ZahlungDtaus::pDupWarn()
{
  if (wCheckDup->checkState()==Qt::Checked)
  {
    QMessageBox::warning(this,"Zahlungen - Emails","Achtung: Wenn diese Option aktiviert ist, ist es nicht ausgeschlossen, dass Lastschriften doppelt eingezogen werden!");
  }
}

void ZahlungDtaus::pDateiWahl()
{
  QString sDatei;
  QFileDialog wDiag;

  wDiag.setDefaultSuffix("dta");
  wDiag.setNameFilters(QString("DTAUS-Datei (*.dta),Any file (*)").split(","));
  wDiag.selectNameFilter("*.dta");
  wDiag.setConfirmOverwrite(true);
  wDiag.setAcceptMode(QFileDialog::AcceptSave);
  wDiag.setFileMode(QFileDialog::AnyFile);
  if (wDiag.exec() == QDialog::Accepted)
  {
     sDatei=wDiag.selectedFiles()[0];
     if (sDatei!="") wDateiname->setText(sDatei);
  }
}
  

void ZahlungDtaus::pFelder()
{
  fDup=(wCheckDup->checkState() == Qt::Checked);
  sDateiname=wDateiname->text().simplified().replace("'","`");
  sEName=pText(wEName->text().simplified().toUpper().replace("'","`"));
  sEKonto=pNummer(wEKonto->text());
  sEBlz=pNummer(wEBlz->text());
  iEBlz=sEBlz.toLongLong();
  iEKonto=sEKonto.toLongLong();
}

QString ZahlungDtaus::pText(QString src)
{
  QString s,s1;
  int i;

  s1=src.toUpper();
  s="";
  for (i=0; i<s1.length(); i++)
  {
    if ((s1[i]>='A' && s1[i]<='Z') || (s1[i]>='0' && s1[i]<='9') ||
        s1[i]==' ' || s1[i]=='.' || s1[i]=='&' || s1[i]=='-' ||
        s1[i]=='+' || s1[i]=='*' || s1[i]=='%' || s1[i]=='/' ||
        s1[i]=='$')
       s=s+s1[i];
    else if (s1[i]=='') s=s+"AE";
    else if (s1[i]=='') s=s+"OE";
    else if (s1[i]=='') s=s+"UE";
    else if (s1[i]=='') s=s+"AE";
    else if (s1[i]=='') s=s+"OE";
    else if (s1[i]=='') s=s+"UE";
    else if (s1[i]=='') s=s+"SS";
    else                 s=s+" ";
  }

  return(s);
}

QString ZahlungDtaus::pNummer(QString src)
{
  QString s;
  int     i;

  s="";
  for (i=0; i<src.length(); i++)
  {
    if (src[i].isDigit()) s=s+src[i];
  }

  return(s);
}

void ZahlungDtaus::pTempTable()
{
  QSqlQuery query,query2;
  QString   s,sName,sBlz,sKonto,sBank,sInhaber;
  int       i;
  bool      flag_fehler;

  pFelder();

  if (fTempTable) query.exec("DROP TABLE zwidtaus");
  query.exec("CREATE TEMPORARY TABLE zwidtaus (mid int, name text, inhaber text, blz int64, konto int64, bank text, cent int64, ff char, PRIMARY KEY (blz ASC, konto ASC, mid ASC))");
  fTempTable=true;

  iOk=0;
  iFehler=0;
  iSumme=0;

  query.exec("SELECT mid,cent,done,sperren FROM zahllauf WHERE art='D'");
  while(query.next())
  {
    flag_fehler=false;
    if (iDtaus==0) iDtaus=1;
    if (query.value(3).toString()=="J") continue;
    if (query.value(2).toString()=="J" && !fDup) continue;
    s=s.sprintf("%08i",query.value(0).toInt());
    query2.exec("SELECT name1,name2 FROM adresse WHERE art='M' AND key='" + s + "'");
    if (!query2.next()) continue;
    sName = query2.value(0).toString();
    if (sName.length()>0 && query2.value(1).toString().length()>0)
       sName=sName + ", ";
    sName=sName + query2.value(1).toString();
    query2.finish();
    query2.exec("SELECT inhaber,nummer,blz,bank FROM bankverbindung WHERE art='M' AND key='" + s + "'");
    if (!query2.next())
    {
       flag_fehler=true;
       sKonto="Keine Bankverbindung eingetragen";
       sBlz=sBank=sInhaber="";
    }
    else
    {
      sBlz=pNummer(query2.value(2).toString());
      sKonto=pNummer(query2.value(1).toString());
      sBank=pText(query2.value(3).toString());
      sInhaber=pText(query2.value(0).toString());
      query2.finish();
      if (sBlz.length()!=8)    { flag_fehler=true; sKonto="Bankleitzahl zu kurz"; }
      if (sKonto.length()<2)   { flag_fehler=true; sKonto="Kontonummer fehlt"; }
      if (sInhaber.length()<2) { flag_fehler=true; sKonto="Kontoinhaber fehlt"; }
      query2.exec("SELECT name FROM blzdatei WHERE land='D' AND blz='" +
                  sBlz + "'");
      if (query2.next())
      {
         sBank=query.value(0).toString();
         query2.finish();
      }
      else
      {
        flag_fehler=true;
        sKonto="Bankleitzahl existiert nicht";
      }
    }
    if (flag_fehler)
       s="F";
    else
       s=" ";
    query2.exec("INSERT INTO zwidtaus (mid,name,inhaber,blz,konto,bank,cent,ff) VALUES ('" +
               query.value(0).toString() + "','" +
               sName                     + "','" +
               sInhaber                  + "','" +
               sBlz                      + "','" +
               sKonto                    + "','" +
               sBank                     + "','" +
               query.value(1).toString() + "','" + s + "')");
    if (query2.numRowsAffected()>0)
    {
      if (flag_fehler)
      {
        iFehler++;
      }
      else
      {
        iOk++;
        iSumme+=query.value(1).toLongLong();
      }
    }
  }
  fTempTable=true;
  pSetEnabled();
  lAnzOk->setText(QString::number(iOk));
  lAnzFehler->setText(QString::number(iFehler));
  QMessageBox::information(this,"Zahlungen Dtaus-Datei","Datei ist vorbereitet");
}

void ZahlungDtaus::pAddString(char *buf, int pos, QString src, int l)
{
  int i,l1;
  l1=src.length();
  if (l1>l) l1=l; else 
  for (i=0; i<l1; i++)
  {
    buf[i+pos]=src[i].toAscii();
  }
  for (; i<l; i++) buf[i+pos]=' ';
}

void ZahlungDtaus::pAddZahl(char *buf, int pos, qint64 src, int l)
{
  int i;
  qint64 n;

  for (n=src, i=l-1; i>=0; i--)
  {
    buf[i+pos]=(char)(n%(qint64)10+(qint64)'0');
    n=n/(qint64)10;
  }
}

QString ZahlungDtaus::pZweck(int zeile,int mid,int jahr)
{
  QString s;
  switch(zeile)
  {
    case  1: s=s.sprintf("MITGLIEDSNR. %i",mid); break;
    case  2: s=s.sprintf("JAHRESBEITRAG PP %04i",jahr); break;
    default: s="";
  }
  return(s);
}

void ZahlungDtaus::pDateiErst()
{
  QFile wDatei;
  QSqlQuery query;
  bool      flag_asatz;
  char      wBuf[512];
  QDate     wHeute=QDate::currentDate();
  qint64    iAnzC,iSumC,iSumCEur,iSumKtonr,iSumBlz;
  QString   sZweck1,sZweck2;

  if (sEName.length()<2) { wEName->setFocus(); return; }
  if (sEKonto.length()<2) { wEKonto->setFocus(); return; }
  if (sEBlz.length()!=8) { wEBlz->setFocus(); return; }
  if (sEName.length()>27) sEName=sEName.left(27);

  wDatei.setFileName(sDateiname);
  if (!wDatei.open(QIODevice::WriteOnly))
  {
    QMessageBox::critical(NULL,sDateiname,wDatei.errorString());
    return;
  }
  pSaveParam();

  iDtaus=0;
  flag_asatz=false;
  iAnzC=iSumC=iSumCEur=iSumKtonr=iSumBlz=0;

  query.exec("SELECT mid,inhaber,blz,konto,bank,cent FROM zwidtaus WHERE ff!='F' ORDER BY blz,konto,mid");
  while(query.next())
  {
    if (!flag_asatz)
    {
       memset(wBuf,' ',128);
       pAddString(wBuf,  0,"0128ALK",7);
       pAddZahl  (wBuf,  7,iEBlz,8);
       pAddZahl  (wBuf, 15,0,8);
       pAddString(wBuf, 23,sEName,27);
       pAddZahl  (wBuf, 50,wHeute.day(),2);
       pAddZahl  (wBuf, 52,wHeute.month(),2);
       pAddZahl  (wBuf, 54,wHeute.year() % 100,2);
       pAddString(wBuf, 56,"",4);
       pAddZahl  (wBuf, 60,iEKonto,10);
       pAddZahl  (wBuf, 70,0,10);
       pAddString(wBuf, 80,"",47);
       pAddString(wBuf,127,"1",1);
      wDatei.write(wBuf,128);
      flag_asatz=true;
    }

    sZweck1=pZweck(1,query.value(0).toInt(),wHeute.year());
    sZweck2=pZweck(2,query.value(0).toInt(),wHeute.year());

    memset(wBuf,' ',256);
    pAddString(wBuf,  0,"0216C",5);
    pAddZahl  (wBuf,  5,0,8);
    pAddZahl  (wBuf, 13,query.value(2).toLongLong(),8);
    pAddZahl  (wBuf, 21,query.value(3).toLongLong(),10);
    pAddZahl  (wBuf, 31,0,13);
    pAddString(wBuf, 44,"05000 ",6);
    pAddZahl  (wBuf, 50,0,11);
    pAddZahl  (wBuf, 61,iEBlz,8);
    pAddZahl  (wBuf, 69,iEKonto,10);
    pAddZahl  (wBuf, 79,query.value(5).toLongLong(),11);
    pAddString(wBuf, 90,"",3);
    pAddString(wBuf, 93,query.value(1).toString(),27);
    pAddString(wBuf,120,"",8);
    pAddString(wBuf,128,sEName,27);
    pAddString(wBuf,155,sZweck1,27);
    pAddString(wBuf,182,"1",1);
    pAddString(wBuf,183,"",2);
    pAddZahl  (wBuf,185,1,2);
    pAddString(wBuf,187,"02",2);
    pAddString(wBuf,189,sZweck2,27);
    wDatei.write(wBuf,256);
    iAnzC++;
    iSumKtonr += query.value(3).toLongLong();
    iSumBlz   += query.value(2).toLongLong();
    iSumCEur  += query.value(5).toLongLong();
  }
  if (flag_asatz)
  {
    memset(wBuf,' ',128);
    pAddString(wBuf,  0,"0128E",5);
    pAddString(wBuf,  5,"",5);
    pAddZahl  (wBuf, 10,iAnzC,7);
    pAddZahl  (wBuf, 17,iSumC,13);
    pAddZahl  (wBuf, 30,iSumKtonr,17);
    pAddZahl  (wBuf, 47,iSumBlz,17);
    pAddZahl  (wBuf, 64,iSumCEur,13);
    pAddString(wBuf, 77,"",51);
    wDatei.write(wBuf,128);
  }

  wDatei.close();
  fErstellen=true;
  pSetEnabled();
  QMessageBox::information(this,"Zahlungen Dtaus-Datei","Datei ist erstellt");
}

void ZahlungDtaus::pAbschluss()
{
  QSqlQuery query,query2;
  QString s;

  if (wBudat!=NULL) dBudat=wBudat->date();
  s="UPDATE zahllauf SET done='J'";
  if (dBudat.year()>1900 && wBudat!=NULL)
     s=s+",bausteinb='" + dBudat.toString("yyyy-MM-dd") + "'";
  query.exec("SELECT mid FROM zwidtaus WHERE ff!='F'");
  while(query.next())
  {
    query2.exec(s + " WHERE mid='" +
                query.value(0).toString() + "'");
  }
  query.exec("SELECT wert FROM parameter WHERE key='Status/Zahlungen'");
  if (query.next())
  {
    s="D" + query.value(0).toString().mid(1);
    query.finish();
    query.exec("UPDATE parameter SET wert='" + s +
               "' WHERE key='Status/Zahlungen'");
  }
  accept();
}

void ZahlungDtaus::pInhalt()
{
  QSqlQuery query;
  QFont     wFont,wFontBold;
  bool      flag_kopf;
  int       wSeite,wLfd;
  int       wPosY,wMaxY,wMaxX,wFontHeight,wFontWidth;
  QString   s,s1;
  QDate     wHeute=QDate::currentDate();

  QPrinter *wDrucker = new QPrinter(QPrinter::HighResolution);
  wDrucker->setOrientation(QPrinter::Landscape);
  QPrintDialog wDiag(wDrucker,this);
  if (wDiag.exec() != QDialog::Accepted) return;

  wDrucker=wDiag.printer();
  QPainter *wPainter = new QPainter();
  wPainter->begin(wDrucker);
  wFont.setFamily("Courier");
  wFont.setFixedPitch(true);
  wFont.setPointSize(10);
  wFontBold=wFont;
  wFontBold.setWeight(QFont::Bold);
  wPainter->setFont(wFont);

  QFontMetrics wMetrics=wPainter->fontMetrics();

  query.exec("SELECT mid,inhaber,blz,konto,bank,cent FROM zwidtaus WHERE ff!='F' ORDER BY blz,konto,mid");
  flag_kopf=true;
  wSeite=wLfd=wPosY=0;
  wMaxY=wDrucker->pageRect().height();
  wMaxX=wDrucker->pageRect().width();
  wFontHeight=wMetrics.height();
  wFontWidth=wMetrics.width("1234567890") / 10;
  while(query.next())
  {
    if ((wPosY+wFontHeight) > wMaxY ||
        (wLfd == (iOk - 1) && (wPosY + 3*wFontHeight) > wMaxY))
        flag_kopf=true;
    if (flag_kopf)
    {
      if (wSeite>0) wDrucker->newPage();
      flag_kopf=false;
      wSeite++;
      wPosY=wDrucker->resolution()*100/254 + wFontHeight;
      wPainter->setFont(wFontBold);
      wPainter->drawText(wFontWidth,wPosY,
                " DT - INHALTSLISTE  -  Lastschriften");
      wPainter->setFont(wFont);
      wPainter->drawText(46*wFontWidth,wPosY,"Kontonr. " +
                QString::number(iEKonto) + " (" + sEName + ")");
      s=s1.sprintf("%5i",wSeite);
      wPainter->drawText(98*wFontWidth,wPosY,"Erstellt: " +
                wHeute.toString("dd.MM.yyyy") + "  Seite:" + s);
      wPosY+=wFontHeight;
      wPosY+=wFontHeight;
      wPainter->drawText(wFontWidth,wPosY,"Lfd.");
      wPainter->drawText(8*wFontWidth,wPosY,"Betrag EUR");
      wPainter->drawText(21*wFontWidth,wPosY,"Kontonummer");
      wPainter->drawText(35*wFontWidth,wPosY,"Blz.");
      wPainter->drawText(46*wFontWidth,wPosY,"Zahlungspflichtiger");
      wPainter->drawText(75*wFontWidth,wPosY,"Verwendungszweck");
      wPosY+=wFontHeight;
      wPainter->drawLine(0,wPosY-wFontHeight/2,wMaxX,wPosY-wFontHeight/2);
      wPosY+=wFontHeight;
    }
    wLfd++;
    s=QString::number(wLfd);
    wPainter->drawText((5-s.length())*wFontWidth,wPosY,s);
    s=QLocale::system().toString((double)query.value(5).toLongLong()/(double)100.0l,'f',2);
    wPainter->drawText((18-s.length())*wFontWidth,wPosY,s);
    wPainter->drawText(21*wFontWidth,wPosY,query.value(3).toString());
    wPainter->drawText(35*wFontWidth,wPosY,query.value(2).toString());
    wPainter->drawText(46*wFontWidth,wPosY,query.value(1).toString());
    wPainter->drawText(75*wFontWidth,wPosY,
              pZweck(1,query.value(0).toInt(),wHeute.year()));
    wPainter->drawText(104*wFontWidth,wPosY,
              pZweck(2,query.value(0).toInt(),wHeute.year()));
    wPosY+=wFontHeight;
  }

  if (wSeite>0)
  {
    wPosY+=wFontHeight;
    wPainter->drawText(0,wPosY,"Summe:");
    s=QLocale::system().toString((double)iSumme/(double)100.0l,'f',2);
    wPainter->drawText((18-s.length())*wFontWidth,wPosY,s);
    wPainter->drawText(21*wFontWidth,wPosY,"Anzahl:");
    s=QString::number(iOk);
    wPainter->drawText((34-s.length())*wFontWidth,wPosY,s);
  }
  wPainter->end();
  delete wPainter;
  delete wDrucker;
  fInhalt=true;
  pSetEnabled();
  QMessageBox::information(this,"Zahlungen Dtaus-Datei","Inhaltsliste ist gedruckt");
}

void ZahlungDtaus::pFehlerliste()
{
  QSqlQuery query;
  QFont     wFont,wFontBold;
  bool      flag_kopf;
  int       wSeite,wLfd;
  int       wPosY,wMaxY,wMaxX,wFontHeight,wFontWidth;
  QString   s,s1;
  QDate     wHeute=QDate::currentDate();

  QPrinter *wDrucker = new QPrinter(QPrinter::HighResolution);
  wDrucker->setOrientation(QPrinter::Portrait);
  QPrintDialog wDiag(wDrucker,this);
  if (wDiag.exec() != QDialog::Accepted) return;

  wDrucker=wDiag.printer();
  QPainter *wPainter = new QPainter();
  wPainter->begin(wDrucker);
  wFont.setFamily("Courier");
  wFont.setFixedPitch(true);
  wFont.setPointSize(10);
  wFontBold=wFont;
  wFontBold.setWeight(QFont::Bold);
  wPainter->setFont(wFont);

  QFontMetrics wMetrics=wPainter->fontMetrics();

  query.exec("SELECT mid,konto FROM zwidtaus WHERE ff='F' ORDER BY mid");
  flag_kopf=true;
  wSeite=wLfd=wPosY=0;
  wMaxY=wDrucker->pageRect().height();
  wMaxX=wDrucker->pageRect().width();
  wFontHeight=wMetrics.height();
  wFontWidth=wMetrics.width("1234567890") / 10;
  while(query.next())
  {
    if ((wPosY+wFontHeight) > wMaxY) flag_kopf=true;
    if (flag_kopf)
    {
      if (wSeite>0) wDrucker->newPage();
      flag_kopf=false;
      wSeite++;
      wPosY=wFontHeight;
      wPainter->setFont(wFontBold);
      wPainter->drawText(5*wFontWidth,wPosY, " DT - Fehlerliste");
      wPainter->setFont(wFont);
      s=s1.sprintf("%5i",wSeite);
      wPainter->drawText(47*wFontWidth,wPosY,"Erstellt: " +
                wHeute.toString("dd.MM.yyyy") + "  Seite:" + s);
      wPosY+=wFontHeight;
      wPosY+=wFontHeight;
      wPainter->drawText(8*wFontWidth,wPosY,"MID    Fehler");
      wPosY+=wFontHeight;
      wPainter->drawLine(5*wFontWidth,wPosY-wFontHeight/2,wMaxX,
                         wPosY-wFontHeight/2);
      wPosY+=wFontHeight;
    }
    s=query.value(0).toString();
    wPainter->drawText((11-s.length())*wFontWidth,wPosY,s);
    wPainter->drawText(15*wFontWidth,wPosY,query.value(1).toString());
    wPosY+=wFontHeight;
  }
  wPainter->end();
  delete wPainter;
  delete wDrucker;
  QMessageBox::information(this,"Zahlungen Dtaus-Datei","Fehlerliste ist gedruckt");
}

void ZahlungDtaus::closeEvent(QCloseEvent *e)
{
  if (pTestAbbruch())
     e->accept();
  else
     e->ignore();
}

void ZahlungDtaus::reject()
{
  if (pTestAbbruch()) QDialog::reject();
}

bool ZahlungDtaus::pTestAbbruch()
{
  if (fErstellen)
  {
    if (QMessageBox::question(this,"Zahlungen Dtaus-Datei",
        "Erstellte Datei verwerfen",QMessageBox::Ok|QMessageBox::Cancel,
        QMessageBox::Cancel)!=QMessageBox::Ok)
       return(false);
  }
  return(true);
}
