#include <QtSql>
#include <QtGui>

#include "zahlmail.h"
#include "utils.h"
#include "smtp.h"

ZahlungMail::ZahlungMail(QWidget *parent) : QDialog(parent)
{
  QGridLayout *layout;
  QLabel      *l;
  int y;

  setWindowTitle("Zahlungen - Emails");
  layout=new QGridLayout();

  fLauf=fAbbruch=false;

  wCheckRech    = new QCheckBox("&Vorkasse",this);
  wCheckRechNeu = new QCheckBox("Vorkasse &Neumitglieder",this);
  wCheckLast    = new QCheckBox("&Lastschrifthinweis",this);
  wCheckLastNeu = new QCheckBox("Lastschrifthinweis N&eumitglieder",this);
  wCheckPlatz   = new QCheckBox("&Geplatzte Lastschrift",this);
  wCheckMahn1   = new QCheckBox("&1. Mahnung",this);
  wCheckMahn2   = new QCheckBox("&2. Mahnung",this);
  wCheckMahn3   = new QCheckBox("&3. Mahnung",this);
  wCheckDup     = new QCheckBox("&Bereits erstellte Mails neu erstellen",this);
  lMid          = new QLabel(this);
  bOk     = new QPushButton("&OK",this);
  bCancel = new QPushButton("&Abbruch",this);

  y=1;
  layout->addWidget(wCheckRech,y,1,1,3); y++;
  layout->addWidget(wCheckRechNeu,y,1,1,3); y++;
  layout->addWidget(wCheckLast,y,1,1,3); y++;
  layout->addWidget(wCheckLastNeu,y,1,1,3); y++;
  layout->addWidget(wCheckPlatz,y,1,1,3); y++;
  layout->addWidget(wCheckMahn1,y,1,1,3); y++;
  layout->addWidget(wCheckMahn2,y,1,1,3); y++;
  layout->addWidget(wCheckMahn3,y,1,1,3); y++;
  layout->addWidget(wCheckDup,y,1,1,3); y++;
  layout->addWidget(bOk,y,1);
  layout->addWidget(bCancel,y,2);
  layout->addWidget(lMid,y,3);
  // layout->setColumnStretch(3,1);

  wCheckRech->setCheckState(Qt::Checked);
  wCheckRechNeu->setCheckState(Qt::Checked);
  wCheckLast->setCheckState(Qt::Checked);
  wCheckLastNeu->setCheckState(Qt::Checked);
  wCheckPlatz->setCheckState(Qt::Checked);
  wCheckMahn1->setCheckState(Qt::Checked);
  wCheckMahn2->setCheckState(Qt::Checked);
  wCheckMahn3->setCheckState(Qt::Checked);
  wCheckDup->setCheckState(Qt::Unchecked);

  setLayout(layout);

  connect(bOk,SIGNAL(clicked()),this,SLOT(pLauf()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(reject()));
  connect(wCheckDup,SIGNAL(stateChanged(int)),this,SLOT(pDupWarn()));
}

ZahlungMail::~ZahlungMail()
{
  delete wCheckRech;
  delete wCheckRechNeu;
  delete wCheckLast;
  delete wCheckLastNeu;
  delete wCheckPlatz;
  delete wCheckMahn1;
  delete wCheckMahn2;
  delete wCheckMahn3;
  delete wCheckDup;
  delete bOk;
  delete bCancel;
  return;
}

void ZahlungMail::reject()
{
  if (fLauf)
      fAbbruch=true;
  else
      QDialog::reject();
}

void ZahlungMail::pDupWarn()
{
  if (wCheckDup->checkState()==Qt::Checked)
  {
    QMessageBox::warning(this,"Zahlungen - Emails","Achtung: Wenn diese Option aktiviert ist, ist es nicht ausgeschlossen, dass Emails doppelt versendet werden!");
  }
}

void ZahlungMail::closeEvent(QCloseEvent *e)
{
  if (fLauf) fAbbruch=true;
  else       e->accept();
}
void ZahlungMail::pFelder()
{
  fRech=fRechNeu=fLast=fLastNeu=fPlatz=fMahn1=fMahn2=fMahn3=fDup=false;
  if (wCheckRech->checkState()==Qt::Checked)    fRech=true;
  if (wCheckRechNeu->checkState()==Qt::Checked) fRechNeu=true;
  if (wCheckLast->checkState()==Qt::Checked)    fLast=true;
  if (wCheckLastNeu->checkState()==Qt::Checked) fLastNeu=true;
  if (wCheckPlatz->checkState()==Qt::Checked)   fPlatz=true;
  if (wCheckMahn1->checkState()==Qt::Checked)   fMahn1=true;
  if (wCheckMahn2->checkState()==Qt::Checked)   fMahn2=true;
  if (wCheckMahn3->checkState()==Qt::Checked)   fMahn3=true;
  if (wCheckDup->checkState()==Qt::Checked)     fDup=true;
}

void ZahlungMail::pSetLauf(bool flag_lauf)
{
  fLauf=flag_lauf;
  wCheckRech->setEnabled(!fLauf);
  wCheckRechNeu->setEnabled(!fLauf);
  wCheckLast->setEnabled(!fLauf);
  wCheckLastNeu->setEnabled(!fLauf);
  wCheckPlatz->setEnabled(!fLauf);
  wCheckMahn1->setEnabled(!fLauf);
  wCheckMahn2->setEnabled(!fLauf);
  wCheckMahn3->setEnabled(!fLauf);
  bOk->setEnabled(!fLauf);
}

void ZahlungMail::pLauf()
{
  QSqlQuery query,query2;
  QString   s,s1;
  Smtp      wSmtp;
  bool      flag_fehler;
  int       iFehler;
  int       iRech,iRechNeu,iLast,iLastNeu,iPlatz,iMahn1,iMahn2,iMahn3;

  pFelder();
  pSetLauf(true);
  fAbbruch=false;
  iFehler=0;
  iRech=iRechNeu=iLast=iLastNeu=iPlatz=iMahn1=iMahn2=iMahn3=0;

  query.exec("SELECT mid,art,bausteinm,cent,done,sperren FROM zahllauf WHERE form='M' ORDER BY mid");
  while(query.next() && !fAbbruch)
  {
    s=query.value(1).toString();

    if      (s=="L" && iLast   ==0) iLast   =1;
    else if (s=="l" && iLastNeu==0) iLastNeu=1;
    else if (s=="R" && iRech   ==0) iRech   =1;
    else if (s=="r" && iRechNeu==0) iRechNeu=1;
    else if (s=="G" && iPlatz  ==0) iPlatz  =1;
    else if (s=="1" && iMahn1  ==0) iMahn1  =1;
    else if (s=="2" && iMahn2  ==0) iMahn2  =1;
    else if (s=="3" && iMahn3  ==0) iMahn3  =1;

    if (query.value(5).toString()=="J") continue;
    if (query.value(4).toString()=="J" && !fDup) continue;

    if ((s=="L" && fLast)    ||
        (s=="l" && fLastNeu) ||
        (s=="R" && fRech)    ||
        (s=="r" && fRechNeu) ||
        (s=="G" && fPlatz)   ||
        (s=="1" && fMahn1)   ||
        (s=="2" && fMahn2)   ||
        (s=="3" && fMahn3)     )
    {
      flag_fehler=false;
      if (wSmtp.setText(query.value(2).toString(),"M",
          s.sprintf("%08i",query.value(0).toInt()),query.value(0).toInt(),
          QDate::currentDate().year(),query.value(3).toLongLong()))
      {
         lMid->setText(query.value(0).toString());
         if (wSmtp.sendMail())
         {
           query2.exec("UPDATE zahllauf SET done='J' WHERE mid='" +
                        query.value(0).toString() + "'");
         }
         else
           flag_fehler=true;
      }
      else
        flag_fehler=true;
      if (flag_fehler)
      {
        iFehler++;
        if (query.value(4).toString()!="J")
        {
          if      (s=="L") iLast   =2;
          else if (s=="l") iLastNeu=2;
          else if (s=="R") iRech   =2;
          else if (s=="r") iRechNeu=2;
          else if (s=="G") iPlatz  =2;
          else if (s=="1") iMahn1  =2;
          else if (s=="2") iMahn2  =2;
          else if (s=="3") iMahn3  =2;
        }
      }
    }
  }
  lMid->setText("");
  query.finish();
  pSetLauf(false);
  if (fAbbruch)
     QMessageBox::information(this,"Zahlungen Email","Der Lauf wurde abgebrochen");
  else if (iFehler>0)
     QMessageBox::information(this,"Zahlungen Email",QString::number(iFehler) + "Emails konnten nicht gesendet wewrden");
  else
  {
     query.exec("SELECT wert FROM parameter WHERE key='Status/Zahlungen'");
     if (query.next())
     {
       s=query.value(0).toString();
       query.finish();
       if (iRech   ==1 && s.length()> 1) s=s.mid(0, 1) + "D" + s.mid(2);
       if (iRechNeu==1 && s.length()> 3) s=s.mid(0, 3) + "D" + s.mid(4);
       if (iLast   ==1 && s.length()> 5) s=s.mid(0, 5) + "D" + s.mid(6);
       if (iLastNeu==1 && s.length()> 7) s=s.mid(0, 7) + "D" + s.mid(8);
       if (iPlatz  ==1 && s.length()> 9) s=s.mid(0, 9) + "D" + s.mid(10);
       if (iMahn1  ==1 && s.length()>11) s=s.mid(0,11) + "D" + s.mid(12);
       if (iMahn2  ==1 && s.length()>13) s=s.mid(0,13) + "D" + s.mid(14);
       if (iMahn3  ==1 && s.length()>15) s=s.mid(0,15) + "D" + s.mid(16);
       query.exec("UPDATE parameter SET wert='" + s +
                  "' WHERE key='Status/Zahlungen'");
     }
     QMessageBox::information(this,"Zahlungen Email","Fertig");
  }
  accept();
}
