#include "zahlungen.h"
#include "utils.h"
#include "wkonto.h"

Zahlungen::Zahlungen(QWidget *parent) : QDialog(parent)
{
  QLabel *l;
  int    i,z;
  QVBoxLayout *topLayout = new QVBoxLayout;
  QGridLayout *layout = new QGridLayout;
  QHBoxLayout *buttonLayout = new QHBoxLayout;

  setWindowTitle("Bank/Kasse");

  lKontoText = new QLabel("",this);
  lGKontoText = new QLabel("",this);
  wKonto  = new WKonto(lKontoText,true,this);
  wDatum  = new QDateEdit(QDate::currentDate(),this);
  wDatum->setDisplayFormat("dd.MM.yyyy");
  wDatum->setFixedSize(wDatum->sizeHint());
  wBetrag = new QDoubleSpinBox(this);;
  wBetrag->setDecimals(2);
  wBetrag->setMinimum(-99999999.99);
  wBetrag->setMaximum( 99999999.99);
  wBetrag->setSuffix(" EUR");
  wBetrag->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
  wBetrag->setFixedSize(150,25);
  bSpende = new QPushButton("S&penden",this);
  bSpende->setFixedSize(bSpende->sizeHint());
  bSpende->setAutoDefault(false);
  lSpenderName = new QLabel("",this);
  wText   = new QLineEdit(this);
  wText->setFixedSize(850,25);
  wKost   = new QLineEdit(this);
  wKost->setFixedSize(200,25);
  wBeschluss= new QLineEdit(this);
  wBeschluss->setFixedSize(200,25);
  wBeleg  = new QLineEdit(this);
  wBeleg->setFixedSize(100,25);
  wGKonto = new WKonto(lGKontoText,true,this);

  for (i=0; i<ZAHLUNGEN_MAX; i++)
  {
    lAuftKonto[i]=new QLabel("",this);
    wAuftBetrag[i]=new QDoubleSpinBox(this);
    wAuftKost[i]=new QLineEdit(this);
    wAuftKonto[i]=new WKonto(lAuftKonto[i],true,this);
    wAuftBetrag[i]->setDecimals(2);
    wAuftBetrag[i]->setMinimum(-99999999.99);
    wAuftBetrag[i]->setMaximum( 99999999.99);
    wAuftBetrag[i]->setSuffix(" EUR");
    wAuftBetrag[i]->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
    wAuftBetrag[i]->setFixedSize(150,25);
    wAuftKost[i]->setFixedSize(200,25);
  }

  bOk = new QPushButton("&Speichern",this); bOk->setAutoDefault(false);
  bCancel = new QPushButton("&Abbruch",this); bCancel->setAutoDefault(false);

  z=1;
  l=new QLabel("Konto",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wKonto,z,2);
  layout->addWidget(lKontoText,z,3,1,5);
  z++;

  l=new QLabel("Datum",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wDatum,z,2);
  z++;

  l=new QLabel("Betrag",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wBetrag,z,2);
  layout->addWidget(bSpende,z,3);
  layout->addWidget(lSpenderName,z,4,1,4);
  z++;

  l=new QLabel("Text",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wText,z,2,1,6);
  z++;
  layout->setColumnStretch(7,1);

  l=new QLabel("Kostenstelle",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wKost,z,2);
  l=new QLabel("Beschluss",this);
  layout->addWidget(l,z,3);
  layout->addWidget(wBeschluss,z,4);
  l=new QLabel("Beleg",this);
  layout->addWidget(l,z,5);
  layout->addWidget(wBeleg,z,6);
  z++;

  l=new QLabel("Gegenkonto:",this);
  layout->addWidget(l,z,1);
  layout->addWidget(wGKonto,z,2);
  layout->addWidget(lGKontoText,z,3,1,5);
  z++;

  l=new QLabel("Aufteilung",this);
  layout->addWidget(l,z,1);
  l=new QLabel("Kostenstelle",this);
  layout->addWidget(l,z,2);
  l=new QLabel("Konto",this);
  layout->addWidget(l,z,3);
  z++;
  for (i=0; i<ZAHLUNGEN_MAX; i++)
  {
    layout->addWidget(wAuftBetrag[i],z,1);
    layout->addWidget(wAuftKost[i],z,2);
    layout->addWidget(wAuftKonto[i],z,3);
    layout->addWidget(lAuftKonto[i],z,4,1,4);
    z++;
  }

  buttonLayout->addWidget(bOk);
  buttonLayout->addWidget(bCancel);
  buttonLayout->addStretch(1);

  topLayout->addLayout(layout);
  topLayout->addLayout(buttonLayout);
  setLayout(topLayout);

  connect(bSpende,SIGNAL(clicked()),this,SLOT(pSpende()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(reject()));
  connect(bOk,SIGNAL(clicked()),this,SLOT(pWrite()));
  return;
}

Zahlungen::~Zahlungen()
{
  return;
}

void Zahlungen::pSpende()
{
  int i,r;
  bool   flag_bar;
  qint64 wTotal,wTeilBetrag;
  QString wTeilKonto,wTeilKost;
  QString sKto;

  sKto=wKonto->text().simplified().toUpper().replace("'","`");
  if (sKto>="1000" && sKto<="1099")
     flag_bar=true;
  else
     flag_bar=false;
  r=wSpende.exec(round(wBetrag->value()*100.0l));
  if (r!=QDialog::Accepted) return;
  lSpenderName->setText(wSpende.disName());
  wTotal=round(wBetrag->value() * 100.0l);
  for (i=0; i<ZAHLUNGEN_MAX; i++)
  {
    wSpende.aufteilung(flag_bar,i,wTotal,&wTeilBetrag,&wTeilKonto,&wTeilKost);
    wAuftKonto[i]->setText(wTeilKonto);
    wAuftKost[i]->setText(wTeilKost);
    wAuftBetrag[i]->setValue((double)wTeilBetrag/(double)100.l);
  }
  wSpende.aufteilung(flag_bar,-1,wTotal,&wTeilBetrag,&wTeilKonto,&wTeilKost);
  wGKonto->setText(wTeilKonto);
  wKost->setText(wTeilKost);
  wSpende.aufteilung(flag_bar,-2,wTotal,&wTeilBetrag,&wTeilKonto,&wTeilKost);
  wText->setText(wTeilKost);
}

void Zahlungen::pWrite()
{
  int     wBuLfd,i;
  qint64  iCent,iTeilCent;
  QString sTeilKost,sTeilKonto;
  QString sKonto=wKonto->text().simplified().toUpper().replace("'","`");
  QString sGKonto=wGKonto->text().simplified().toUpper().replace("'","`");
  QString sText=wText->text().simplified().replace("'","`");
  QString sKost=wKost->text().simplified().replace("'","`");
  QString sBeschluss=wBeschluss->text().simplified().toUpper().replace("'","`");
  QString sBeleg=wBeleg->text().simplified().toUpper().replace("'","`");
  iCent=round(wBetrag->value()*(double)100.0l);

  if (sKonto.length()<2)  { wKonto->setFocus(); return; }
  if (iCent==0)           { wBetrag->setFocus(); return; }
  if (sText.length()<1)   { wText->setFocus(); return; }
  if (sKost.length()<1)   { wKost->setFocus(); return; }
  if (sGKonto.length()<2) { wGKonto->setFocus(); return; }

  for (i=0; i<ZAHLUNGEN_MAX; i++)
  {
    iTeilCent=round(wAuftBetrag[i]->value()*(double)100.0l);
    if (iTeilCent!=0)
    {
      if (wAuftKonto[i]->text().simplified().length()<2)
      {
        wAuftKonto[i]->setFocus();
        return;
      }
      if (wAuftKost[i]->text().simplified().length()<1)
         wAuftKost[i]->setText(sKost);
    }
  }

  wBuLfd=UtilBuch(wDatum->date(),0,sKost,sBeleg,sText,iCent,sKonto,sGKonto,
                  sBeschluss,"");
  if (wBuLfd<1)
  {
    QMessageBox::critical(this,"Journal","Fehler beim Generieren der Nummer");
    return;
  }
  wSpende.write(wDatum->date(),wBuLfd);

  for (i=0; i<ZAHLUNGEN_MAX; i++)
  {
    iTeilCent=round(wAuftBetrag[i]->value()*(double)100.0l);
    if (iTeilCent!=0)
    {
      sTeilKonto=wAuftKonto[i]->text().simplified().toUpper().replace("'","`");
      sTeilKost=wAuftKost[i]->text().simplified().toUpper().replace("'","`");
      wBuLfd=UtilBuch(wDatum->date(),0,sTeilKost,"",sText,iTeilCent,
                     sGKonto,sTeilKonto,"","");
    }
  }

  wBetrag->setValue(0.0l);
  wText->setText("");
  wKost->setText("");
  wBeschluss->setText("");
  wBeleg->setText("");
  wGKonto->setText("");
  for (i=0; i<ZAHLUNGEN_MAX; i++)
  {
    wAuftBetrag[i]->setValue(0.0l);
    wAuftKost[i]->setText("");
    wAuftKonto[i]->setText("");
  }
  wSpende.clear();
  lSpenderName->setText("");
  wDatum->setFocus();
}
