#include <QtSql>

#include "zuwenddruck.h"
#include "utils.h"
#include "druckfkt.h"

Zuwenddruck::Zuwenddruck(int mid, QWidget *parent) : QDialog(parent)
{
  QLabel      *l;
  int         y;

  iMid=mid;
  fLauf=fAbbruch=false;

  QVBoxLayout *topLayout    = new QVBoxLayout();
  QGridLayout *layout       = new QGridLayout();
  QHBoxLayout *buttonLayout = new QHBoxLayout();

  setWindowTitle("Zuwendungsbesttigungen");

  topLayout->addLayout(layout);
  topLayout->addLayout(buttonLayout);

  wDoppel=new QComboBox(this);
  wDoppel->insertItem(0,"nur noch nicht gedruckte");
  wDoppel->insertItem(1,"nur bereits gedruckte");
  wDoppel->insertItem(2,"alle");
  wDoppel->setFixedSize(wDoppel->sizeHint());
  wVonDat=new QDateEdit(this);
  wVonDat->setDisplayFormat("dd.MM.yyyy");
  wVonDat->setFixedSize(wVonDat->sizeHint());
  wVonDat->setDate(UtilGJVon().addYears(-1));
  wBisDat=new QDateEdit(this);
  wBisDat->setDisplayFormat("dd.MM.yyyy");
  wBisDat->setFixedSize(wVonDat->sizeHint());
  if (iMid>0)
     wBisDat->setDate(QDate::currentDate());
  else
     wBisDat->setDate(UtilGJVon().addDays(-1));
  wDruckVon=new QDateEdit(this);
  wDruckVon->setDisplayFormat("dd.MM.yyyy");
  wDruckVon->setFixedSize(wDruckVon->sizeHint());
  wDruckVon->setDate(QDate(2000,1,1));
  wDruckBis=new QDateEdit(this);
  wDruckBis->setDisplayFormat("dd.MM.yyyy");
  wDruckBis->setFixedSize(wDruckBis->sizeHint());
  wDruckBis->setDate(QDate::currentDate());
  bDrucken=new QPushButton("&Drucken",this); bDrucken->setAutoDefault(false);
  bCancel =new QPushButton("&Abbruch",this); bCancel->setAutoDefault(false);
  lDisp   = new QLabel;
  lDisp->setFixedSize(100,25);

  y=1;

  if (iMid>0)
  {
    l=new QLabel(UtilKtoText(UtilMIDKonto(iMid)));
    layout->addWidget(l,y,1,1,3);
    y++;
  }
  l=new QLabel("Auswahl");
  layout->addWidget(l,y,1);
  layout->addWidget(wDoppel,y,2,1,2);
  y++;
  l=new QLabel("Datum von/bis");
  layout->addWidget(l,y,1);
  layout->addWidget(wVonDat,y,2);
  layout->addWidget(wBisDat,y,3);
  y++;
  l=new QLabel("Druck von/bis");
  layout->addWidget(l,y,1);
  layout->addWidget(wDruckVon,y,2);
  layout->addWidget(wDruckBis,y,3);

  buttonLayout->addWidget(bDrucken);
  buttonLayout->addWidget(bCancel);
  buttonLayout->addStretch(1);
  buttonLayout->addWidget(lDisp);

  setLayout(topLayout);
  pDoppelChg();

  connect(bDrucken,SIGNAL(clicked()),this,SLOT(pDrucken()));
  connect(bCancel,SIGNAL(clicked()),this,SLOT(reject()));
  connect(wDoppel,SIGNAL(currentIndexChanged(int)),this,SLOT(pDoppelChg()));
}

Zuwenddruck::~Zuwenddruck()
{
  delete wDoppel;
  delete wVonDat;
  delete wBisDat;
  delete wDruckVon;
  delete wDruckBis;
  delete bDrucken;
  delete bCancel;
  delete lDisp;
}

void Zuwenddruck::reject()
{
  if (fLauf)
     fAbbruch=true;
  else
     QDialog::reject();
}

void Zuwenddruck::closeEvent(QCloseEvent *e)
{
  if (fLauf) fAbbruch=true;
  else       e->accept();
}

void Zuwenddruck::pSetLauf(bool flag_lauf)
{
  fLauf=flag_lauf;
  wDoppel->setEnabled(!fLauf);
  wVonDat->setEnabled(!fLauf);
  wBisDat->setEnabled(!fLauf);
  wDruckVon->setEnabled(!fLauf);
  wDruckBis->setEnabled(!fLauf);
  bDrucken->setEnabled(!fLauf);
}

void Zuwenddruck::pDoppelChg()
{
  bool f=(wDoppel->currentIndex()>0);
  wDruckVon->setEnabled(f);
  wDruckBis->setEnabled(f);
}

void Zuwenddruck::pDrucken()
{
  QSqlQuery query;
  QString   s;
  int       i,wAnzZuwend,wAnzSeiten,z1;
  int       iDoppel;
  bool      flag_anfang;
  QPrinter  *wDrucker;
  QPainter  *wPainter;

  fAbbruch=false;
  pSetLauf(true);
  iDoppel=wDoppel->currentIndex();
  s="";
  if (iMid>0) s=s+" mid='" + QString::number(iMid) + "' AND";
  s=s+" jdatum>='" + wVonDat->date().toString("yyyy-MM-dd") +
     "' AND jdatum<='" + wBisDat->date().toString("yyyy-MM-dd") + "'";
  if (iDoppel==0)
     s=s+" AND quittung<'2000-01-01'";
  else if (iDoppel==1)
     s=s+" AND quittung>='" + wDruckVon->date().toString("yyyy-MM-dd") +
        "' AND quittung<='" + wDruckBis->date().toString("yyyy-MM-dd") + "'";
  s=s+" AND art!='S'";
  query.exec("SELECT DISTINCT mid FROM spende WHERE" + s + " ORDER BY mid");
  flag_anfang=true;
  wAnzSeiten=0;
  wDrucker=NULL;
  wPainter=NULL;
  z1=0;
  while(query.next() && !fAbbruch)
  {
    if (flag_anfang) ZuwendZwidatStart();
    wAnzZuwend=PrepareZuwend(query.value(0).toInt(),iDoppel,wVonDat->date(),
               wBisDat->date(),wDruckVon->date(),wDruckBis->date(),(iMid<1));
    if (flag_anfang && wAnzZuwend>0)
    {
      wDrucker=new QPrinter(QPrinter::HighResolution);
      QPrintDialog wDiag(wDrucker,this);
      if (wDiag.exec() != QDialog::Accepted) break;
      wPainter=new QPainter();
      flag_anfang=false;
    }
    for (i=0; i<wAnzZuwend; i++)
    {
      if (DruckZuwend(wDrucker,wPainter,i,(wAnzSeiten==0))) wAnzSeiten++;
      z1++;
      if (z1>5)
      {
        QCoreApplication::processEvents();
        z1=0;
        lDisp->setText(query.value(0).toString());
      }
    }
  }
  query.finish();
  if (wAnzSeiten>0) wPainter->end();
  if (wPainter!=NULL) delete wPainter;
  if (wDrucker!=NULL) delete wDrucker;
  pSetLauf(false);
  lDisp->setText("");
  if (wAnzSeiten>0)
  {
    if (QMessageBox::question(this,"Zuwendungsbest#tigungen",
           QString::number(wAnzSeiten) +
           " Seiten gedruckt (abgebrochen).\nDruck in Stammdaten vermerken?",
           QMessageBox::Yes|QMessageBox::No,QMessageBox::No) ==
        QMessageBox::Yes)
    {
      ZuwendZwidatEnd(true);
    }
  }
  ZuwendZwidatEnd(false);
  accept();
}
